@@ -34,7 +34,7 @@ namespace Goals
 		{
 			tile = cluster->blocker->visitablePos();
 			hero = pathToCenter.targetHero;
-			objid = cluster->blocker->id;
+			objid = cluster->blocker->id.getNum();
 		}
 
 		bool operator==(const UnlockCluster & other) const override;
