@@ -13,38 +13,45 @@
 
 #include "AdventureMapInterface.h"
 
-#include "../widgets/Images.h"
-#include "../CGameInfo.h"
 #include "../CPlayerInterface.h"
-#include "../gui/CGuiHandler.h"
+#include "../GameEngine.h"
+#include "../GameInstance.h"
 #include "../gui/MouseButton.h"
 #include "../gui/WindowHandler.h"
-#include "../render/Colors.h"
+#include "../render/CAnimation.h"
 #include "../render/Canvas.h"
+#include "../render/Colors.h"
 #include "../render/Graphics.h"
+#include "../render/IRenderHandler.h"
+#include "../widgets/Images.h"
 #include "../windows/InfoWindows.h"
 
-#include "../../CCallback.h"
-#include "../../lib/texts/CGeneralTextHandler.h"
+#include "../../lib/CConfigHandler.h"
 #include "../../lib/TerrainHandler.h"
+#include "../../lib/callback/CCallback.h"
 #include "../../lib/mapObjects/CGHeroInstance.h"
-#include "../../lib/mapping/CMapDefines.h"
+#include "../../lib/mapping/TerrainTile.h"
+#include "../../lib/texts/CGeneralTextHandler.h"
 
 ColorRGBA CMinimapInstance::getTileColor(const int3 & pos) const
 {
-	const TerrainTile * tile = LOCPLINT->cb->getTile(pos, false);
+	const TerrainTile * tile = GAME->interface()->cb->getTile(pos, false);
 
 	// if tile is not visible it will be black on minimap
 	if(!tile)
 		return Colors::BLACK;
 
 	// if object at tile is owned - it will be colored as its owner
-	for (const CGObjectInstance *obj : tile->blockingObjects)
+	for (const ObjectInstanceID objectID : tile->blockingObjects)
 	{
+		const auto * obj = GAME->interface()->cb->getObj(objectID);
 		PlayerColor player = obj->getOwner();
 		if(player == PlayerColor::NEUTRAL)
 			return graphics->neutralColor;
 
+		if (settings["adventure"]["minimapShowHeroes"].Bool() && obj->ID == MapObjectID::HERO)
+			continue;
+
 		if (player.isValidPlayer())
 			return graphics->playerColors[player.getNum()];
 	}
@@ -63,7 +70,7 @@ void CMinimapInstance::refreshTile(const int3 &tile)
 
 void CMinimapInstance::redrawMinimap()
 {
-	int3 mapSizes = LOCPLINT->cb->getMapSize();
+	int3 mapSizes = GAME->interface()->cb->getMapSize();
 
 	for (int y = 0; y < mapSizes.y; ++y)
 		for (int x = 0; x < mapSizes.x; ++x)
@@ -71,7 +78,7 @@ void CMinimapInstance::redrawMinimap()
 }
 
 CMinimapInstance::CMinimapInstance(const Point & position, const Point & dimensions, int Level):
-	minimap(new Canvas(Point(LOCPLINT->cb->getMapSize().x, LOCPLINT->cb->getMapSize().y), CanvasScalingPolicy::IGNORE)),
+	minimap(new Canvas(Point(GAME->interface()->cb->getMapSize().x, GAME->interface()->cb->getMapSize().y), CanvasScalingPolicy::IGNORE)),
 	level(Level)
 {
 	pos += position;
@@ -88,15 +95,16 @@ void CMinimapInstance::showAll(Canvas & to)
 }
 
 CMinimap::CMinimap(const Rect & position)
-	: CIntObject(LCLICK | SHOW_POPUP | DRAG | MOVE | GESTURE, position.topLeft()),
-	level(0)
+	: CIntObject(LCLICK | SHOW_POPUP | DRAG | MOVE | GESTURE, position.topLeft())
+	, heroIcon(ENGINE->renderHandler().loadImage(ImagePath::builtin("minimapIcons/hero"), EImageBlitMode::WITH_SHADOW_AND_FLAG_COLOR))
+	, level(0)
 {
 	OBJECT_CONSTRUCTION;
 
-	double maxSideLengthSrc = std::max(LOCPLINT->cb->getMapSize().x, LOCPLINT->cb->getMapSize().y);
+	double maxSideLengthSrc = std::max(GAME->interface()->cb->getMapSize().x, GAME->interface()->cb->getMapSize().y);
 	double maxSideLengthDst = std::max(position.w, position.h);
 	double resize = maxSideLengthSrc / maxSideLengthDst;
-	Point newMinimapSize = Point(LOCPLINT->cb->getMapSize().x/ resize, LOCPLINT->cb->getMapSize().y / resize);
+	Point newMinimapSize(GAME->interface()->cb->getMapSize().x/ resize, GAME->interface()->cb->getMapSize().y / resize);
 	Point offset = Point((std::max(newMinimapSize.x, newMinimapSize.y) - newMinimapSize.x) / 2, (std::max(newMinimapSize.x, newMinimapSize.y) - newMinimapSize.y) / 2);
 
 	pos.x += offset.x;
@@ -114,7 +122,7 @@ int3 CMinimap::pixelToTile(const Point & cursorPos) const
 	double dx = static_cast<double>(cursorPos.x) / pos.w;
 	double dy = static_cast<double>(cursorPos.y) / pos.h;
 
-	int3 mapSizes = LOCPLINT->cb->getMapSize();
+	int3 mapSizes = GAME->interface()->cb->getMapSize();
 
 	int tileX(std::round(mapSizes.x * dx));
 	int tileY(std::round(mapSizes.y * dy));
@@ -124,13 +132,13 @@ int3 CMinimap::pixelToTile(const Point & cursorPos) const
 
 Point CMinimap::tileToPixels(const int3 &tile) const
 {
-	int3 mapSizes = LOCPLINT->cb->getMapSize();
+	int3 mapSizes = GAME->interface()->cb->getMapSize();
 
 	double stepX = static_cast<double>(pos.w) / mapSizes.x;
 	double stepY = static_cast<double>(pos.h) / mapSizes.y;
 
-	int x = static_cast<int>(stepX * tile.x);
-	int y = static_cast<int>(stepY * tile.y);
+	int x = static_cast<int>(stepX * (tile.x + 0.5));
+	int y = static_cast<int>(stepY * (tile.y + 0.5));
 
 	return Point(x,y);
 }
@@ -141,7 +149,7 @@ void CMinimap::moveAdvMapSelection(const Point & positionGlobal)
 	adventureInt->centerOnTile(newLocation);
 
 	if (!(adventureInt->isActive()))
-		GH.windows().totalRedraw(); //redraw this as well as inactive adventure map
+		ENGINE->windows().totalRedraw(); //redraw this as well as inactive adventure map
 	else
 		redraw();//redraw only this
 }
@@ -159,15 +167,15 @@ void CMinimap::clickPressed(const Point & cursorPosition)
 
 void CMinimap::showPopupWindow(const Point & cursorPosition)
 {
-	CRClickPopup::createAndPush(CGI->generaltexth->zelp[291].second);
+	CRClickPopup::createAndPush(LIBRARY->generaltexth->zelp[291].second);
 }
 
 void CMinimap::hover(bool on)
 {
 	if(on)
-		GH.statusbar()->write(CGI->generaltexth->zelp[291].first);
+		ENGINE->statusbar()->write(LIBRARY->generaltexth->zelp[291].first);
 	else
-		GH.statusbar()->clear();
+		ENGINE->statusbar()->clear();
 }
 
 void CMinimap::mouseDragged(const Point & cursorPosition, const Point & lastUpdateDistance)
@@ -182,18 +190,34 @@ void CMinimap::showAll(Canvas & to)
 
 	if(minimap)
 	{
-		int3 mapSizes = LOCPLINT->cb->getMapSize();
+		int3 mapSizes = GAME->interface()->cb->getMapSize();
+
+		Canvas clippedTarget(to, pos);
+
+		if (settings["adventure"]["minimapShowHeroes"].Bool())
+		{
+			const auto & visibleHeroes = getVisibleHeroes();
+
+			for (const auto objectID : visibleHeroes)
+			{
+				const auto * object = GAME->interface()->cb->getObj(objectID);
+
+				if (object->anchorPos().z != level)
+					continue;
+
+				heroIcon->setOverlayColor(graphics->playerColors[object->getOwner().getNum()]);
+				clippedTarget.draw(heroIcon, tileToPixels(object->visitablePos()) - heroIcon->dimensions() / 2);
+			}
+		}
 
 		//draw radar
 		Rect radar =
-		{
-			screenArea.x * pos.w / mapSizes.x,
-			screenArea.y * pos.h / mapSizes.y,
-			screenArea.w * pos.w / mapSizes.x - 1,
-			screenArea.h * pos.h / mapSizes.y - 1
-		};
-
-		Canvas clippedTarget(to, pos);
+			{
+				screenArea.x * pos.w / mapSizes.x,
+				screenArea.y * pos.h / mapSizes.y,
+				screenArea.w * pos.w / mapSizes.x - 1,
+				screenArea.h * pos.h / mapSizes.y - 1
+			};
 		clippedTarget.drawBorderDashed(radar, Colors::PURPLE);
 	}
 }
@@ -221,7 +245,10 @@ void CMinimap::onMapViewMoved(const Rect & visibleArea, int mapLevel)
 		update();
 	}
 	else
+	{
+		setRedrawParent(true); // needed for non square map to redraw black background when viewarea rectangle is moved
 		redraw();
+	}
 }
 
 void CMinimap::setAIRadar(bool on)
@@ -239,12 +266,29 @@ void CMinimap::setAIRadar(bool on)
 	redraw();
 }
 
-void CMinimap::updateTiles(const std::unordered_set<int3> & positions)
+std::vector<ObjectInstanceID> CMinimap::getVisibleHeroes()
+{
+	std::vector<ObjectInstanceID> visibleHeroes;
+
+	for (const auto & player : PlayerColor::ALL_PLAYERS())
+	{
+		if (GAME->interface()->cb->getPlayerStatus(player, false) != EPlayerStatus::INGAME)
+			continue;
+
+		for (const auto & hero : GAME->interface()->cb->getHeroes(player))
+			visibleHeroes.push_back(hero->id);
+	}
+
+	return visibleHeroes;
+}
+
+void CMinimap::updateTiles(const FowTilesType & positions)
 {
 	if(minimap)
 	{
 		for (auto const & tile : positions)
 			minimap->refreshTile(tile);
 	}
+
 	redraw();
 }
