@@ -13,10 +13,12 @@
 
 VCMI_LIB_NAMESPACE_BEGIN
 class ColorRGBA;
+using FowTilesType = std::set<int3>;
 VCMI_LIB_NAMESPACE_END
 
 class Canvas;
 class CMinimap;
+class IImage;
 
 class CMinimapInstance : public CIntObject
 {
@@ -40,6 +42,8 @@ class CMinimap : public CIntObject
 {
 	std::shared_ptr<CPicture> aiShield; //the graphic displayed during AI turn
 	std::shared_ptr<CMinimapInstance> minimap;
+	std::shared_ptr<IImage> heroIcon;
+
 	Rect screenArea;
 	int level;
 
@@ -48,6 +52,7 @@ class CMinimap : public CIntObject
 	void showPopupWindow(const Point & cursorPosition) override;
 	void hover(bool on) override;
 	void mouseDragged(const Point & cursorPosition, const Point & lastUpdateDistance) override;
+	std::vector<ObjectInstanceID> getVisibleHeroes();
 
 	/// relocates center of adventure map screen to currently hovered tile
 	void moveAdvMapSelection(const Point & positionGlobal);
@@ -68,6 +73,6 @@ public:
 
 	void showAll(Canvas & to) override;
 
-	void updateTiles(const std::unordered_set<int3> & positions);
+	void updateTiles(const FowTilesType & positions);
 };
 
