@@ -24,6 +24,9 @@ class InputSourceMouse
 	Point middleClickPosition;
 	int mouseButtonsMask = 0;
 	const int mouseToleranceDistance;
+
+	float motionAccumulatedX;
+	float motionAccumulatedY;
 public:
 	InputSourceMouse();
 
