@@ -28,6 +28,7 @@ class CTextBox;
 class CButton;
 class CSlider;
 class LRClickableArea;
+class CTextInputWithConfirm;
 
 class FilledTexturePlayerColored;
 class TransparentFilledRectangle;
@@ -196,7 +197,7 @@ private:
 		std::unique_ptr<PlayerInfo> pi;
 		std::unique_ptr<PlayerSettings> s;
 		std::shared_ptr<CLabel> labelPlayerName;
-		std::shared_ptr<CTextInput> labelPlayerNameEdit;
+		std::shared_ptr<CTextInputWithConfirm> labelPlayerNameEdit;
 		std::shared_ptr<CMultiLineLabel> labelWhoCanPlay;
 		std::shared_ptr<CPicture> background;
 		std::shared_ptr<CButton> buttonTownLeft;
@@ -215,10 +216,6 @@ private:
 
 		PlayerOptionsEntry(const PlayerSettings & S, const OptionsTab & parentTab);
 		void hideUnavailableButtons();
-		bool captureThisKey(EShortcut key) override;
-		void keyPressed(EShortcut key) override;
-		void clickReleased(const Point & cursorPosition) override;
-		bool receiveEvent(const Point & position, int eventType) const override;
 
 	private:
 		const OptionsTab & parentTab;
