@@ -12,13 +12,14 @@
 #include "IMapRendererObserver.h"
 #include "mapHandler.h"
 
-#include "../CCallback.h"
-#include "../CGameInfo.h"
 #include "../CPlayerInterface.h"
-#include "../gui/CGuiHandler.h"
+#include "../GameEngine.h"
+#include "../GameInstance.h"
 
 #include "../../lib/texts/CGeneralTextHandler.h"
 #include "../../lib/TerrainHandler.h"
+#include "../../lib/callback/CCallback.h"
+#include "../../lib/callback/IGameInfoCallback.h"
 #include "../../lib/mapObjectConstructors/CObjectClassesHandler.h"
 #include "../../lib/mapObjects/CGHeroInstance.h"
 #include "../../lib/mapObjects/ObjectTemplate.h"
@@ -53,92 +54,30 @@ std::string CMapHandler::getTerrainDescr(const int3 & pos, bool rightClick) cons
 	const TerrainTile & t = map->getTile(pos);
 
 	if(t.hasFavorableWinds())
-		return CGI->objtypeh->getObjectName(Obj::FAVORABLE_WINDS, 0);
+		return LIBRARY->objtypeh->getObjectName(Obj::FAVORABLE_WINDS, 0);
 
 	std::string result = t.getTerrain()->getNameTranslated();
 
-	for(const auto & object : map->objects)
+	for(const auto & object : map->getObjects())
 	{
-		if(object && object->coveringAt(pos) && object->isTile2Terrain())
+		if(object->coveringAt(pos) && object->isTile2Terrain())
 		{
 			result = object->getObjectName();
 			break;
 		}
 	}
 
-	if(LOCPLINT->cb->getTileDigStatus(pos, false) == EDiggingStatus::CAN_DIG)
+	if(GAME->interface()->cb->getTileDigStatus(pos, false) == EDiggingStatus::CAN_DIG)
 	{
 		return boost::str(
 			boost::format(rightClick ? "%s\r\n%s" : "%s %s") // New line for the Message Box, space for the Status Bar
-			% result % CGI->generaltexth->allTexts[330]
+			% result % LIBRARY->generaltexth->allTexts[330]
 		); // 'digging ok'
 	}
 
 	return result;
 }
 
-bool CMapHandler::compareObjectBlitOrder(const CGObjectInstance * a, const CGObjectInstance * b)
-{
-	//FIXME: Optimize
-	// this method is called A LOT on game start and some parts, e.g. for loops are too slow for that
-
-	assert(a && b);
-	if(!a)
-		return true;
-	if(!b)
-		return false;
-
-	// Background objects will always be placed below foreground objects
-	if(a->appearance->printPriority != 0 || b->appearance->printPriority != 0)
-	{
-		if(a->appearance->printPriority != b->appearance->printPriority)
-			return a->appearance->printPriority > b->appearance->printPriority;
-
-		//Two background objects will be placed based on their placement order on map
-		return a->id < b->id;
-	}
-
-	int aBlocksB = 0;
-	int bBlocksA = 0;
-
-	for(const auto & aOffset : a->getBlockedOffsets())
-	{
-		int3 testTarget = a->anchorPos() + aOffset + int3(0, 1, 0);
-		if(b->blockingAt(testTarget))
-			bBlocksA += 1;
-	}
-
-	for(const auto & bOffset : b->getBlockedOffsets())
-	{
-		int3 testTarget = b->anchorPos() + bOffset + int3(0, 1, 0);
-		if(a->blockingAt(testTarget))
-			aBlocksB += 1;
-	}
-
-	// Discovered by experimenting with H3 maps - object priority depends on how many tiles of object A are "blocked" by object B
-	// For example if blockmap of two objects looks like this:
-	//  ABB
-	//  AAB
-	// Here, in middle column object A has blocked tile that is immediately below tile blocked by object B
-	// Meaning, object A blocks 1 tile of object B and object B blocks 0 tiles of object A
-	// In this scenario in H3 object A will always appear above object B, irregardless of H3M order
-	if(aBlocksB != bBlocksA)
-		return aBlocksB < bBlocksA;
-
-	// object that don't have clear priority via tile blocking will appear based on their row
-	if(a->anchorPos().y != b->anchorPos().y)
-		return a->anchorPos().y < b->anchorPos().y;
-
-	// heroes should appear on top of objects on the same tile
-	if(b->ID==Obj::HERO && a->ID!=Obj::HERO)
-		return true;
-	if(b->ID!=Obj::HERO && a->ID==Obj::HERO)
-		return false;
-
-	// or, if all other tests fail to determine priority - simply based on H3M order
-	return a->id < b->id;
-}
-
 CMapHandler::CMapHandler(const CMap * map)
 	: map(map)
 {
@@ -235,11 +174,10 @@ void CMapHandler::removeMapObserver(IMapObjectObserver * object)
 
 IMapObjectObserver::IMapObjectObserver()
 {
-	CGI->mh->addMapObserver(this);
+	GAME->map().addMapObserver(this);
 }
 
 IMapObjectObserver::~IMapObjectObserver()
 {
-	if (CGI && CGI->mh)
-		CGI->mh->removeMapObserver(this);
+	GAME->map().removeMapObserver(this);
 }
