@@ -27,7 +27,7 @@
 #include "../../lib/json/JsonNode.h"
 #include "../lib/modding/CModHandler.h"
 #include "../lib/modding/ModScope.h"
-#include "../lib/VCMI_Lib.h"
+#include "../lib/GameLibrary.h"
 
 #include <SDL_surface.h>
 
@@ -89,7 +89,7 @@ void Graphics::loadPaletteAndColors()
 
 void Graphics::initializeBattleGraphics()
 {
-	auto allConfigs = VLC->modh->getActiveMods();
+	auto allConfigs = LIBRARY->modh->getActiveMods();
 	allConfigs.insert(allConfigs.begin(), ModScope::scopeBuiltin());
 	for(auto & mod : allConfigs)
 	{
@@ -118,18 +118,19 @@ Graphics::Graphics()
 {
 	loadPaletteAndColors();
 	initializeBattleGraphics();
-	loadErmuToPicture();
 
 	//(!) do not load any CAnimation here
 }
 
 void Graphics::setPlayerPalette(SDL_Palette * targetPalette, PlayerColor player)
 {
+	assert(targetPalette);
+
 	SDL_Color palette[32];
 	if(player.isValidPlayer())
 	{
 		for(int i=0; i<32; ++i)
-			palette[i] = CSDL_Ext::toSDL(playerColorPalette[player][i]);
+			palette[i] = CSDL_Ext::toSDL(playerColorPalette[player.getNum()][i]);
 	}
 	else
 	{
@@ -153,21 +154,3 @@ void Graphics::setPlayerFlagColor(SDL_Palette * targetPalette, PlayerColor playe
 		SDL_SetPaletteColors(targetPalette, &color, 5, 1);
 	}
 }
-
-void Graphics::loadErmuToPicture()
-{
-	//loading ERMU to picture
-	const JsonNode config(JsonPath::builtin("config/ERMU_to_picture.json"));
-	int etp_idx = 0;
-	for(const JsonNode &etp : config["ERMU_to_picture"].Vector()) {
-		int idx = 0;
-		for(const JsonNode &n : etp.Vector()) {
-			ERMUtoPicture[idx][etp_idx] = n.String();
-			idx ++;
-		}
-		assert (idx == std::size(ERMUtoPicture));
-
-		etp_idx ++;
-	}
-	assert (etp_idx == 44);
-}
