@@ -10,25 +10,35 @@
 
 #pragma once
 
+#include "../../lib/constants/Enumerations.h"
+
 VCMI_LIB_NAMESPACE_BEGIN
 class Point;
 class Rect;
 VCMI_LIB_NAMESPACE_END
 
+class Canvas;
+
 class IScreenHandler
 {
 public:
 	virtual ~IScreenHandler() = default;
 
 	/// Updates window state after fullscreen state has been changed in settings
-	virtual void onScreenResize() = 0;
-
-	/// De-initializes window state
-	virtual void close() = 0;
+	virtual bool onScreenResize(bool keepWindowResolution) = 0;
 
 	/// Fills screen with black color, erasing any existing content
 	virtual void clearScreen() = 0;
 
+	/// Returns canvas that can be used to display objects on screen
+	virtual Canvas getScreenCanvas() const = 0;
+
+	/// Synchronizes internal screen texture. Screen canvas may not be modified during this call
+	virtual void updateScreenTexture() = 0;
+
+	/// Presents screen texture on the screen
+	virtual void presentScreenTexture() = 0;
+
 	/// Returns list of resolutions supported by current screen
 	virtual std::vector<Point> getSupportedResolutions() const = 0;
 
@@ -48,6 +58,10 @@ public:
 
 	virtual int getScalingFactor() const = 0;
 
+	virtual void screenShot() const = 0;
+
 	/// Window has focus
 	virtual bool hasFocus() = 0;
+
+	virtual void setColorScheme(ColorScheme scheme) = 0;
 };
