@@ -13,20 +13,9 @@
 #include "../../lib/Color.h"
 
 struct SDL_Rect;
-struct SDL_Window;
-struct SDL_Renderer;
-struct SDL_Texture;
 struct SDL_Surface;
 struct SDL_Color;
 
-VCMI_LIB_NAMESPACE_BEGIN
-
-class PlayerColor;
-class Rect;
-class Point;
-
-VCMI_LIB_NAMESPACE_END
-
 namespace CSDL_Ext
 {
 
@@ -42,12 +31,6 @@ ColorRGBA fromSDL(const SDL_Color & color);
 /// creates SDL_Color using provided Color
 SDL_Color toSDL(const ColorRGBA & color);
 
-void setColors(SDL_Surface *surface, SDL_Color *colors, int firstcolor, int ncolors);
-void setAlpha(SDL_Surface * bg, int value);
-
-using TColorPutter = void (*)(uint8_t *&, const uint8_t &, const uint8_t &, const uint8_t &);
-using TColorPutterAlpha = void (*)(uint8_t *&, const uint8_t &, const uint8_t &, const uint8_t &, const uint8_t &);
-
 	void blitAt(SDL_Surface * src, int x, int y, SDL_Surface * dst);
 	void blitAt(SDL_Surface * src, const Rect & pos, SDL_Surface * dst);
 
@@ -66,12 +49,10 @@ using TColorPutterAlpha = void (*)(uint8_t *&, const uint8_t &, const uint8_t &,
 
 	SDL_Surface * verticalFlip(SDL_Surface * toRot); //vertical flip
 	SDL_Surface * horizontalFlip(SDL_Surface * toRot); //horizontal flip
+	SDL_Surface * Rotate90(SDL_Surface * src);
 	uint32_t getPixel(SDL_Surface * surface, const int & x, const int & y, bool colorByte = false);
-	bool isTransparent(SDL_Surface * srf, int x, int y); //checks if surface is transparent at given position
-	bool isTransparent(SDL_Surface * srf, const Point & position); //checks if surface is transparent at given position
 
 	uint8_t * getPxPtr(const SDL_Surface * const & srf, const int x, const int y);
-	TColorPutter getPutterFor(SDL_Surface * const & dest);
 
 	template<int bpp, bool useAlpha>
 	int blit8bppAlphaTo24bppT(const SDL_Surface * src, const Rect & srcRect, SDL_Surface * dst, const Point & dstPoint, uint8_t alpha); //blits 8 bpp surface with alpha channel to 24 bpp surface
@@ -86,13 +67,9 @@ using TColorPutterAlpha = void (*)(uint8_t *&, const uint8_t &, const uint8_t &,
 
 	SDL_Surface * newSurface(const Point & dimensions, SDL_Surface * mod); //creates new surface, with flags/format same as in surface given
 	SDL_Surface * newSurface(const Point & dimensions); //creates new surface, with flags/format same as in screen surface
-	SDL_Surface * copySurface(SDL_Surface * mod); //returns copy of given surface
-	template<int bpp>
-	SDL_Surface * createSurfaceWithBpp(int width, int height); //create surface with give bits per pixels value
 
-	template<int bpp>
-	void convertToGrayscaleBpp(SDL_Surface * surf, const Rect & rect);
 	void convertToGrayscale(SDL_Surface * surf, const Rect & rect);
+	void convertToH2Scheme(SDL_Surface * surf, const Rect & rect);
 
 	void setColorKey(SDL_Surface * surface, SDL_Color color);
 
@@ -101,24 +78,8 @@ using TColorPutterAlpha = void (*)(uint8_t *&, const uint8_t &, const uint8_t &,
 	///set key-color to 0,255,255 only if it exactly mapped
 	void setDefaultColorKeyPresize(SDL_Surface * surface);
 
-	/// helper that will safely set and un-set ClipRect for SDL_Surface
-	class CClipRectGuard: boost::noncopyable
-	{
-		SDL_Surface * surf;
-		Rect oldRect;
+	SDL_Surface * drawOutline(SDL_Surface * source, const SDL_Color & color, int thickness);
+	SDL_Surface * drawShadow(SDL_Surface * source, bool doSheer);
 
-		int getScalingFactor() const;
-
-	public:
-		CClipRectGuard(SDL_Surface * surface, const Rect & rect): surf(surface)
-		{
-			CSDL_Ext::getClipRect(surf, oldRect);
-			CSDL_Ext::setClipRect(surf, rect * getScalingFactor());
-		}
-
-		~CClipRectGuard()
-		{
-			CSDL_Ext::setClipRect(surf, oldRect);
-		}
-	};
+	void adjustBrightness(SDL_Surface* surface, float factor);
 }
