@@ -13,9 +13,6 @@
 #include "Buttons.h"
 #include "../CPlayerInterface.h"
 
-#include "../../CCallback.h"
-
-#include "../../lib/ArtifactUtils.h"
 #include "../../lib/mapObjects/CGHeroInstance.h"
 #include "../../lib/networkPacks/ArtifactLocation.h"
 
