@@ -14,12 +14,12 @@
 #include "TextControls.h"
 
 #include "../eventsSDL/InputHandler.h"
-#include "../gui/CGuiHandler.h"
+#include "../GameEngine.h"
 #include "../gui/WindowHandler.h"
 #include "../render/IImage.h"
-#include "../CGameInfo.h"
 
 #include "../../lib/texts/CGeneralTextHandler.h"
+#include "../../lib/GameLibrary.h"
 
 RadialMenuItem::RadialMenuItem(const std::string & imageName, const std::string & hoverText, const std::function<void()> & callback, bool alternativeLayout)
 	: callback(callback)
@@ -72,7 +72,7 @@ void RadialMenu::addItem(const Point & offset, bool enabled, const std::string &
 	if (!enabled)
 		return;
 
-	auto item = std::make_shared<RadialMenuItem>(path, CGI->generaltexth->translate(hoverText), callback, alternativeLayout);
+	auto item = std::make_shared<RadialMenuItem>(path, LIBRARY->generaltexth->translate(hoverText), callback, alternativeLayout);
 
 	item->moveBy(offset);
 
@@ -101,6 +101,9 @@ std::shared_ptr<RadialMenuItem> RadialMenu::findNearestItem(const Point & cursor
 		}
 	}
 
+	if (bestDistance > 2 * requiredDistanceFromCenter)
+		return nullptr;
+
 	assert(bestItem);
 	return bestItem;
 }
@@ -117,15 +120,15 @@ void RadialMenu::gesturePanning(const Point & initialPosition, const Point & cur
 
 		if (newSelection)
 		{
-			GH.statusbar()->write(newSelection->hoverText);
+			ENGINE->statusbar()->write(newSelection->hoverText);
 			newSelection->setSelected(true);
 		}
 		else
-			GH.statusbar()->clear();
+			ENGINE->statusbar()->clear();
 
 		selectedItem = newSelection;
 
-		GH.windows().totalRedraw();
+		ENGINE->windows().totalRedraw();
 	}
 }
 
@@ -137,10 +140,10 @@ void RadialMenu::gesture(bool on, const Point & initialPosition, const Point & f
 	auto item = findNearestItem(finalPosition);
 
 	// we need to close this window first so if action spawns a new window it won't be closed instead
-	GH.windows().popWindows(1);
+	ENGINE->windows().popWindows(1);
 	if (item)
 	{
-		GH.input().hapticFeedback();
+		ENGINE->input().hapticFeedback();
 		item->callback();
 	}
 }
