@@ -10,18 +10,21 @@
 #include "StdInc.h"
 #include "TradePanels.h"
 
-#include "../../gui/CGuiHandler.h"
+#include "../../GameEngine.h"
+#include "../../GameInstance.h"
 #include "../../render/Canvas.h"
+#include "../../widgets/Slider.h"
 #include "../../widgets/TextControls.h"
 #include "../../windows/InfoWindows.h"
 
-#include "../../CGameInfo.h"
 #include "../../CPlayerInterface.h"
 
-#include "../../../CCallback.h"
-
+#include "../../../lib/GameLibrary.h"
+#include "../../../lib/callback/CCallback.h"
+#include "../../../lib/entities/artifact/CArtHandler.h"
 #include "../../../lib/texts/CGeneralTextHandler.h"
 #include "../../../lib/mapObjects/CGHeroInstance.h"
+#include "../../../lib/entities/ResourceTypeHandler.h"
 
 CTradeableItem::CTradeableItem(const Rect & area, EType Type, int32_t ID, int32_t serial)
 	: SelectableSlot(area, Point(1, 1))
@@ -139,9 +142,9 @@ int CTradeableItem::getIndex()
 		return id;
 	case EType::ARTIFACT_TYPE:
 	case EType::ARTIFACT:
-		return CGI->artifacts()->getByIndex(id)->getIconIndex();
+		return LIBRARY->artifacts()->getByIndex(id)->getIconIndex();
 	case EType::CREATURE:
-		return CGI->creatures()->getByIndex(id)->getIconIndex();
+		return LIBRARY->creatures()->getByIndex(id)->getIconIndex();
 	default:
 		return -1;
 	}
@@ -157,27 +160,27 @@ void CTradeableItem::hover(bool on)
 {
 	if(!on || id == -1)
 	{
-		GH.statusbar()->clear();
+		ENGINE->statusbar()->clear();
 		return;
 	}
 
 	switch(type)
 	{
 	case EType::CREATURE:
-		GH.statusbar()->write(boost::str(boost::format(CGI->generaltexth->allTexts[481]) % CGI->creh->objects[id]->getNamePluralTranslated()));
+		ENGINE->statusbar()->write(boost::str(boost::format(LIBRARY->generaltexth->allTexts[481]) % LIBRARY->creh->objects[id]->getNamePluralTranslated()));
 		break;
 	case EType::ARTIFACT_TYPE:
 	case EType::ARTIFACT:
 		if(id < 0)
-			GH.statusbar()->write(CGI->generaltexth->zelp[582].first);
+			ENGINE->statusbar()->write(LIBRARY->generaltexth->zelp[582].first);
 		else
-			GH.statusbar()->write(CGI->artifacts()->getByIndex(id)->getNameTranslated());
+			ENGINE->statusbar()->write(LIBRARY->artifacts()->getByIndex(id)->getNameTranslated());
 		break;
 	case EType::RESOURCE:
-		GH.statusbar()->write(CGI->generaltexth->restypes[id]);
+		ENGINE->statusbar()->write(GameResID(id).toResource()->getNameTranslated());
 		break;
 	case EType::PLAYER:
-		GH.statusbar()->write(CGI->generaltexth->capColors[id]);
+		ENGINE->statusbar()->write(LIBRARY->generaltexth->capColors[id]);
 		break;
 	}
 }
@@ -191,7 +194,7 @@ void CTradeableItem::showPopupWindow(const Point & cursorPosition)
 	case EType::ARTIFACT_TYPE:
 	case EType::ARTIFACT:
 		if (id >= 0)
-			CRClickPopup::createAndPush(CGI->artifacts()->getByIndex(id)->getDescriptionTranslated());
+			CRClickPopup::createAndPush(LIBRARY->artifacts()->getByIndex(id)->getDescriptionTranslated());
 		break;
 	}
 }
@@ -260,20 +263,50 @@ bool TradePanelBase::isHighlighted() const
 
 ResourcesPanel::ResourcesPanel(const CTradeableItem::ClickPressedFunctor & clickPressedCallback,
 	const UpdateSlotsFunctor & updateSubtitles)
+	: clickPressedCallback(std::move(clickPressedCallback))
 {
-	assert(resourcesForTrade.size() == slotsPos.size());
 	OBJECT_CONSTRUCTION;
 
-	for(const auto & res : resourcesForTrade)
+	resourcesForTrade = LIBRARY->resourceTypeHandler->getAllObjects();
+
+	int lines = vstd::divideAndCeil(resourcesForTrade.size(), 3);
+	if(lines > 3)
 	{
-		auto slot = slots.emplace_back(std::make_shared<CTradeableItem>(Rect(slotsPos[res.num], slotDimension), EType::RESOURCE, res.num, res.num));
-		slot->clickPressedCallback = clickPressedCallback;
-		slot->setSelectionWidth(selectionWidth);
+		slider = std::make_shared<CSlider>(Point(240, 0), 224, [this](int to){ updateSlots(to); setRedrawParent(true); redraw(); }, 3, lines, 0, Orientation::VERTICAL, CSlider::BROWN);
+		slider->setPanningStep(72);
+		slider->setScrollBounds(Rect(-240, 0, 240, 224));
 	}
-	updateSlotsCallback = updateSubtitles;
+
+	updateSlotsCallback = std::move(updateSubtitles);
+
+	updateSlots(0);
 	showcaseSlot = std::make_shared<CTradeableItem>(Rect(selectedPos, slotDimension), EType::RESOURCE, 0, 0);
 }
 
+void ResourcesPanel::updateSlots(int line)
+{
+	OBJECT_CONSTRUCTION;
+
+	clickPressedCallback(nullptr);
+
+	int offset = line * 3;
+	slots.clear();
+	for (int i = 0; i < std::min(static_cast<int>(resourcesForTrade.size() - offset), 9); i++)
+	{
+		const auto& res = resourcesForTrade[i + offset];
+		auto slotPos = slotsPos[i];
+		if(resourcesForTrade.size() == 7 && i == 6) // for 7 ressources place gold in the middle
+			slotPos = slotsPos[i + 1];
+
+		auto slotPtr = std::make_shared<CTradeableItem>(Rect(slotPos, slotDimension), EType::RESOURCE, res.num, res.num);
+		slotPtr->clickPressedCallback = clickPressedCallback;
+		slotPtr->setSelectionWidth(selectionWidth);
+		slots.push_back(slotPtr);
+	}
+
+	updateSlotsCallback();
+}
+
 ArtifactsPanel::ArtifactsPanel(const CTradeableItem::ClickPressedFunctor & clickPressedCallback,
 	const UpdateSlotsFunctor & updateSubtitles, const std::vector<TradeItemBuy> & arts)
 {
@@ -305,7 +338,7 @@ PlayersPanel::PlayersPanel(const CTradeableItem::ClickPressedFunctor & clickPres
 	std::vector<PlayerColor> players;
 	for(auto player = PlayerColor(0); player < PlayerColor::PLAYER_LIMIT_I; player++)
 	{
-		if(player != LOCPLINT->playerID && LOCPLINT->cb->getPlayerStatus(player) == EPlayerStatus::INGAME)
+		if(player != GAME->interface()->playerID && GAME->interface()->cb->getPlayerStatus(player) == EPlayerStatus::INGAME)
 			players.emplace_back(player);
 	}
 
@@ -316,7 +349,7 @@ PlayersPanel::PlayersPanel(const CTradeableItem::ClickPressedFunctor & clickPres
 		slot = std::make_shared<CTradeableItem>(Rect(slotsPos[slotNum], slotDimension), EType::PLAYER, players[slotNum].num, slotNum);
 		slot->clickPressedCallback = clickPressedCallback;
 		slot->setSelectionWidth(selectionWidth);
-		slot->subtitle->setText(CGI->generaltexth->capColors[players[slotNum].num]);
+		slot->subtitle->setText(LIBRARY->generaltexth->capColors[players[slotNum].num]);
 		slotNum++;
 	}
 	showcaseSlot = std::make_shared<CTradeableItem>(Rect(selectedPos, slotDimension), EType::PLAYER, 0, 0);
