@@ -13,6 +13,8 @@
 #include "../widgets/CExchangeController.h"
 
 class CGarrisonSlot;
+class CMultiLineLabel;
+class LRClickableAreaWText;
 
 class CExchangeWindow : public CStatusbarWindow, public IGarrisonHolder, public CWindowWithArtifacts
 {
@@ -27,6 +29,8 @@ class CExchangeWindow : public CStatusbarWindow, public IGarrisonHolder, public
 
 	std::vector<std::shared_ptr<LRClickableAreaWTextComp>> primSkillAreas;
 	std::array<std::vector<std::shared_ptr<CSecSkillPlace>>, 2> secSkills;
+	std::array<std::shared_ptr<CMultiLineLabel>, 2> secSkillsFull;
+	std::array<std::shared_ptr<LRClickableAreaWText>, 2> secSkillsFullArea;
 
 	std::array<std::shared_ptr<CHeroArea>, 2> heroAreas;
 	std::array<std::shared_ptr<LRClickableAreaWText>, 2> specialtyAreas;
@@ -71,7 +75,7 @@ public:
 
 	void keyPressed(EShortcut key) override;
 
-	void update() override;
+	void updateArtifacts() override;
 
 	// IGarrisonHolder impl
 	void updateGarrisons() override;
