@@ -36,6 +36,7 @@ class FilledTexturePlayerColored;
 class TransparentFilledRectangle;
 class CMinimapInstance;
 class CLabel;
+class CSlider;
 
 /// text + comp. + ok button
 class CInfoWindow : public WindowBase
@@ -58,7 +59,7 @@ public:
 	CInfoWindow();
 	~CInfoWindow();
 
-	//use only before the game starts! (showYesNoDialog in LOCPLINT must be used then)
+	//use only before the game starts! (showYesNoDialog in GAME->interface() must be used then)
 	static void showInfoDialog(const std::string & text, const TCompsInfo & components, PlayerColor player = PlayerColor(1));
 	static void showYesNoDialog(const std::string & text, const TCompsInfo & components, const CFunctionList<void()> & onYes, const CFunctionList<void()> & onNo, PlayerColor player = PlayerColor(1));
 	static std::shared_ptr<CInfoWindow> create(const std::string & text, PlayerColor playerID = PlayerColor(1), const TCompsInfo & components = TCompsInfo());
@@ -127,17 +128,20 @@ public:
 
 class MinimapWithIcons : public CIntObject
 {
-	std::shared_ptr<TransparentFilledRectangle> backgroundSurface;
-	std::shared_ptr<TransparentFilledRectangle> backgroundUnderground;
+	std::shared_ptr<TransparentFilledRectangle> background1;
+	std::shared_ptr<TransparentFilledRectangle> background2;
 
-	std::shared_ptr<CMinimapInstance> surface;
-	std::shared_ptr<CMinimapInstance> undergroud;
+	std::shared_ptr<CMinimapInstance> map1;
+	std::shared_ptr<CMinimapInstance> map2;
 
 	std::vector<std::shared_ptr<CPicture>> iconsOverlay;
+	std::shared_ptr<CSlider> slider;
 
+	std::vector<std::pair<const int3, const ImagePath>> icons;
 public:
 	MinimapWithIcons(const Point & position);
 
+	void recreate();
 	void addIcon(const int3 & coordinates, const ImagePath & image);
 };
 
@@ -172,3 +176,13 @@ class ObeliskPopup : public AdventureMapPopup
 public:
 	ObeliskPopup(const Point & position, const CGObelisk * obelisk);
 };
+
+class SearchPopup : public AdventureMapPopup
+{
+	std::shared_ptr<FilledTexturePlayerColored> filledBackground;
+	std::shared_ptr<MinimapWithIcons> minimap;
+	std::shared_ptr<CLabel> labelTitle;
+
+public:
+	SearchPopup(std::vector<const CGObjectInstance *> objs);
+};
