@@ -10,6 +10,7 @@
 #pragma once
 
 #include <TargetConditionals.h>
+#include <string>
 
 #pragma GCC visibility push(default)
 namespace iOS_utils
@@ -17,6 +18,11 @@ namespace iOS_utils
 const char *documentsPath();
 const char *cachesPath();
 
+// share file using system share sheet (e.g. Mail app)
+void shareFile(const std::string & filePath);
+
+std::string iphoneHardwareId();
+
 #if TARGET_OS_SIMULATOR
 const char *hostApplicationSupportPath();
 #endif
@@ -27,5 +33,6 @@ const char *frameworksPath();
 const char *bundleIdentifier();
 
 bool isOsVersionAtLeast(unsigned int osMajorVersion);
+void keepScreenOn(bool isEnabled);
 }
 #pragma GCC visibility pop
