@@ -28,6 +28,7 @@ class AboutProjectView : public QWidget
 
 public:
 	explicit AboutProjectView(QWidget * parent = nullptr);
+	~AboutProjectView() override;
 
 private slots:
 	void on_updatesButton_clicked();
@@ -46,8 +47,10 @@ private slots:
 
 	void on_pushButtonBugreport_clicked();
 
+	void on_pushButtonExportLogs_clicked();
+
 	void on_openConfigDir_clicked();
 
 private:
-	Ui::AboutProjectView * ui;
+	std::unique_ptr<Ui::AboutProjectView> ui;
 };
