@@ -62,6 +62,28 @@ public:
 	{
 		return r == rhs.r && g == rhs.g && b == rhs.b && a == rhs.a;
 	}
+	
+	bool operator<(const ColorRGBA& rhs) const
+	{
+		auto mean_lhs = (r + g + b + a) / 4.0;
+		auto mean_rhs = (rhs.r + rhs.g + rhs.b + rhs.a) / 4.0;
+		return mean_lhs < mean_rhs;
+	}
+
 };
 
+namespace vstd
+{
+template<typename Floating>
+ColorRGBA lerp(const ColorRGBA & left, const ColorRGBA & right, const Floating & factor)
+{
+	return ColorRGBA(
+		vstd::lerp(left.r, right.r, factor),
+		vstd::lerp(left.g, right.g, factor),
+		vstd::lerp(left.b, right.b, factor),
+		vstd::lerp(left.a, right.a, factor)
+	);
+}
+}
+
 VCMI_LIB_NAMESPACE_END
