@@ -10,6 +10,7 @@
 #pragma once
 
 #include "filesystem/ResourcePath.h"
+#include "IHandlerBase.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
@@ -18,12 +19,12 @@ struct Bonus;
 
 ///High level interface for BonusTypeHandler
 
-class DLL_LINKAGE IBonusTypeHandler
+class DLL_LINKAGE IBonusTypeHandler : public IHandlerBase
 {
 public:
 	virtual ~IBonusTypeHandler() = default;
 
-	virtual std::string bonusToString(const std::shared_ptr<Bonus> & bonus, const IBonusBearer * bearer, bool description) const = 0;
+	virtual std::string bonusToString(const std::shared_ptr<Bonus> & bonus, const IBonusBearer * bearer) const = 0;
 	virtual ImagePath bonusToGraphics(const std::shared_ptr<Bonus> & bonus) const = 0;
 };
 
