@@ -26,16 +26,19 @@ class ResourceSet;
 class ResourceSet
 {
 private:
-	std::array<TResource, GameConstants::RESOURCE_QUANTITY> container = {};
+	std::vector<TResource> container = {};
+	DLL_LINKAGE void resizeContainer();
 public:
-	// read resources set from json. Format example: { "gold": 500, "wood":5 }
-	DLL_LINKAGE ResourceSet(const JsonNode & node);
 	DLL_LINKAGE ResourceSet();
+	DLL_LINKAGE ResourceSet(const ResourceSet& rhs);
+
+	DLL_LINKAGE void resolveFromJson(const JsonNode & node);
 
 
 #define scalarOperator(OPSIGN)									\
 	ResourceSet& operator OPSIGN ## =(const TResource &rhs) \
 	{														\
+		resizeContainer(); \
 		for(auto i = 0; i < container.size(); i++)						\
 			container.at(i) OPSIGN ## = rhs;						\
 															\
@@ -45,6 +48,7 @@ public:
 #define vectorOperator(OPSIGN)										\
 	ResourceSet& operator OPSIGN ## =(const ResourceSet &rhs)	\
 	{															\
+		resizeContainer(); \
 		for(auto i = 0; i < container.size(); i++)							\
 			container.at(i) OPSIGN ## = rhs[i];						\
 																\
@@ -84,21 +88,31 @@ public:
 	// Array-like interface
 	TResource & operator[](GameResID index)
 	{
+		resizeContainer();
 		return operator[](index.getNum());
 	}
 
 	const TResource & operator[](GameResID index) const 
 	{
+		if (index.getNum() >= container.size()) {
+			static const TResource defaultValue{};
+			return defaultValue;
+		}
 		return operator[](index.getNum());
 	}
 
 	TResource & operator[](size_t index)
 	{
+		resizeContainer();
 		return container.at(index);
 	}
 
 	const TResource & operator[](size_t index) const 
 	{
+		if (index >= container.size()) {
+			static const TResource defaultValue{};
+			return defaultValue;
+		}
 		return container.at(index);
 	}
 
@@ -176,6 +190,15 @@ public:
 		return *this;
 	}
 
+	ResourceSet& operator=(const ResourceSet& rhs)
+	{
+		if (this != &rhs)
+		{
+			container = rhs.container;
+		}
+		return *this;
+	}
+
 	ResourceSet operator-() const
 	{
 		ResourceSet ret;
@@ -184,6 +207,14 @@ public:
 		return ret;
 	}
 
+	ResourceSet multipliedBy(double factor) const
+	{
+		ResourceSet ret;
+		for(int i = 0; i < container.size(); i++)
+			ret[i] = static_cast<int>(std::round(container.at(i) * factor));
+		return ret;
+	}
+
 	bool operator==(const ResourceSet &rhs) const
 	{
 		return this->container == rhs.container;
@@ -191,7 +222,25 @@ public:
 
 	template <typename Handler> void serialize(Handler &h)
 	{
-		h & container;
+		if (h.version >= Handler::Version::CONFIGURABLE_RESOURCES)
+			h & container;
+		else
+		{
+			if (h.saving)
+			{
+				std::array<TResource, 8> tmp = {};
+				for (size_t i = 0; i < 7; ++i)
+        			tmp[i] = container[i];
+				tmp[7] = TResource{};
+				h & tmp;
+			}
+			else
+			{
+				std::array<TResource, 8> tmp = {};
+				h & tmp;
+				container = std::vector<TResource>(tmp.begin(), tmp.begin() + 7);
+			}
+		}
 	}
 
 	DLL_LINKAGE void serializeJson(JsonSerializeFormat & handler, const std::string & fieldName);
