@@ -11,7 +11,7 @@
 #include "StartInfo.h"
 
 #include "texts/CGeneralTextHandler.h"
-#include "VCMI_Lib.h"
+#include "GameLibrary.h"
 #include "entities/faction/CFaction.h"
 #include "entities/faction/CTownHandler.h"
 #include "entities/hero/CHeroHandler.h"
@@ -21,6 +21,7 @@
 #include "mapping/CMapHeader.h"
 #include "mapping/CMapService.h"
 #include "modding/ModIncompatibility.h"
+#include "serializer/JsonSerializeFormat.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
@@ -33,7 +34,7 @@ FactionID PlayerSettings::getCastleValidated() const
 {
 	if (!castle.isValid())
 		return FactionID(0);
-	if (castle.getNum() < VLC->townh->size() && castle.toEntity(VLC)->hasTown())
+	if (castle.getNum() < LIBRARY->townh->size() && castle.toEntity(LIBRARY)->hasTown())
 		return castle;
 
 	return FactionID(0);
@@ -43,7 +44,7 @@ HeroTypeID PlayerSettings::getHeroValidated() const
 {
 	if (!hero.isValid())
 		return HeroTypeID(0);
-	if (hero.getNum() < VLC->heroh->size())
+	if (hero.getNum() < LIBRARY->heroh->size())
 		return hero;
 
 	return HeroTypeID(0);
@@ -71,7 +72,7 @@ const PlayerSettings & StartInfo::getIthPlayersSettings(const PlayerColor & no)
 	return const_cast<StartInfo &>(*this).getIthPlayersSettings(no);
 }
 
-PlayerSettings * StartInfo::getPlayersSettings(const ui8 connectedPlayerId)
+PlayerSettings * StartInfo::getPlayersSettings(PlayerConnectionID connectedPlayerId)
 {
 	for(auto & elem : playerInfos)
 	{
@@ -87,31 +88,23 @@ std::string StartInfo::getCampaignName() const
 	if(!campState->getNameTranslated().empty())
 		return campState->getNameTranslated();
 	else
-		return VLC->generaltexth->allTexts[508];
+		return LIBRARY->generaltexth->allTexts[508];
 }
 
-bool StartInfo::isRestorationOfErathiaCampaign() const
+bool StartInfo::restrictedGarrisonsForAI() const
 {
-	constexpr std::array roeCampaigns = {
-		"DATA/GOOD1",
-		"DATA/EVIL1",
-		"DATA/GOOD2",
-		"DATA/NEUTRAL1",
-		"DATA/EVIL2",
-		"DATA/GOOD3",
-		"DATA/SECRET1",
-	};
+	return campState && campState->restrictedGarrisonsForAI();
+}
 
-	if (!campState)
-		return false;
-
-	return vstd::contains(roeCampaigns, campState->getFilename());
+EMapDifficulty StartInfo::getDifficulty() const
+{
+	return static_cast<EMapDifficulty>(difficulty);
 }
 
 void LobbyInfo::verifyStateBeforeStart(bool ignoreNoHuman) const
 {
 	if(!mi || !mi->mapHeader)
-		throw std::domain_error(VLC->generaltexth->translate("core.genrltxt.529"));
+		throw std::domain_error(LIBRARY->generaltexth->translate("core.genrltxt.529"));
 	
 	auto missingMods = CMapService::verifyMapHeaderMods(*mi->mapHeader);
 	ModIncompatibility::ModList modList;
@@ -128,16 +121,16 @@ void LobbyInfo::verifyStateBeforeStart(bool ignoreNoHuman) const
 			break;
 
 	if(i == si->playerInfos.cend() && !ignoreNoHuman)
-		throw std::domain_error(VLC->generaltexth->translate("core.genrltxt.530"));
+		throw std::domain_error(LIBRARY->generaltexth->translate("core.genrltxt.530"));
 
 	if(si->mapGenOptions && si->mode == EStartMode::NEW_GAME)
 	{
 		if(!si->mapGenOptions->checkOptions())
-			throw std::domain_error(VLC->generaltexth->translate("core.genrltxt.751"));
+			throw std::domain_error(LIBRARY->generaltexth->translate("core.genrltxt.751"));
 	}
 }
 
-bool LobbyInfo::isClientHost(int clientId) const
+bool LobbyInfo::isClientHost(GameConnectionID clientId) const
 {
 	return clientId == hostClientId;
 }
@@ -147,7 +140,7 @@ bool LobbyInfo::isPlayerHost(const PlayerColor & color) const
 	return vstd::contains(getAllClientPlayers(hostClientId), color);
 }
 
-std::set<PlayerColor> LobbyInfo::getAllClientPlayers(int clientId) const
+std::set<PlayerColor> LobbyInfo::getAllClientPlayers(GameConnectionID clientId) const
 {
 	std::set<PlayerColor> players;
 	for(auto & elem : si->playerInfos)
@@ -155,7 +148,7 @@ std::set<PlayerColor> LobbyInfo::getAllClientPlayers(int clientId) const
 		if(isClientHost(clientId) && elem.second.isControlledByAI())
 			players.insert(elem.first);
 
-		for(ui8 id : elem.second.connectedPlayerIDs)
+		for(PlayerConnectionID id : elem.second.connectedPlayerIDs)
 		{
 			if(vstd::contains(getConnectedPlayerIdsForClient(clientId), id))
 				players.insert(elem.first);
@@ -167,9 +160,9 @@ std::set<PlayerColor> LobbyInfo::getAllClientPlayers(int clientId) const
 	return players;
 }
 
-std::vector<ui8> LobbyInfo::getConnectedPlayerIdsForClient(int clientId) const
+std::vector<PlayerConnectionID> LobbyInfo::getConnectedPlayerIdsForClient(GameConnectionID clientId) const
 {
-	std::vector<ui8> ids;
+	std::vector<PlayerConnectionID> ids;
 
 	for(const auto & pair : playerNames)
 	{
@@ -185,12 +178,12 @@ std::vector<ui8> LobbyInfo::getConnectedPlayerIdsForClient(int clientId) const
 	return ids;
 }
 
-std::set<PlayerColor> LobbyInfo::clientHumanColors(int clientId)
+std::set<PlayerColor> LobbyInfo::clientHumanColors(GameConnectionID clientId)
 {
 	std::set<PlayerColor> players;
 	for(auto & elem : si->playerInfos)
 	{
-		for(ui8 id : elem.second.connectedPlayerIDs)
+		for(PlayerConnectionID id : elem.second.connectedPlayerIDs)
 		{
 			if(vstd::contains(getConnectedPlayerIdsForClient(clientId), id))
 			{
@@ -203,7 +196,7 @@ std::set<PlayerColor> LobbyInfo::clientHumanColors(int clientId)
 }
 
 
-PlayerColor LobbyInfo::clientFirstColor(int clientId) const
+PlayerColor LobbyInfo::clientFirstColor(GameConnectionID clientId) const
 {
 	for(auto & pair : si->playerInfos)
 	{
@@ -214,11 +207,11 @@ PlayerColor LobbyInfo::clientFirstColor(int clientId) const
 	return PlayerColor::CANNOT_DETERMINE;
 }
 
-bool LobbyInfo::isClientColor(int clientId, const PlayerColor & color) const
+bool LobbyInfo::isClientColor(GameConnectionID clientId, const PlayerColor & color) const
 {
 	if(si->playerInfos.find(color) != si->playerInfos.end())
 	{
-		for(ui8 id : si->playerInfos.find(color)->second.connectedPlayerIDs)
+		for(PlayerConnectionID id : si->playerInfos.find(color)->second.connectedPlayerIDs)
 		{
 			if(playerNames.find(id) != playerNames.end())
 			{
@@ -230,7 +223,7 @@ bool LobbyInfo::isClientColor(int clientId, const PlayerColor & color) const
 	return false;
 }
 
-ui8 LobbyInfo::clientFirstId(int clientId) const
+PlayerConnectionID LobbyInfo::clientFirstId(GameConnectionID clientId) const
 {
 	for(const auto & pair : playerNames)
 	{
@@ -238,7 +231,7 @@ ui8 LobbyInfo::clientFirstId(int clientId) const
 			return pair.first;
 	}
 
-	return 0;
+	throw std::runtime_error("LobbyInfo::clientFirstId: invalid GameConnectionID!");
 }
 
 PlayerInfo & LobbyInfo::getPlayerInfo(PlayerColor color)
@@ -254,4 +247,82 @@ TeamID LobbyInfo::getPlayerTeamId(const PlayerColor & color)
 		return TeamID::NO_TEAM;
 }
 
+BattleOnlyModeStartInfo::BattleOnlyModeStartInfo()
+	: selectedTerrain(TerrainId::DIRT)
+	, selectedTown(FactionID::ANY)
+{
+	for(auto & element : primSkillLevel)
+		for(size_t i=0; i<GameConstants::PRIMARY_SKILLS; i++)
+			element[i] = 0;
+	for(auto & element : warMachines)
+		element = false;
+	for(auto & element : spellBook)
+		element = true;
+}
+
+void BattleOnlyModeStartInfo::serializeJson(JsonSerializeFormat & handler)
+{
+	handler.serializeId("selectedTerrain", selectedTerrain);
+	handler.serializeId("selectedTown", selectedTown);
+	if(!handler.saving && selectedTown == FactionID::NONE)
+		selectedTown = FactionID::ANY;
+
+	auto slots = handler.enterArray("slots");
+	slots.resize(2);
+	for(int i = 0; i < 2; i++)
+	{
+		auto s = slots.enterStruct(i);
+		s->serializeId("selectedHero", selectedHero[i]);
+		{
+			auto selectedArmyArray = s->enterArray("selectedArmy");
+			selectedArmyArray.resize(GameConstants::ARMY_SIZE, JsonNode::JsonType::DATA_VECTOR);
+			for(int j = 0; j < GameConstants::ARMY_SIZE; j++)
+			{
+				JsonArraySerializer inner = selectedArmyArray.enterArray(j);
+				selectedArmy[i][j].serializeJson(handler);
+			}
+		}
+		{
+			auto primSkillLevelArray = s->enterArray("primSkillLevel");
+			primSkillLevelArray.resize(4, JsonNode::JsonType::DATA_VECTOR);
+			for(int j = 0; j < 4; j++)
+				primSkillLevelArray.serializeInt(j, primSkillLevel[i][j]);
+		}
+		{
+			auto secSkillLevelArray = s->enterArray("secSkillLevel");
+			secSkillLevelArray.resize(8, JsonNode::JsonType::DATA_VECTOR);
+			for(int j = 0; j < 8; j++)
+			{
+				JsonArraySerializer inner = secSkillLevelArray.enterArray(j);
+				inner->serializeId("skill", secSkillLevel[i][j].first);
+				inner->serializeEnum("masteryLevel", secSkillLevel[i][j].second, MasteryLevel::NONE, {"none", "basic", "advanced", "expert"});
+			}
+		}
+		if(handler.saving)
+		{
+			auto reversed = vstd::reverseMap(artifacts[i]);
+			std::map<ArtifactID, uint16_t> tmp;
+			for (auto& [id, pos] : reversed)
+    			tmp[id] = static_cast<uint16_t>(pos);
+			tmp.erase(ArtifactID::NONE);
+			s->serializeIdMap("artifacts", tmp);
+		}
+		else
+		{
+			std::map<ArtifactID, uint16_t> tmp;
+			s->serializeIdMap("artifacts", tmp);
+			std::map<ArtifactID, ArtifactPosition> converted;
+			for (auto &[id, pos] : tmp)
+				if(id != ArtifactID::NONE)
+					converted[id] = ArtifactPosition(pos);
+			artifacts[i] = vstd::reverseMap(converted);
+		}
+		s->serializeIdArray("spells", spells[i]);
+		if(!handler.saving)
+			spells[i].erase(std::remove(spells[i].begin(), spells[i].end(), SpellID::NONE), spells[i].end());
+		s->serializeBool("warMachines", warMachines[i]);
+		s->serializeBool("spellBook", spellBook[i]);
+	}
+}
+
 VCMI_LIB_NAMESPACE_END
