@@ -18,7 +18,7 @@ VCMI_LIB_NAMESPACE_BEGIN
 static const int32_t INVALID_UNIT_ID = -1000;
 
 BattleAction::BattleAction():
-	side(-1),
+	side(BattleSide::NONE),
 	stackNumber(-1),
 	actionType(EActionType::NO_ACTION)
 {
@@ -43,7 +43,7 @@ BattleAction BattleAction::makeDefend(const battle::Unit * stack)
 	return ba;
 }
 
-BattleAction BattleAction::makeMeleeAttack(const battle::Unit * stack, BattleHex destination, BattleHex attackFrom, bool returnAfterAttack)
+BattleAction BattleAction::makeMeleeAttack(const battle::Unit * stack, const BattleHex & destination, const BattleHex & attackFrom, bool returnAfterAttack)
 {
 	BattleAction ba;
 	ba.side = stack->unitSide(); //FIXME: will it fail if stack mind controlled?
@@ -56,6 +56,20 @@ BattleAction BattleAction::makeMeleeAttack(const battle::Unit * stack, BattleHex
 	return ba;
 }
 
+/// This is for cases where target hex does not matter (for example berserk spell) and we did not calculate it (for example we only calculated closest attackable hex of two hex unit)
+BattleAction BattleAction::makeMeleeAttack(const battle::Unit * stack, const battle::Unit * target, const BattleHex & attackFrom, bool returnAfterAttack)
+{
+	BattleAction ba;
+	ba.side = stack->unitSide(); //FIXME: will it fail if stack mind controlled?
+	ba.actionType = EActionType::WALK_AND_ATTACK;
+	ba.stackNumber = stack->unitId();
+	ba.aimToHex(attackFrom);
+	ba.aimToUnit(target);
+	if(returnAfterAttack && stack->hasBonusOfType(BonusType::RETURN_AFTER_STRIKE))
+		ba.aimToHex(stack->getPosition());
+	return ba;
+}
+
 BattleAction BattleAction::makeWait(const battle::Unit * stack)
 {
 	BattleAction ba;
@@ -86,7 +100,19 @@ BattleAction BattleAction::makeCreatureSpellcast(const battle::Unit * stack, con
 	return ba;
 }
 
-BattleAction BattleAction::makeMove(const battle::Unit * stack, BattleHex dest)
+BattleAction BattleAction::makeWalkAndCast(const battle::Unit * stack, const BattleHex & castFrom, const battle::Unit * target, const SpellID & spellID)
+{
+	BattleAction ba;
+	ba.actionType = EActionType::WALK_AND_CAST;
+	ba.spell = spellID;
+	ba.side = stack->unitSide();
+	ba.stackNumber = stack->unitId();
+	ba.aimToHex(castFrom);
+	ba.aimToUnit(target);
+	return ba;
+}
+
+BattleAction BattleAction::makeMove(const battle::Unit * stack, const BattleHex & dest)
 {
 	BattleAction ba;
 	ba.side = stack->unitSide();
@@ -96,7 +122,7 @@ BattleAction BattleAction::makeMove(const battle::Unit * stack, BattleHex dest)
 	return ba;
 }
 
-BattleAction BattleAction::makeEndOFTacticPhase(ui8 side)
+BattleAction BattleAction::makeEndOFTacticPhase(BattleSide side)
 {
 	BattleAction ba;
 	ba.side = side;
@@ -104,7 +130,7 @@ BattleAction BattleAction::makeEndOFTacticPhase(ui8 side)
 	return ba;
 }
 
-BattleAction BattleAction::makeSurrender(ui8 side)
+BattleAction BattleAction::makeSurrender(BattleSide side)
 {
 	BattleAction ba;
 	ba.side = side;
@@ -112,7 +138,7 @@ BattleAction BattleAction::makeSurrender(ui8 side)
 	return ba;
 }
 
-BattleAction BattleAction::makeRetreat(ui8 side)
+BattleAction BattleAction::makeRetreat(BattleSide side)
 {
 	BattleAction ba;
 	ba.side = side;
@@ -201,16 +227,18 @@ bool BattleAction::isUnitAction() const
 		EActionType::CATAPULT,
 		EActionType::MONSTER_SPELL,
 		EActionType::BAD_MORALE,
-		EActionType::STACK_HEAL
+		EActionType::STACK_HEAL,
+		EActionType::WALK_AND_CAST
 	};
 	return vstd::contains(actions, actionType);
 }
 
 bool BattleAction::isSpellAction() const
 {
-	static const std::array<EActionType, 2> actions = {
+	static const std::array<EActionType, 3> actions = {
 		EActionType::HERO_SPELL,
-		EActionType::MONSTER_SPELL
+		EActionType::MONSTER_SPELL,
+		EActionType::WALK_AND_CAST
 	};
 	return vstd::contains(actions, actionType);
 }
