@@ -15,8 +15,12 @@
 #include "bonuses/Limiters.h"
 #include "bonuses/Updaters.h"
 #include "../CStack.h"
+#include "../callback/IGameInfoCallback.h"
+#include "../entities/artifact/CArtifact.h"
 #include "../entities/building/TownFortifications.h"
 #include "../filesystem/Filesystem.h"
+#include "../spells/CSpellHandler.h"
+#include "../GameLibrary.h"
 #include "../mapObjects/CGTownInstance.h"
 #include "../texts/CGeneralTextHandler.h"
 #include "../BattleFieldHandler.h"
@@ -24,9 +28,6 @@
 
 #include <vstd/RNG.h>
 
-//TODO: remove
-#include "../IGameCallback.h"
-
 VCMI_LIB_NAMESPACE_BEGIN
 
 const SideInBattle & BattleInfo::getSide(BattleSide side) const
@@ -40,24 +41,22 @@ SideInBattle & BattleInfo::getSide(BattleSide side)
 }
 
 ///BattleInfo
-CStack * BattleInfo::generateNewStack(uint32_t id, const CStackInstance & base, BattleSide side, const SlotID & slot, const BattleHex & position)
+void BattleInfo::generateNewStack(uint32_t id, const CStackInstance & base, BattleSide side, const SlotID & slot, const BattleHex & position)
 {
 	PlayerColor owner = getSide(side).color;
-	assert(!owner.isValidPlayer() || (base.armyObj && base.armyObj->tempOwner == owner));
+	assert(!owner.isValidPlayer() || (base.getArmy() && base.getArmy()->tempOwner == owner));
 
-	auto * ret = new CStack(&base, owner, id, side, slot);
+	auto ret = std::make_unique<CStack>(&base, owner, id, side, slot);
 	ret->initialPosition = getAvailableHex(base.getCreatureID(), side, position.toInt()); //TODO: what if no free tile on battlefield was found?
-	stacks.push_back(ret);
-	return ret;
+	stacks.push_back(std::move(ret));
 }
 
-CStack * BattleInfo::generateNewStack(uint32_t id, const CStackBasicDescriptor & base, BattleSide side, const SlotID & slot, const BattleHex & position)
+void BattleInfo::generateNewStack(uint32_t id, const CStackBasicDescriptor & base, BattleSide side, const SlotID & slot, const BattleHex & position)
 {
 	PlayerColor owner = getSide(side).color;
-	auto * ret = new CStack(&base, owner, id, side, slot);
+	auto ret = std::make_unique<CStack>(&base, owner, id, side, slot);
 	ret->initialPosition = position;
-	stacks.push_back(ret);
-	return ret;
+	stacks.push_back(std::move(ret));
 }
 
 void BattleInfo::localInit()
@@ -69,7 +68,7 @@ void BattleInfo::localInit()
 		armyObj->attachTo(*this);
 	}
 
-	for(CStack * s : stacks)
+	for(auto & s : stacks)
 		s->localInit(this);
 
 	exportBonuses();
@@ -158,23 +157,22 @@ struct RangeGenerator
 	std::function<int()> myRand;
 };
 
-BattleInfo * BattleInfo::setupBattle(const int3 & tile, TerrainId terrain, const BattleField & battlefieldType, BattleSideArray<const CArmedInstance *> armies, BattleSideArray<const CGHeroInstance *> heroes, const BattleLayout & layout, const CGTownInstance * town)
+std::unique_ptr<BattleInfo> BattleInfo::setupBattle(IGameInfoCallback *cb, const int3 & tile, TerrainId terrain, const BattleField & battlefieldType, BattleSideArray<const CArmedInstance *> armies, BattleSideArray<const CGHeroInstance *> heroes, const BattleLayout & layout, const CGTownInstance * town)
 {
 	CMP_stack cmpst;
-	auto * currentBattle = new BattleInfo(layout);
+	auto currentBattle = std::make_unique<BattleInfo>(cb, layout);
 
 	for(auto i : { BattleSide::LEFT_SIDE, BattleSide::RIGHT_SIDE})
-		currentBattle->sides[i].init(heroes[i], armies[i]);
-
-	std::vector<CStack*> & stacks = (currentBattle->stacks);
+		currentBattle->sides[i].init(heroes[i], armies[i], i == BattleSide::RIGHT_SIDE ? town : nullptr);
 
 	currentBattle->tile = tile;
 	currentBattle->terrainType = terrain;
 	currentBattle->battlefieldType = battlefieldType;
-	currentBattle->round = -2;
+	currentBattle->round = 0;
 	currentBattle->activeStack = -1;
 	currentBattle->replayAllowed = false;
-	currentBattle->town = town;
+	if (town)
+		currentBattle->townID = town->id;
 
 	//setting up siege obstacles
 	if (town && town->fortificationsLevel().wallsHealth != 0)
@@ -224,7 +222,7 @@ BattleInfo * BattleInfo::setupBattle(const int3 & tile, TerrainId terrain, const
 		{
 			try
 			{
-				RangeGenerator obidgen(0, VLC->obstacleHandler->size() - 1, ourRand);
+				RangeGenerator obidgen(0, LIBRARY->obstacleHandler->size() - 1, ourRand);
 				auto obstPtr = std::make_shared<CObstacleInstance>();
 				obstPtr->obstacleType = CObstacleInstance::ABSOLUTE_OBSTACLE;
 				obstPtr->ID = obidgen.getSuchNumber(appropriateAbsoluteObstacle);
@@ -246,7 +244,7 @@ BattleInfo * BattleInfo::setupBattle(const int3 & tile, TerrainId terrain, const
 		{
 			while(tilesToBlock > 0)
 			{
-				RangeGenerator obidgen(0, VLC->obstacleHandler->size() - 1, ourRand);
+				RangeGenerator obidgen(0, LIBRARY->obstacleHandler->size() - 1, ourRand);
 				auto tileAccessibility = currentBattle->getAccessibility();
 				const int obid = obidgen.getSuchNumber(appropriateUsualObstacle);
 				const ObstacleInfo &obi = *Obstacle(obid).getInfo();
@@ -345,39 +343,41 @@ BattleInfo * BattleInfo::setupBattle(const int3 & tile, TerrainId terrain, const
 
 			if (pos.isValid())
 				currentBattle->generateNewStack(currentBattle->nextUnitId(), *i->second, side, i->first, pos);
+			else
+				logMod->warn("Invalid battlefield layout! Failed to find position for unit %d for %s", k, side == BattleSide::ATTACKER ? "attacker" : "defender");
 		}
 	}
 
 	//adding commanders
 	for(BattleSide i : {BattleSide::ATTACKER, BattleSide::DEFENDER})
 	{
-		if (heroes[i] && heroes[i]->commander && heroes[i]->commander->alive)
+		if (heroes[i] && heroes[i]->getCommander() && heroes[i]->getCommander()->alive)
 		{
-			currentBattle->generateNewStack(currentBattle->nextUnitId(), *heroes[i]->commander, i, SlotID::COMMANDER_SLOT_PLACEHOLDER, layout.commanders.at(i));
+			currentBattle->generateNewStack(currentBattle->nextUnitId(), *heroes[i]->getCommander(), i, SlotID::COMMANDER_SLOT_PLACEHOLDER, layout.commanders.at(i));
 		}
 	}
 
-	if (currentBattle->town)
+	if (currentBattle->townID.hasValue())
 	{
-		if (currentBattle->town->fortificationsLevel().citadelHealth != 0)
+		if (currentBattle->getDefendedTown()->fortificationsLevel().citadelHealth != 0)
 			currentBattle->generateNewStack(currentBattle->nextUnitId(), CStackBasicDescriptor(CreatureID::ARROW_TOWERS, 1), BattleSide::DEFENDER, SlotID::ARROW_TOWERS_SLOT, BattleHex::CASTLE_CENTRAL_TOWER);
 
-		if (currentBattle->town->fortificationsLevel().upperTowerHealth != 0)
+		if (currentBattle->getDefendedTown()->fortificationsLevel().upperTowerHealth != 0)
 			currentBattle->generateNewStack(currentBattle->nextUnitId(), CStackBasicDescriptor(CreatureID::ARROW_TOWERS, 1), BattleSide::DEFENDER, SlotID::ARROW_TOWERS_SLOT, BattleHex::CASTLE_UPPER_TOWER);
 
-		if (currentBattle->town->fortificationsLevel().lowerTowerHealth != 0)
+		if (currentBattle->getDefendedTown()->fortificationsLevel().lowerTowerHealth != 0)
 			currentBattle->generateNewStack(currentBattle->nextUnitId(), CStackBasicDescriptor(CreatureID::ARROW_TOWERS, 1), BattleSide::DEFENDER, SlotID::ARROW_TOWERS_SLOT, BattleHex::CASTLE_BOTTOM_TOWER);
 
 		//Moat generating is done on server
 	}
 
-	std::stable_sort(stacks.begin(),stacks.end(),cmpst);
+	std::stable_sort(currentBattle->stacks.begin(), currentBattle->stacks.end(), [cmpst](const auto & left, const auto & right){ return cmpst(left.get(), right.get());});
 
 	auto neutral = std::make_shared<CreatureAlignmentLimiter>(EAlignment::NEUTRAL);
 	auto good = std::make_shared<CreatureAlignmentLimiter>(EAlignment::GOOD);
 	auto evil = std::make_shared<CreatureAlignmentLimiter>(EAlignment::EVIL);
 
-	const auto * bgInfo = VLC->battlefields()->getById(battlefieldType);
+	const auto * bgInfo = LIBRARY->battlefields()->getById(battlefieldType);
 
 	for(const std::shared_ptr<Bonus> & bonus : bgInfo->bonuses)
 	{
@@ -385,8 +385,10 @@ BattleInfo * BattleInfo::setupBattle(const int3 & tile, TerrainId terrain, const
 	}
 
 	//native terrain bonuses
-	auto nativeTerrain = std::make_shared<CreatureTerrainLimiter>();
-	
+	auto nativeTerrain = std::make_shared<AllOfLimiter>();
+	nativeTerrain->add(std::make_shared<TerrainLimiter>());
+	nativeTerrain->add(std::make_shared<CreatureLevelLimiter>()); // creature only limiter - exclude hero
+
 	currentBattle->addNewBonus(std::make_shared<Bonus>(BonusDuration::ONE_BATTLE, BonusType::STACKS_SPEED, BonusSource::TERRAIN_NATIVE, 1,  BonusSourceID())->addLimiter(nativeTerrain));
 	currentBattle->addNewBonus(std::make_shared<Bonus>(BonusDuration::ONE_BATTLE, BonusType::PRIMARY_SKILL, BonusSource::TERRAIN_NATIVE, 1, BonusSourceID(), BonusSubtypeID(PrimarySkill::ATTACK))->addLimiter(nativeTerrain));
 	currentBattle->addNewBonus(std::make_shared<Bonus>(BonusDuration::ONE_BATTLE, BonusType::PRIMARY_SKILL, BonusSource::TERRAIN_NATIVE, 1, BonusSourceID(), BonusSubtypeID(PrimarySkill::DEFENSE))->addLimiter(nativeTerrain));
@@ -399,7 +401,8 @@ BattleInfo * BattleInfo::setupBattle(const int3 & tile, TerrainId terrain, const
 	{
 		if(heroes[i])
 		{
-			battleRepositionHex[i] += heroes[i]->valOfBonuses(BonusType::BEFORE_BATTLE_REPOSITION);
+			if(heroes[i]->tacticFormationEnabled)
+				battleRepositionHex[i] += heroes[i]->valOfBonuses(BonusType::BEFORE_BATTLE_REPOSITION);
 			battleRepositionHexBlock[i] += heroes[i]->valOfBonuses(BonusType::BEFORE_BATTLE_REPOSITION_BLOCK);
 		}
 	}
@@ -440,7 +443,7 @@ const CGHeroInstance * BattleInfo::getHero(const PlayerColor & player) const
 {
 	for(const auto & side : sides)
 		if(side.color == player)
-			return side.hero;
+			return side.getHero();
 
 	logGlobal->error("Player %s is not in battle!", player.toString());
 	return nullptr;
@@ -461,23 +464,24 @@ CStack * BattleInfo::getStack(int stackID, bool onlyAlive)
 	return const_cast<CStack *>(battleGetStackByID(stackID, onlyAlive));
 }
 
-BattleInfo::BattleInfo(const BattleLayout & layout):
-	BattleInfo()
+BattleInfo::BattleInfo(IGameInfoCallback *cb, const BattleLayout & layout):
+	BattleInfo(cb)
 {
 	*this->layout = layout;
 }
 
-BattleInfo::BattleInfo():
+BattleInfo::BattleInfo(IGameInfoCallback *cb)
+	:CBonusSystemNode(BonusNodeType::BATTLE_WIDE),
+	GameCallbackHolder(cb),
+	sides({SideInBattle(cb), SideInBattle(cb)}),
 	layout(std::make_unique<BattleLayout>()),
 	round(-1),
 	activeStack(-1),
-	town(nullptr),
 	tile(-1,-1,-1),
 	battlefieldType(BattleField::NONE),
 	tacticsSide(BattleSide::NONE),
 	tacticDistance(0)
 {
-	setNodeType(BATTLE);
 }
 
 BattleLayout BattleInfo::getLayout() const
@@ -502,8 +506,7 @@ std::optional<PlayerColor> BattleInfo::getPlayerID() const
 
 BattleInfo::~BattleInfo()
 {
-	for (auto & elem : stacks)
-		delete elem;
+	stacks.clear();
 
 	for(auto i : {BattleSide::ATTACKER, BattleSide::DEFENDER})
 		if(auto * _armyObj = battleGetArmyObject(i))
@@ -518,14 +521,18 @@ int32_t BattleInfo::getActiveStackID() const
 TStacks BattleInfo::getStacksIf(const TStackFilter & predicate) const
 {
 	TStacks ret;
-	vstd::copy_if(stacks, std::back_inserter(ret), predicate);
+	for (const auto & stack : stacks)
+		if (predicate(stack.get()))
+			ret.push_back(stack.get());
 	return ret;
 }
 
 battle::Units BattleInfo::getUnitsIf(const battle::UnitFilter & predicate) const
 {
 	battle::Units ret;
-	vstd::copy_if(stacks, std::back_inserter(ret), predicate);
+	for (const auto & stack : stacks)
+		if (predicate(stack.get()))
+			ret.push_back(stack.get());
 	return ret;
 }
 
@@ -557,12 +564,12 @@ PlayerColor BattleInfo::getSidePlayer(BattleSide side) const
 
 const CArmedInstance * BattleInfo::getSideArmy(BattleSide side) const
 {
-	return getSide(side).armyObject;
+	return getSide(side).getArmy();
 }
 
 const CGHeroInstance * BattleInfo::getSideHero(BattleSide side) const
 {
-	return getSide(side).hero;
+	return getSide(side).getHero();
 }
 
 uint8_t BattleInfo::getTacticDist() const
@@ -575,9 +582,16 @@ BattleSide BattleInfo::getTacticsSide() const
 	return tacticsSide;
 }
 
+int32_t BattleInfo::getRound() const
+{
+	return round;
+}
+
 const CGTownInstance * BattleInfo::getDefendedTown() const
 {
-	return town;
+	if (townID.hasValue())
+		return cb->getTown(townID);
+	return nullptr;
 }
 
 EWallState BattleInfo::getWallState(EWallPart partOfWall) const
@@ -590,7 +604,7 @@ EGateState BattleInfo::getGateState() const
 	return si.gateState;
 }
 
-uint32_t BattleInfo::getCastSpells(BattleSide side) const
+int32_t BattleInfo::getCastSpells(BattleSide side) const
 {
 	return getSide(side).castSpellsCount;
 }
@@ -643,7 +657,7 @@ void BattleInfo::nextRound()
 	}
 	round += 1;
 
-	for(CStack * s : stacks)
+	for(auto & s : stacks)
 	{
 		// new turn effects
 		s->reduceBonusDurations(Bonus::NTurns);
@@ -655,16 +669,19 @@ void BattleInfo::nextRound()
 		obst->battleTurnPassed();
 }
 
-void BattleInfo::nextTurn(uint32_t unitId)
+void BattleInfo::nextTurn(uint32_t unitId, BattleUnitTurnReason reason)
 {
 	activeStack = unitId;
 
 	CStack * st = getStack(activeStack);
 
-	//remove bonuses that last until when stack gets new turn
-	st->removeBonusesRecursive(Bonus::UntilGetsTurn);
+	if (reason != BattleUnitTurnReason::UNIT_SPELLCAST)
+	{
+		//remove bonuses that last until when stack gets new turn
+		st->removeBonusesRecursive(Bonus::UntilGetsTurn);
+	}
 
-	st->afterGetsTurn();
+	st->afterGetsTurn(reason);
 }
 
 void BattleInfo::addUnit(uint32_t id, const JsonNode & data)
@@ -675,11 +692,11 @@ void BattleInfo::addUnit(uint32_t id, const JsonNode & data)
 
 	PlayerColor owner = getSidePlayer(info.side);
 
-	auto * ret = new CStack(&base, owner, info.id, info.side, SlotID::SUMMONED_SLOT_PLACEHOLDER);
+	auto ret = std::make_unique<CStack>(&base, owner, info.id, info.side, SlotID::SUMMONED_SLOT_PLACEHOLDER);
 	ret->initialPosition = info.position;
-	stacks.push_back(ret);
-	ret->localInit(this);
-	ret->summoned = info.summoned;
+	stacks.push_back(std::move(ret));
+	stacks.back()->localInit(this);
+	stacks.back()->summoned = info.summoned;
 }
 
 void BattleInfo::moveUnit(uint32_t id, const BattleHex & destination)
@@ -727,6 +744,11 @@ void BattleInfo::setUnitState(uint32_t id, const JsonNode & data, int64_t health
 		changedStack->removeBonusesRecursive(Bonus::UntilBeingAttacked);
 	}
 
+	if(healthDelta < 0)
+	{
+		changedStack->nodeHasChanged();	//bonuses with TIMES_STACK_SIZE updater may change
+	}
+
 	resurrected = resurrected || (killed && changedStack->alive());
 
 	if(killed)
@@ -748,14 +770,14 @@ void BattleInfo::setUnitState(uint32_t id, const JsonNode & data, int64_t health
 		auto selector = [](const Bonus * b)
 		{
 			//Special case: DISRUPTING_RAY is absolutely permanent
-			return b->source == BonusSource::SPELL_EFFECT && b->sid.as<SpellID>() != SpellID::DISRUPTING_RAY;
+			return b->source == BonusSource::SPELL_EFFECT && b->sid.as<SpellID>().toSpell()->isPersistent();
 		};
 		changedStack->removeBonusesRecursive(selector);
 	}
 
 	if(!changedStack->alive() && changedStack->isClone())
 	{
-		for(CStack * s : stacks)
+		for(auto & s : stacks)
 		{
 			if(s->cloneID == changedStack->unitId())
 				s->cloneID = -1;
@@ -793,7 +815,7 @@ void BattleInfo::removeUnit(uint32_t id)
 			}
 
 			//cleanup remaining clone links if any
-			for(auto * s : stacks)
+			for(const auto & s : stacks)
 			{
 				if(s->cloneID == toRemoveId)
 					s->cloneID = -1;
@@ -859,7 +881,6 @@ void BattleInfo::removeUnitBonus(uint32_t id, const std::vector<Bonus> & bonus)
 			&& one.val == b->val
 			&& one.sid == b->sid
 			&& one.valType == b->valType
-			&& one.additionalInfo == b->additionalInfo
 			&& one.effectRange == b->effectRange;
 		};
 		sta->removeBonusesRecursive(selector);
@@ -948,6 +969,12 @@ CGHeroInstance * BattleInfo::battleGetFightingHero(BattleSide side) const
 	return const_cast<CGHeroInstance*>(CBattleInfoEssentials::battleGetFightingHero(side));
 }
 
+void BattleInfo::postDeserialize()
+{
+	for (const auto & unit : stacks)
+		unit->postDeserialize(getSideArmy(unit->unitSide()));
+}
+
 #if SCRIPTING_ENABLED
 scripting::Pool * BattleInfo::getContextPool() const
 {
