@@ -10,17 +10,17 @@
 
 #include "StdInc.h"
 #include "Bonus.h"
+#include "BonusParameters.h"
 #include "Limiters.h"
 #include "Updaters.h"
 #include "Propagators.h"
 
-#include "../CArtHandler.h"
+#include "../CBonusTypeHandler.h"
 #include "../CCreatureHandler.h"
-#include "../CCreatureSet.h"
 #include "../CSkillHandler.h"
-#include "../IGameCallback.h"
 #include "../TerrainHandler.h"
-#include "../VCMI_Lib.h"
+#include "../GameLibrary.h"
+#include "../callback/IGameInfoCallback.h"
 #include "../mapObjects/CGObjectInstance.h"
 #include "../mapObjectConstructors/CObjectClassesHandler.h"
 #include "../battle/BattleInfo.h"
@@ -32,64 +32,6 @@
 
 VCMI_LIB_NAMESPACE_BEGIN
 
-//This constructor should be placed here to avoid side effects
-CAddInfo::CAddInfo() = default;
-
-CAddInfo::CAddInfo(si32 value)
-{
-	if(value != CAddInfo::NONE)
-		push_back(value);
-}
-
-bool CAddInfo::operator==(si32 value) const
-{
-	switch(size())
-	{
-	case 0:
-		return value == CAddInfo::NONE;
-	case 1:
-		return operator[](0) == value;
-	default:
-		return false;
-	}
-}
-
-bool CAddInfo::operator!=(si32 value) const
-{
-	return !operator==(value);
-}
-
-si32 & CAddInfo::operator[](size_type pos)
-{
-	if(pos >= size())
-		resize(pos + 1, CAddInfo::NONE);
-	return vector::operator[](pos);
-}
-
-si32 CAddInfo::operator[](size_type pos) const
-{
-	return pos < size() ? vector::operator[](pos) : CAddInfo::NONE;
-}
-
-std::string CAddInfo::toString() const
-{
-	return toJsonNode().toCompactString();
-}
-
-JsonNode CAddInfo::toJsonNode() const
-{
-	if(size() < 2)
-	{
-		return JsonNode(operator[](0));
-	}
-	else
-	{
-		JsonNode node;
-		for(si32 value : *this)
-			node.Vector().emplace_back(value);
-		return node;
-	}
-}
 std::string Bonus::Description(const IGameInfoCallback * cb, std::optional<si32> customValue) const
 {
 	MetaString descriptionHelper = description;
@@ -110,15 +52,15 @@ std::string Bonus::Description(const IGameInfoCallback * cb, std::optional<si32>
 				descriptionHelper.appendNamePlural(sid.as<CreatureID>());
 				break;
 			case BonusSource::SECONDARY_SKILL:
-				descriptionHelper.appendTextID(sid.as<SecondarySkill>().toEntity(VLC)->getNameTextID());
+				descriptionHelper.appendTextID(sid.as<SecondarySkill>().toEntity(LIBRARY)->getNameTextID());
 				break;
 			case BonusSource::HERO_SPECIAL:
-				descriptionHelper.appendTextID(sid.as<HeroTypeID>().toEntity(VLC)->getNameTextID());
+				descriptionHelper.appendTextID(sid.as<HeroTypeID>().toEntity(LIBRARY)->getNameTextID());
 				break;
 			case BonusSource::OBJECT_INSTANCE:
 				const auto * object = cb->getObj(sid.as<ObjectInstanceID>());
 				if (object)
-					descriptionHelper.appendTextID(VLC->objtypeh->getObjectName(object->ID, object->subID));
+					descriptionHelper.appendTextID(LIBRARY->objtypeh->getObjectName(object->ID, object->subID));
 		}
 	}
 
@@ -149,10 +91,7 @@ std::string Bonus::Description(const IGameInfoCallback * cb, std::optional<si32>
 		// there is one known string that uses '%s' placeholder for bonus value:
 		// "core.arraytxt.69" : "\nFountain of Fortune Visited %s",
 		// So also add string replacement to handle this case
-		if (valueToShow > 0)
-			descriptionHelper.replaceRawString(std::to_string(valueToShow));
-		else
-			descriptionHelper.replaceRawString("-" + std::to_string(valueToShow));
+		descriptionHelper.replaceRawString(std::to_string(valueToShow));
 
 		if(type == BonusType::CREATURE_GROWTH_PERCENT)
 			descriptionHelper.appendRawString(" +" + std::to_string(valueToShow));
@@ -161,26 +100,15 @@ std::string Bonus::Description(const IGameInfoCallback * cb, std::optional<si32>
 	return descriptionHelper.toString();
 }
 
-static JsonNode additionalInfoToJson(BonusType type, CAddInfo addInfo)
-{
-	switch(type)
-	{
-	case BonusType::SPECIAL_UPGRADE:
-		return JsonNode(ModUtility::makeFullIdentifier("", "creature", CreatureID::encode(addInfo[0])));
-	default:
-		return addInfo.toJsonNode();
-	}
-}
-
 JsonNode Bonus::toJsonNode() const
 {
 	JsonNode root;
 	// only add values that might reasonably be found in config files
-	root["type"].String() = vstd::findKey(bonusNameMap, type);
+	root["type"].String() = LIBRARY->bth->bonusToString(type);
 	if(subtype != BonusSubtypeID())
 		root["subtype"].String() = subtype.toString();
-	if(additionalInfo != CAddInfo::NONE)
-		root["addInfo"] = additionalInfoToJson(type, additionalInfo);
+	if(parameters)
+		root["addInfo"] = parameters->toJsonNode();
 	if(source != BonusSource::OTHER)
 		root["sourceType"].String() = vstd::findKey(bonusSourceMap, source);
 	if(targetSourceType != BonusSource::OTHER)
@@ -207,9 +135,27 @@ JsonNode Bonus::toJsonNode() const
 		root["updater"] = updater->toJsonNode();
 	if(propagator)
 		root["propagator"].String() = vstd::findKey(bonusPropagatorMap, propagator);
+	if(hidden)
+		root["hidden"].Bool() = hidden;
 	return root;
 }
 
+void Bonus::convertAddInfo(const std::vector<int> & oldAddInfo)
+{
+	if (oldAddInfo.size() > 1)
+		parameters = std::make_shared<BonusParameters>(oldAddInfo);
+
+	if (oldAddInfo.size() == 1 && oldAddInfo[0] != -1)
+	{
+		if (type == BonusType::SPECIAL_UPGRADE || type == BonusType::TRANSMUTATION)
+			parameters = std::make_shared<BonusParameters>(CreatureID(oldAddInfo[0]));
+		else if (type == BonusType::DEATH_STARE)
+			parameters = std::make_shared<BonusParameters>(SpellID(oldAddInfo[0]));
+		else
+			parameters = std::make_shared<BonusParameters>(oldAddInfo[0]);
+	}
+}
+
 Bonus::Bonus(BonusDuration::Type Duration, BonusType Type, BonusSource Src, si32 Val, BonusSourceID ID)
 	: Bonus(Duration, Type, Src, Val, ID, BonusSubtypeID())
 {}
@@ -239,6 +185,12 @@ Bonus::Bonus(BonusDuration::Type Duration, BonusType Type, BonusSource Src, si32
 	targetSourceType = BonusSource::OTHER;
 }
 
+Bonus::Bonus(const Bonus & inst, const BonusSourceID & sourceId)
+	: Bonus(inst)
+{
+	sid = sourceId;
+}
+
 std::shared_ptr<Bonus> Bonus::addPropagator(const TPropagatorPtr & Propagator)
 {
 	propagator = Propagator;
@@ -247,9 +199,7 @@ std::shared_ptr<Bonus> Bonus::addPropagator(const TPropagatorPtr & Propagator)
 
 DLL_LINKAGE std::ostream & operator<<(std::ostream &out, const Bonus &bonus)
 {
-	for(const auto & i : bonusNameMap)
-	if(i.second == bonus.type)
-		out << "\tType: " << i.first << " \t";
+	out << "\tType: " << LIBRARY->bth->bonusToString(bonus.type) << " \t";
 
 #define printField(field) out << "\t" #field ": " << (int)bonus.field << "\n"
 	printField(val);
@@ -257,8 +207,8 @@ DLL_LINKAGE std::ostream & operator<<(std::ostream &out, const Bonus &bonus)
 	printField(duration);
 	printField(source);
 	out << "\tSource ID: " << bonus.sid.toString() << "\n";
-	if(bonus.additionalInfo != CAddInfo::NONE)
-		out << "\taddInfo: " << bonus.additionalInfo.toString() << "\n";
+	if(bonus.parameters)
+		out << "\taddInfo: " << bonus.parameters->toString() << "\n";
 	printField(turnsRemain);
 	printField(valType);
 	if(!bonus.stacking.empty())
@@ -278,17 +228,14 @@ std::shared_ptr<Bonus> Bonus::addLimiter(const TLimiterPtr & Limiter)
 {
 	if (limiter)
 	{
-		//If we already have limiter list, retrieve it
-		auto limiterList = std::dynamic_pointer_cast<AllOfLimiter>(limiter);
-		if(!limiterList)
-		{
-			//Create a new limiter list with old limiter and the new one will be pushed later
-			limiterList = std::make_shared<AllOfLimiter>();
-			limiterList->add(limiter);
-			limiter = limiterList;
-		}
+		auto newLimiterList = std::make_shared<AllOfLimiter>();
+		auto oldLimiterList = std::dynamic_pointer_cast<const AllOfLimiter>(limiter);
 
-		limiterList->add(Limiter);
+		if(oldLimiterList)
+			newLimiterList->limiters = oldLimiterList->limiters;
+
+		newLimiterList->add(Limiter);
+		limiter = newLimiterList;
 	}
 	else
 	{
@@ -299,9 +246,30 @@ std::shared_ptr<Bonus> Bonus::addLimiter(const TLimiterPtr & Limiter)
 
 // Updaters
 
-std::shared_ptr<Bonus> Bonus::addUpdater(const TUpdaterPtr & Updater)
+static TUpdaterPtr appendToUpdaters(const TUpdaterPtr & current, const TUpdaterPtr & added)
 {
-	updater = Updater;
+	if (!current)
+		return added;
+
+	auto newList = std::make_shared<CompositeUpdater>();
+	auto oldList = std::dynamic_pointer_cast<const CompositeUpdater>(current);
+
+	if(oldList)
+		newList->updaters = oldList->updaters;
+
+	newList->updaters.push_back(added);
+	return newList;
+}
+
+std::shared_ptr<Bonus> Bonus::addUpdater(const TUpdaterPtr & newUpdater)
+{
+	updater = appendToUpdaters(updater, newUpdater);
+	return this->shared_from_this();
+}
+
+std::shared_ptr<Bonus> Bonus::addPropagationUpdater(const TUpdaterPtr & newUpdater)
+{
+	propagationUpdater = appendToUpdaters(propagationUpdater, newUpdater);
 	return this->shared_from_this();
 }
 
