@@ -26,7 +26,7 @@ void BonusList::stackBonuses()
 		COMPARE_ATT(subtype);
 		COMPARE_ATT(valType);
 #undef COMPARE_ATT
-		return b1->val > b2->val;
+		return std::abs(b1->val) > std::abs(b2->val);
 	});
 	// remove non-stacking
 	size_t next = 1;
@@ -69,7 +69,14 @@ int BonusList::totalValue(int baseValue) const
 		int indepMax = std::numeric_limits<int>::min();
 	};
 
-	auto applyPercentage = [](int base, int percent) -> int {
+	auto applyPercentageRoundUp = [](int base, int percent) -> int {
+		if (base >= 0)
+			return (static_cast<int64_t>(base) * (100 + percent) + 99) / 100;
+		else
+			return (static_cast<int64_t>(base) * (100 + percent) - 99) / 100;
+	};
+
+	auto applyPercentageRoundDown = [](int base, int percent) -> int {
 		return (static_cast<int64_t>(base) * (100 + percent)) / 100;
 	};
 
@@ -96,7 +103,11 @@ int BonusList::totalValue(int baseValue) const
 	for(const auto & b : bonuses)
 	{
 		int sourceIndex = vstd::to_underlying(b->source);
-		int valModified	= applyPercentage(b->val, percentToSource[sourceIndex]);
+		// Workaround: creature hero specialties in H3 is the only place that uses rounding up in bonuses
+		// TODO: try to find more elegant solution?
+		int valModified	= b->source == BonusSource::CREATURE_ABILITY ?
+			applyPercentageRoundUp(b->val, percentToSource[sourceIndex]):
+			applyPercentageRoundDown(b->val, percentToSource[sourceIndex]);
 
 		switch(b->valType)
 		{
@@ -123,9 +134,9 @@ int BonusList::totalValue(int baseValue) const
 		}
 	}
 
-	accumulated.base = applyPercentage(accumulated.base, accumulated.percentToBase);
+	accumulated.base = applyPercentageRoundDown(accumulated.base, accumulated.percentToBase);
 	accumulated.base += accumulated.additive;
-	auto valFirst = applyPercentage(accumulated.base ,accumulated.percentToAll);
+	auto valFirst = applyPercentageRoundDown(accumulated.base ,accumulated.percentToAll);
 
 	if(indexMinCount && indexMaxCount && accumulated.indepMin < accumulated.indepMax)
 		accumulated.indepMax = accumulated.indepMin;
@@ -164,11 +175,11 @@ std::shared_ptr<const Bonus> BonusList::getFirst(const CSelector &selector) cons
 	return nullptr;
 }
 
-void BonusList::getBonuses(BonusList & out, const CSelector &selector, const CSelector &limit) const
+void BonusList::getBonuses(BonusList & out, const CSelector &selector) const
 {
 	for(const auto & b : bonuses)
 	{
-		if(selector(b.get()) && (!limit || ((bool)limit && limit(b.get()))))
+		if(selector(b.get()))
 			out.push_back(b);
 	}
 }
@@ -182,8 +193,7 @@ void BonusList::getAllBonuses(BonusList &out) const
 int BonusList::valOfBonuses(const CSelector &select, int baseValue) const
 {
 	BonusList ret;
-	CSelector limit = nullptr;
-	getBonuses(ret, select, limit);
+	getBonuses(ret, select);
 	return ret.totalValue(baseValue);
 }
 
