@@ -28,7 +28,7 @@ class JsonSerializeFormat;
  * Font: .fnt
  * Image: .bmp, .jpg, .pcx, .png, .tga
  * Sound: .wav .82m
- * Video: .smk, .bik .mjpg .mpg .webm
+ * Video: .smk, .bik .ogv .webm
  * Music: .mp3, .ogg
  * Archive: .lod, .snd, .vid .pac .zip
  * Palette: .pal
@@ -46,11 +46,13 @@ enum class EResType
 	TTF_FONT,
 	IMAGE,
 	VIDEO,
+	VIDEO_LOW_QUALITY,
 	SOUND,
 	ARCHIVE_VID,
 	ARCHIVE_ZIP,
 	ARCHIVE_SND,
 	ARCHIVE_LOD,
+	ARCHIVE_PAK,
 	PALETTE,
 	SAVEGAME,
 	DIRECTORY,
@@ -58,6 +60,7 @@ enum class EResType
 	ERT,
 	ERS,
 	LUA,
+	AI_MODEL,
 	OTHER,
 	UNDEFINED,
 };
