@@ -9,7 +9,7 @@
  */
 #pragma once
 
-#include "MetaString.h"
+#include "../texts/MetaString.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
