@@ -18,8 +18,13 @@
 VCMI_LIB_NAMESPACE_BEGIN
 
 JsonParser::JsonParser(const std::byte * inputString, size_t stringSize, const JsonParsingSettings & settings)
+    : JsonParser(reinterpret_cast<const char*>(inputString), stringSize, settings) // NOSONAR
+{
+}
+
+JsonParser::JsonParser(const char * inputString, size_t stringSize, const JsonParsingSettings & settings)
 	: settings(settings)
-	, input(reinterpret_cast<const char *>(inputString), stringSize)
+	, input(inputString, stringSize)
 	, lineCount(1)
 	, currentDepth(0)
 	, lineStart(0)
@@ -423,7 +428,7 @@ bool JsonParser::extractStruct(JsonNode & node)
 bool JsonParser::extractArray(JsonNode & node)
 {
 	if(currentDepth > settings.maxDepth)
-		error("Macimum allowed depth of json structure has been reached", true);
+		error("Maximum allowed depth of json structure has been reached", true);
 
 	currentDepth++;
 	auto guard = vstd::makeScopeGuard([this]()
