@@ -8,23 +8,24 @@
  *
  */
 #include "StdInc.h"
+#include "CConfigHandler.h"
 #include "CObjectClassesHandler.h"
 
 #include "../filesystem/Filesystem.h"
 #include "../filesystem/CBinaryReader.h"
 #include "../json/JsonUtils.h"
-#include "../VCMI_Lib.h"
+#include "../GameLibrary.h"
 #include "../GameConstants.h"
 #include "../constants/StringConstants.h"
 #include "../IGameSettings.h"
 #include "../CSoundBase.h"
 
-#include "../mapObjectConstructors/CBankInstanceConstructor.h"
 #include "../mapObjectConstructors/CRewardableConstructor.h"
 #include "../mapObjectConstructors/CommonConstructors.h"
 #include "../mapObjectConstructors/DwellingInstanceConstructor.h"
 #include "../mapObjectConstructors/FlaggableInstanceConstructor.h"
 #include "../mapObjectConstructors/HillFortInstanceConstructor.h"
+#include "../mapObjectConstructors/MarketInstanceConstructor.h"
 #include "../mapObjectConstructors/ShipyardInstanceConstructor.h"
 
 #include "../mapObjects/CGCreature.h"
@@ -59,7 +60,6 @@ CObjectClassesHandler::CObjectClassesHandler()
 	SET_HANDLER_CLASS("dwelling", DwellingInstanceConstructor);
 	SET_HANDLER_CLASS("hero", CHeroInstanceConstructor);
 	SET_HANDLER_CLASS("town", CTownInstanceConstructor);
-	SET_HANDLER_CLASS("bank", CBankInstanceConstructor);
 	SET_HANDLER_CLASS("boat", BoatInstanceConstructor);
 	SET_HANDLER_CLASS("flaggable", FlaggableInstanceConstructor);
 	SET_HANDLER_CLASS("market", MarketInstanceConstructor);
@@ -67,6 +67,7 @@ CObjectClassesHandler::CObjectClassesHandler()
 	SET_HANDLER_CLASS("shipyard", ShipyardInstanceConstructor);
 	SET_HANDLER_CLASS("monster", CreatureInstanceConstructor);
 	SET_HANDLER_CLASS("resource", ResourceInstanceConstructor);
+	SET_HANDLER_CLASS("mine", MineInstanceConstructor);
 
 	SET_HANDLER_CLASS("static", CObstacleConstructor);
 	SET_HANDLER_CLASS("", CObstacleConstructor);
@@ -88,7 +89,6 @@ CObjectClassesHandler::CObjectClassesHandler()
 	SET_HANDLER("heroPlaceholder", CGHeroPlaceholder);
 	SET_HANDLER("keymaster", CGKeymasterTent);
 	SET_HANDLER("magi", CGMagi);
-	SET_HANDLER("mine", CGMine);
 	SET_HANDLER("obelisk", CGObelisk);
 	SET_HANDLER("pandora", CGPandoraBox);
 	SET_HANDLER("prison", CGHeroInstance);
@@ -109,7 +109,7 @@ CObjectClassesHandler::~CObjectClassesHandler() = default;
 
 std::vector<JsonNode> CObjectClassesHandler::loadLegacyData()
 {
-	size_t dataSize = VLC->engineSettings()->getInteger(EGameSettings::TEXTS_OBJECT);
+	size_t dataSize = LIBRARY->engineSettings()->getInteger(EGameSettings::TEXTS_OBJECT);
 
 	CLegacyConfigParser parser(TextPath::builtin("Data/Objects.txt"));
 	auto totalNumber = static_cast<size_t>(parser.readNumber()); // first line contains number of objects to read and nothing else
@@ -175,9 +175,7 @@ void CObjectClassesHandler::loadSubObject(const std::string & scope, const std::
 	assert(subObject);
 	baseObject->objectTypeHandlers.push_back(subObject);
 
-	registerObject(scope, baseObject->getJsonKey(), subObject->getSubTypeName(), subObject->subtype);
-	for(const auto & compatID : entry["compatibilityIdentifiers"].Vector())
-		registerObject(scope, baseObject->getJsonKey(), compatID.String(), subObject->subtype);
+	registerObject(scope, baseObject->getJsonKey(), subObject->getSubTypeName(), entry, subObject->subtype);
 }
 
 void CObjectClassesHandler::loadSubObject(const std::string & scope, const std::string & identifier, const JsonNode & entry, ObjectClass * baseObject, size_t index)
@@ -190,16 +188,25 @@ void CObjectClassesHandler::loadSubObject(const std::string & scope, const std::
 
 	baseObject->objectTypeHandlers.at(index) = subObject;
 
-	registerObject(scope, baseObject->getJsonKey(), subObject->getSubTypeName(), subObject->subtype);
-	for(const auto & compatID : entry["compatibilityIdentifiers"].Vector())
-		registerObject(scope, baseObject->getJsonKey(), compatID.String(), subObject->subtype);
+	registerObject(scope, baseObject->getJsonKey(), subObject->getSubTypeName(), entry, subObject->subtype);
 }
 
 TObjectTypeHandler CObjectClassesHandler::loadSubObjectFromJson(const std::string & scope, const std::string & identifier, const JsonNode & entry, ObjectClass * baseObject, size_t index)
 {
-	assert(identifier.find(':') == std::string::npos);
 	assert(!scope.empty());
 
+	if (settings["mods"]["validation"].String() != "off")
+	{
+		size_t separator = identifier.find(':');
+
+		if (separator != std::string::npos)
+		{
+			std::string modName = identifier.substr(0, separator);
+			std::string objectName = identifier.substr(separator + 1);
+			logMod->warn("Mod %s: Map object type with format '%s' will add new map object, not modify it! Please use '%s' form and add dependency on mod '%s' instead!", scope, identifier, modName, identifier );
+		}
+	}
+
 	std::string handler = baseObject->handlerName;
 	if(!handlerConstructors.count(handler))
 	{
@@ -209,16 +216,6 @@ TObjectTypeHandler CObjectClassesHandler::loadSubObjectFromJson(const std::strin
 		assert(handlerConstructors.count(handler) != 0);
 	}
 
-	// Compatibility with 1.5 mods for 1.6. To be removed in 1.7
-	// Detect banks that use old format and load them using old bank hander
-	if (baseObject->id == Obj::CREATURE_BANK)
-	{
-		if (entry.Struct().count("levels") && !entry.Struct().count("rewards"))
-			handler = "bank";
-		else
-			handler = "configurable";
-	}
-
 	auto createdObject = handlerConstructors.at(handler)();
 
 	createdObject->modScope = scope;
@@ -235,7 +232,7 @@ TObjectTypeHandler CObjectClassesHandler::loadSubObjectFromJson(const std::strin
 		for (auto & templ : createdObject->getTemplates())
 		{
 			// Register templates for new objects from mods
-			VLC->biomeHandler->addTemplate(scope, templ->stringID, templ);
+			LIBRARY->biomeHandler->addTemplate(scope, templ->stringID, templ);
 		}
 	}
 
@@ -246,7 +243,7 @@ TObjectTypeHandler CObjectClassesHandler::loadSubObjectFromJson(const std::strin
 		{
 			// Register legacy templates as "core"
 			// FIXME: Why does it clear stringID?
-			VLC->biomeHandler->addTemplate("core", templ.second->stringID, templ.second);
+			LIBRARY->biomeHandler->addTemplate("core", templ.second->stringID, templ.second);
 		}
 
 		createdObject->addTemplate(templ.second);
@@ -274,7 +271,7 @@ std::string ObjectClass::getNameTextID() const
 
 std::string ObjectClass::getNameTranslated() const
 {
-	return VLC->generaltexth->translate(getNameTextID());
+	return LIBRARY->generaltexth->translate(getNameTextID());
 }
 
 std::unique_ptr<ObjectClass> CObjectClassesHandler::loadFromJson(const std::string & scope, const JsonNode & json, const std::string & name, size_t index)
@@ -287,7 +284,7 @@ std::unique_ptr<ObjectClass> CObjectClassesHandler::loadFromJson(const std::stri
 	newObject->base = json["base"];
 	newObject->id = index;
 
-	VLC->generaltexth->registerString(scope, newObject->getNameTextID(), json["name"]);
+	LIBRARY->generaltexth->registerString(scope, newObject->getNameTextID(), json["name"]);
 
 	newObject->objectTypeHandlers.resize(json["lastReservedIndex"].Float() + 1);
 
@@ -322,7 +319,7 @@ void CObjectClassesHandler::loadObject(std::string scope, std::string name, cons
 {
 	mapObjectTypes.push_back(loadFromJson(scope, data, name, mapObjectTypes.size()));
 
-	VLC->identifiersHandler->registerObject(scope, "object", name, mapObjectTypes.back()->id);
+	LIBRARY->identifiersHandler->registerObject(scope, "object", name, mapObjectTypes.back()->id);
 }
 
 void CObjectClassesHandler::loadObject(std::string scope, std::string name, const JsonNode & data, size_t index)
@@ -330,7 +327,7 @@ void CObjectClassesHandler::loadObject(std::string scope, std::string name, cons
 	assert(mapObjectTypes.at(index) == nullptr); // ensure that this id was not loaded before
 
 	mapObjectTypes.at(index) = loadFromJson(scope, data, name, index);
-	VLC->identifiersHandler->registerObject(scope, "object", name, mapObjectTypes.at(index)->id);
+	LIBRARY->identifiersHandler->registerObject(scope, "object", name, mapObjectTypes.at(index)->id);
 }
 
 void CObjectClassesHandler::loadSubObject(const std::string & identifier, JsonNode config, MapObjectID ID, MapObjectSubID subID)
@@ -345,6 +342,12 @@ void CObjectClassesHandler::loadSubObject(const std::string & identifier, JsonNo
 	}
 
 	JsonUtils::inherit(config, mapObjectTypes.at(ID.getNum())->base);
+	for (auto & templ : config["templates"].Struct())
+		JsonUtils::inherit(templ.second, config["base"]);
+
+	if (settings["mods"]["validation"].String() != "off")
+		JsonUtils::validate(config, "vcmi:objectType", identifier);
+
 	loadSubObject(config.getModScope(), identifier, config, mapObjectTypes.at(ID.getNum()).get(), subID.getNum());
 }
 
@@ -381,19 +384,19 @@ TObjectTypeHandler CObjectClassesHandler::getHandlerFor(MapObjectID type, MapObj
 
 TObjectTypeHandler CObjectClassesHandler::getHandlerFor(const std::string & scope, const std::string & type, const std::string & subtype) const
 {
-	std::optional<si32> id = VLC->identifiers()->getIdentifier(scope, "object", type);
+	std::optional<si32> id = LIBRARY->identifiers()->getIdentifier(scope, "object", type);
 	if(id)
 	{
 		const auto & object = mapObjectTypes.at(id.value());
-		std::optional<si32> subID = VLC->identifiers()->getIdentifier(scope, object->getJsonKey(), subtype);
+		std::optional<si32> subID = LIBRARY->identifiers()->getIdentifier(scope, object->getJsonKey(), subtype);
 
 		if (subID)
 			return object->objectTypeHandlers.at(subID.value());
 	}
 
-	std::string errorString = "Failed to find object of type " + type + "::" + subtype;
-	logGlobal->error(errorString);
-	throw std::runtime_error(errorString);
+	std::string objectType = type + "::" + subtype;
+	logGlobal->error("Failed to find object of type %s", objectType);
+	throw IdentifierResolutionException(objectType);
 }
 
 TObjectTypeHandler CObjectClassesHandler::getHandlerFor(CompoundMapObjectID compoundIdentifier) const
@@ -406,20 +409,20 @@ CompoundMapObjectID CObjectClassesHandler::getCompoundIdentifier(const std::stri
 	std::optional<si32> id;
 	if (scope.empty())
 	{
-		id = VLC->identifiers()->getIdentifier("object", type);
+		id = LIBRARY->identifiers()->getIdentifier("object", type);
 	}
 	else
 	{
-		id = VLC->identifiers()->getIdentifier(scope, "object", type);
+		id = LIBRARY->identifiers()->getIdentifier(scope, "object", type);
 	}
 
 	if(id)
 	{
 		if (subtype.empty())
-			return CompoundMapObjectID(id.value(), 0);
+			return CompoundMapObjectID(id.value(), -1);
 
 		const auto & object = mapObjectTypes.at(id.value());
-		std::optional<si32> subID = VLC->identifiers()->getIdentifier(scope, object->getJsonKey(), subtype);
+		std::optional<si32> subID = LIBRARY->identifiers()->getIdentifier(scope, object->getJsonKey(), subtype);
 
 		if (subID)
 			return CompoundMapObjectID(id.value(), subID.value());
@@ -432,7 +435,7 @@ CompoundMapObjectID CObjectClassesHandler::getCompoundIdentifier(const std::stri
 
 CompoundMapObjectID CObjectClassesHandler::getCompoundIdentifier(const std::string & objectName) const
 {
-	std::string subtype = "object"; //Default for objects with no subIds
+	std::string subtype;
 	std::string type;
 
 	auto scopeAndFullName = vstd::splitStringToPair(objectName, ':');
@@ -547,6 +550,9 @@ void CObjectClassesHandler::generateExtraMonolithsForRMG(ObjectClass * container
 	//FIXME: Monoliths  in this vector can be already not useful for every terrain
 	const size_t portalCount = portalVec.size();
 
+	if (portalCount == 0)
+		return;
+
 	//Invalid portals will be skipped and portalVec size stays unchanged
 	for (size_t i = portalCount; portalVec.size() < 100; ++i)
 	{
@@ -564,20 +570,20 @@ void CObjectClassesHandler::generateExtraMonolithsForRMG(ObjectClass * container
 		newPortal->templates = portal->getTemplates();
 		newPortal->sounds = portal->getSounds();
 		newPortal->aiValue = portal->getAiValue();
-		newPortal->battlefield = portal->battlefield; //getter is not initialized at this point
+		newPortal->battlefields = portal->battlefields; //getter is not initialized at this point
 		newPortal->modScope = portal->modScope; //private
 		newPortal->typeName = portal->getTypeName(); 
-		newPortal->subTypeName = std::string("monolith") + std::to_string(portalVec.size());
+		newPortal->subTypeName = std::string("monolith") + std::to_string(portalVec.size() + 1);
 		newPortal->type = portal->getIndex();
 
 		// Inconsintent original indexing: monolith1 has index 0
-		newPortal->subtype = portalVec.size() - 1; //indexes must be unique, they are returned as a set
+		newPortal->subtype = portalVec.size(); //indexes must be unique, they are returned as a set
 		newPortal->blockVisit = portal->blockVisit;
 		newPortal->removable = portal->removable;
 
 		portalVec.push_back(newPortal);
 
-		registerObject(newPortal->modScope, container->getJsonKey(), newPortal->subTypeName, newPortal->subtype);
+		registerObject(newPortal->modScope, container->getJsonKey(), newPortal->subTypeName, JsonNode(), newPortal->subtype);
 	}
 }
 
