@@ -24,20 +24,23 @@ class DwellingInstanceConstructor : public CDefaultObjectTypeHandler<CGDwelling>
 
 	JsonNode guards;
 	bool bannedForRandomDwelling = false;
+	AnimationPath kingdomOverviewImage;
 
 protected:
 	bool objectFilter(const CGObjectInstance * obj, std::shared_ptr<const ObjectTemplate> tmpl) const override;
 	void initTypeData(const JsonNode & input) override;
+	void onTemplateAdded(const std::shared_ptr<const ObjectTemplate>) override;
 
 public:
 	bool hasNameTextID() const override;
 
 	void initializeObject(CGDwelling * object) const override;
-	void randomizeObject(CGDwelling * object, CRandomGenerator & rng) const override;
+	void randomizeObject(CGDwelling * object, IGameRandomizer & gameRandomizer) const override;
 
 	bool isBannedForRandomDwelling() const;
 	bool producesCreature(const CCreature * crea) const;
 	std::vector<const CCreature *> getProducedCreatures() const;
+	AnimationPath getKingdomOverviewImage() const;
 };
 
 VCMI_LIB_NAMESPACE_END
