@@ -10,6 +10,7 @@
 #include "StdInc.h"
 #include "HillFortInstanceConstructor.h"
 
+#include "../GameLibrary.h"
 #include "../mapObjects/MiscObjects.h"
 #include "../texts/CGeneralTextHandler.h"
 
@@ -19,9 +20,9 @@ void HillFortInstanceConstructor::initTypeData(const JsonNode & config)
 {
 	parameters = config;
 	if(!parameters["unavailableUpgradeMessage"].isNull())
-		VLC->generaltexth->registerString(parameters.getModScope(), TextIdentifier(getBaseTextID(), "unavailableUpgradeMessage"), parameters["unavailableUpgradeMessage"].String());
+		LIBRARY->generaltexth->registerString(parameters.getModScope(), TextIdentifier(getBaseTextID(), "unavailableUpgradeMessage"), parameters["unavailableUpgradeMessage"].String());
 	
-	VLC->generaltexth->registerString(parameters.getModScope(), TextIdentifier(getBaseTextID(), "description"), parameters["description"].String());
+	LIBRARY->generaltexth->registerString(parameters.getModScope(), TextIdentifier(getBaseTextID(), "description"), parameters["description"].String());
 }
 
 void HillFortInstanceConstructor::initializeObject(HillFort * fort) const
