@@ -10,6 +10,7 @@
 #include "StdInc.h"
 #include "ShipyardInstanceConstructor.h"
 
+#include "../GameLibrary.h"
 #include "../mapObjects/MiscObjects.h"
 #include "../modding/IdentifierStorage.h"
 
@@ -22,7 +23,7 @@ void ShipyardInstanceConstructor::initTypeData(const JsonNode & config)
 
 void ShipyardInstanceConstructor::initializeObject(CGShipyard * shipyard) const
 {
-	shipyard->createdBoat = BoatId(*VLC->identifiers()->getIdentifier("core:boat", parameters["boat"]));
+	shipyard->createdBoat = BoatId(*LIBRARY->identifiers()->getIdentifier("core:boat", parameters["boat"]));
 }
 
 VCMI_LIB_NAMESPACE_END
