@@ -10,6 +10,9 @@
 
 #include "StdInc.h"
 #include "CGDwelling.h"
+#include "../callback/IGameInfoCallback.h"
+#include "../callback/IGameEventCallback.h"
+#include "../callback/IGameRandomizer.h"
 #include "../serializer/JsonSerializeFormat.h"
 #include "../entities/faction/CTownHandler.h"
 #include "../mapping/CMap.h"
@@ -21,11 +24,11 @@
 #include "../networkPacks/StackLocation.h"
 #include "../networkPacks/PacksForClient.h"
 #include "../networkPacks/PacksForClientBattle.h"
-#include "../IGameCallback.h"
 #include "../gameState/CGameState.h"
 #include "../CPlayerState.h"
 #include "../IGameSettings.h"
 #include "../CConfigHandler.h"
+#include "../texts/CGeneralTextHandler.h"
 
 #include <vstd/RNG.h>
 
@@ -46,13 +49,17 @@ void CGDwellingRandomizationInfo::serializeJson(JsonSerializeFormat & handler)
 	}
 }
 
-CGDwelling::CGDwelling(IGameCallback *cb):
-	CArmedInstance(cb)
+CGDwelling::CGDwelling(IGameInfoCallback *cb):
+	CGDwelling(cb, BonusNodeType::ARMY)
+{}
+
+CGDwelling::CGDwelling(IGameInfoCallback *cb, BonusNodeType nodeType):
+	CArmedInstance(cb, nodeType, false)
 {}
 
 CGDwelling::~CGDwelling() = default;
 
-FactionID CGDwelling::randomizeFaction(vstd::RNG & rand)
+FactionID CGDwelling::randomizeFaction(IGameRandomizer & gameRandomizer)
 {
 	if (ID == Obj::RANDOM_DWELLING_FACTION)
 		return FactionID(subID.getNum());
@@ -66,18 +73,18 @@ FactionID CGDwelling::randomizeFaction(vstd::RNG & rand)
 
 	if (!randomizationInfo->instanceId.empty())
 	{
-		auto iter = cb->gameState()->map->instanceNames.find(randomizationInfo->instanceId);
+		auto iter = cb->gameState().getMap().instanceNames.find(randomizationInfo->instanceId);
 
-		if(iter == cb->gameState()->map->instanceNames.end())
+		if(iter == cb->gameState().getMap().instanceNames.end())
 			logGlobal->error("Map object not found: %s", randomizationInfo->instanceId);
 		linkedTown = dynamic_cast<CGTownInstance *>(iter->second.get());
 	}
 
 	if (randomizationInfo->identifier != 0)
 	{
-		for(auto & elem : cb->gameState()->map->objects)
+		for(auto & townID : cb->gameState().getMap().getAllTowns())
 		{
-			auto town = dynamic_cast<CGTownInstance*>(elem.get());
+			auto town = cb->gameState().getTown(townID);
 			if(town && town->identifier == randomizationInfo->identifier)
 			{
 				linkedTown = town;
@@ -89,7 +96,7 @@ FactionID CGDwelling::randomizeFaction(vstd::RNG & rand)
 	if (linkedTown)
 	{
 		if(linkedTown->ID==Obj::RANDOM_TOWN)
-			linkedTown->pickRandomObject(rand); //we have to randomize the castle first
+			linkedTown->pickRandomObject(gameRandomizer); //we have to randomize the castle first
 
 		assert(linkedTown->ID == Obj::TOWN);
 		if(linkedTown->ID==Obj::TOWN)
@@ -97,17 +104,17 @@ FactionID CGDwelling::randomizeFaction(vstd::RNG & rand)
 	}
 
 	if(!randomizationInfo->allowedFactions.empty())
-		return *RandomGeneratorUtil::nextItem(randomizationInfo->allowedFactions, rand);
+		return *RandomGeneratorUtil::nextItem(randomizationInfo->allowedFactions, gameRandomizer.getDefault());
 
 
 	std::vector<FactionID> potentialPicks;
 
-	for (FactionID faction(0); faction < FactionID(VLC->townh->size()); ++faction)
-		if (VLC->factions()->getById(faction)->hasTown())
+	for (FactionID faction(0); faction < FactionID(LIBRARY->townh->size()); ++faction)
+		if (LIBRARY->factions()->getById(faction)->hasTown())
 			potentialPicks.push_back(faction);
 
 	assert(!potentialPicks.empty());
-	return *RandomGeneratorUtil::nextItem(potentialPicks, rand);
+	return *RandomGeneratorUtil::nextItem(potentialPicks, gameRandomizer.getDefault());
 }
 
 int CGDwelling::randomizeLevel(vstd::RNG & rand)
@@ -127,26 +134,26 @@ int CGDwelling::randomizeLevel(vstd::RNG & rand)
 	return rand.nextInt(randomizationInfo->minLevel, randomizationInfo->maxLevel) - 1;
 }
 
-void CGDwelling::pickRandomObject(vstd::RNG & rand)
+void CGDwelling::pickRandomObject(IGameRandomizer & gameRandomizer)
 {
 	if (ID == Obj::RANDOM_DWELLING || ID == Obj::RANDOM_DWELLING_LVL || ID == Obj::RANDOM_DWELLING_FACTION)
 	{
-		FactionID faction = randomizeFaction(rand);
-		int level = randomizeLevel(rand);
+		FactionID faction = randomizeFaction(gameRandomizer);
+		int level = randomizeLevel(gameRandomizer.getDefault());
 		assert(faction != FactionID::NONE && faction != FactionID::NEUTRAL);
 		assert(level >= 0 && level <= 6);
 		randomizationInfo.reset();
 
-		CreatureID cid = (*VLC->townh)[faction]->town->creatures[level][0];
+		CreatureID cid = (*LIBRARY->townh)[faction]->town->creatures[level][0];
 
 		//NOTE: this will pick last dwelling with this creature (Mantis #900)
 		//check for block map equality is better but more complex solution
 		auto testID = [&](const Obj & primaryID) -> MapObjectSubID
 		{
-			auto dwellingIDs = VLC->objtypeh->knownSubObjects(primaryID);
+			auto dwellingIDs = LIBRARY->objtypeh->knownSubObjects(primaryID);
 			for (MapObjectSubID entry : dwellingIDs)
 			{
-				const auto * handler = dynamic_cast<const DwellingInstanceConstructor *>(VLC->objtypeh->getHandlerFor(primaryID, entry).get());
+				const auto * handler = dynamic_cast<const DwellingInstanceConstructor *>(LIBRARY->objtypeh->getHandlerFor(primaryID, entry).get());
 
 				if (!handler->isBannedForRandomDwelling() && handler->producesCreature(cid.toCreature()))
 					return MapObjectSubID(entry);
@@ -165,16 +172,16 @@ void CGDwelling::pickRandomObject(vstd::RNG & rand)
 
 		if (subID == MapObjectSubID())
 		{
-			logGlobal->error("Error: failed to find dwelling for %s of level %d", (*VLC->townh)[faction]->getNameTranslated(), int(level));
+			logGlobal->error("Error: failed to find dwelling for %s of level %d", (*LIBRARY->townh)[faction]->getNameTranslated(), int(level));
 			ID = Obj::CREATURE_GENERATOR1;
-			subID = *RandomGeneratorUtil::nextItem(VLC->objtypeh->knownSubObjects(Obj::CREATURE_GENERATOR1), rand);
+			subID = *RandomGeneratorUtil::nextItem(LIBRARY->objtypeh->knownSubObjects(Obj::CREATURE_GENERATOR1), gameRandomizer.getDefault());
 		}
 
 		setType(ID, subID);
 	}
 }
 
-void CGDwelling::initObj(vstd::RNG & rand)
+void CGDwelling::initObj(IGameRandomizer & gameRandomizer)
 {
 	switch(ID.toEnum())
 	{
@@ -182,7 +189,7 @@ void CGDwelling::initObj(vstd::RNG & rand)
 	case Obj::CREATURE_GENERATOR4:
 	case Obj::WAR_MACHINE_FACTORY:
 		{
-			getObjectHandler()->configureObject(this, rand);
+			getObjectHandler()->configureObject(this, gameRandomizer);
 			assert(!creatures.empty());
 			assert(!creatures[0].second.empty());
 			break;
@@ -209,20 +216,32 @@ void CGDwelling::setPropertyDer(ObjProperty what, ObjPropertyID identifier)
 	}
 }
 
-void CGDwelling::onHeroVisit( const CGHeroInstance * h ) const
+void CGDwelling::onHeroVisit(IGameEventCallback & gameEvents, const CGHeroInstance * h) const
 {
-	if(ID == Obj::REFUGEE_CAMP && !creatures[0].first) //Refugee Camp, no available cres
+	if(ID == Obj::REFUGEE_CAMP)
 	{
-		InfoWindow iw;
-		iw.type = EInfoWindowMode::AUTO;
-		iw.player = h->tempOwner;
-		iw.text.appendLocalString(EMetaText::ADVOB_TXT, 44); //{%s} \n\n The camp is deserted.  Perhaps you should try next week.
-		iw.text.replaceName(ID, subID);
-		cb->sendAndApply(iw);
-		return;
+		ChangeObjectVisitors cow;
+		cow.mode = ChangeObjectVisitors::VISITOR_CLEAR;
+		cow.object = id;
+		gameEvents.sendAndApply(cow);
+
+		cow.mode = ChangeObjectVisitors::VISITOR_ADD_PLAYER;
+		cow.hero = h->id;
+		gameEvents.sendAndApply(cow);
+
+		if (!creatures[0].first) //Refugee Camp, no available cres
+		{
+			InfoWindow iw;
+			iw.type = EInfoWindowMode::AUTO;
+			iw.player = h->tempOwner;
+			iw.text.appendLocalString(EMetaText::ADVOB_TXT, 44); //{%s} \n\n The camp is deserted.  Perhaps you should try next week.
+			iw.text.replaceName(ID, subID);
+			gameEvents.sendAndApply(iw);
+			return;
+		}
 	}
 
-	PlayerRelations relations = cb->gameState()->getPlayerRelations( h->tempOwner, tempOwner );
+	PlayerRelations relations = cb->getPlayerRelations( h->tempOwner, tempOwner );
 
 	if ( relations == PlayerRelations::ALLIES )
 		return;//do not allow recruiting or capturing
@@ -238,21 +257,29 @@ void CGDwelling::onHeroVisit( const CGHeroInstance * h ) const
 		else
 			bd.text.replaceLocalString(EMetaText::ARRAY_TXT, 173 + (int)Slots().begin()->second->getQuantityID()*3);
 		bd.text.replaceName(*Slots().begin()->second);
-		cb->showBlockingDialog(this, &bd);
+		gameEvents.showBlockingDialog(this, &bd);
 		return;
 	}
 
 	// TODO this shouldn't be hardcoded
 	if(relations == PlayerRelations::ENEMIES && ID != Obj::WAR_MACHINE_FACTORY && ID != Obj::REFUGEE_CAMP)
 	{
-		cb->setOwner(this, h->tempOwner);
+		gameEvents.setOwner(this, h->tempOwner);
 	}
 
 	BlockingDialog bd (true,false);
 	bd.player = h->tempOwner;
 	if(ID == Obj::CREATURE_GENERATOR1 || ID == Obj::CREATURE_GENERATOR4)
 	{
-		bd.text.appendLocalString(EMetaText::ADVOB_TXT, ID == Obj::CREATURE_GENERATOR1 ? 35 : 36); //{%s} Would you like to recruit %s? / {%s} Would you like to recruit %s, %s, %s, or %s?
+		const size_t count = std::min<size_t>(creatures.size(), 4);
+		constexpr std::array dwellingVisitTextID = {
+			"core.advevent.35", // 0 creatures, should not happen
+			"core.advevent.35",
+			"vcmi.adventureMap.dwelling2",
+			"vcmi.adventureMap.dwelling3",
+			"core.advevent.36"
+		};
+		bd.text.appendTextID(dwellingVisitTextID[count]);	
 		bd.text.replaceTextID(getObjectHandler()->getNameTextID());
 		for(const auto & elem : creatures)
 			bd.text.replaceNamePlural(elem.second[0]);
@@ -269,15 +296,15 @@ void CGDwelling::onHeroVisit( const CGHeroInstance * h ) const
 	else
 		throw std::runtime_error("Illegal dwelling!");
 
-	if(ID == Obj::REFUGEE_CAMP || (ID == Obj::CREATURE_GENERATOR1 && VLC->creatures()->getById(creatures[0].second[0])->getLevel() != 1))
+	if(ID == Obj::REFUGEE_CAMP || (ID == Obj::CREATURE_GENERATOR1 && LIBRARY->creatures()->getById(creatures[0].second[0])->getLevel() != 1))
 	{
 		bd.flags |= BlockingDialog::SAFE_TO_AUTOACCEPT;
 	}
 
-	cb->showBlockingDialog(this, &bd);
+	gameEvents.showBlockingDialog(this, &bd);
 }
 
-void CGDwelling::newTurn(vstd::RNG & rand) const
+void CGDwelling::newTurn(IGameEventCallback & gameEvents, IGameRandomizer & gameRandomizer) const
 {
 	if(cb->getDate(Date::DAY_OF_WEEK) != 1) //not first day of week
 		return;
@@ -288,7 +315,12 @@ void CGDwelling::newTurn(vstd::RNG & rand) const
 
 	if(ID == Obj::REFUGEE_CAMP) //if it's a refugee camp, we need to pick an available creature
 	{
-		cb->setObjPropertyID(id, ObjProperty::AVAILABLE_CREATURE, VLC->creh->pickRandomMonster(rand));
+		ChangeObjectVisitors cow;
+		cow.mode = ChangeObjectVisitors::VISITOR_CLEAR;
+		cow.object = id;
+		gameEvents.sendAndApply(cow);
+
+		gameEvents.setObjPropertyID(id, ObjProperty::AVAILABLE_CREATURE, gameRandomizer.rollCreature());
 	}
 
 	bool change = false;
@@ -318,45 +350,46 @@ void CGDwelling::newTurn(vstd::RNG & rand) const
 	}
 
 	if(change)
-		cb->sendAndApply(sac);
+		gameEvents.sendAndApply(sac);
 
-	updateGuards();
+	updateGuards(gameEvents);
+}
+
+bool CGDwelling::wasVisited (PlayerColor player) const
+{
+	return cb->getPlayerState(player)->visitedObjects.count(id) != 0;
 }
 
 std::vector<Component> CGDwelling::getPopupComponents(PlayerColor player) const
 {
 	bool visitedByOwner = getOwner() == player;
+	bool isDwelling = ID == Obj::CREATURE_GENERATOR1 || ID == Obj::CREATURE_GENERATOR4;
+	bool isRefugeeCamp = ID == Obj::REFUGEE_CAMP;
+	bool canSeeUnitTypes = isDwelling || (isRefugeeCamp && wasVisited(player));
+	bool canSeeUnitAmount = (isDwelling && visitedByOwner) || (isRefugeeCamp && wasVisited(player));
 
 	std::vector<Component> result;
 
-	if (ID == Obj::CREATURE_GENERATOR1 && !creatures.empty())
-	{
-		for (auto const & creature : creatures.front().second)
-		{
-			if (visitedByOwner)
-				result.emplace_back(ComponentType::CREATURE, creature, creatures.front().first);
-			else
-				result.emplace_back(ComponentType::CREATURE, creature);
-		}
-	}
+	if (creatures.empty())
+		return result;
 
-	if (ID == Obj::CREATURE_GENERATOR4)
+	if (canSeeUnitTypes)
 	{
 		for (auto const & creatureLevel : creatures)
 		{
-			if (!creatureLevel.second.empty())
+			for (auto const & creature : creatureLevel.second)
 			{
-				if (visitedByOwner)
-					result.emplace_back(ComponentType::CREATURE, creatureLevel.second.back(), creatureLevel.first);
+				if (canSeeUnitAmount)
+					result.emplace_back(ComponentType::CREATURE, creature, creatures.front().first);
 				else
-					result.emplace_back(ComponentType::CREATURE, creatureLevel.second.back());
+					result.emplace_back(ComponentType::CREATURE, creature);
 			}
 		}
 	}
 	return result;
 }
 
-void CGDwelling::updateGuards() const
+void CGDwelling::updateGuards(IGameEventCallback & gameEvents) const
 {
 	//TODO: store custom guard config and use it
 	//TODO: store boolean flag for guards
@@ -365,7 +398,7 @@ void CGDwelling::updateGuards() const
 	//default condition - creatures are of level 5 or higher
 	for (auto creatureEntry : creatures)
 	{
-		if (VLC->creatures()->getById(creatureEntry.second.at(0))->getLevel() >= 5 && ID != Obj::REFUGEE_CAMP)
+		if (LIBRARY->creatures()->getById(creatureEntry.second.at(0))->getLevel() >= 5 && ID != Obj::REFUGEE_CAMP)
 		{
 			guarded = true;
 			break;
@@ -385,8 +418,8 @@ void CGDwelling::updateGuards() const
 				csc.army = this->id;
 				csc.slot = slot;
 				csc.count = crea->getGrowth() * 3;
-				csc.absoluteValue = true;
-				cb->sendAndApply(csc);
+				csc.mode = ChangeValueMode::ABSOLUTE;
+				gameEvents.sendAndApply(csc);
 			}
 			else //slot is empty, create whole new stack
 			{
@@ -395,13 +428,13 @@ void CGDwelling::updateGuards() const
 				ns.slot = slot;
 				ns.type = crea->getId();
 				ns.count = crea->getGrowth() * 3;
-				cb->sendAndApply(ns);
+				gameEvents.sendAndApply(ns);
 			}
 		}
 	}
 }
 
-void CGDwelling::heroAcceptsCreatures( const CGHeroInstance *h) const
+void CGDwelling::heroAcceptsCreatures(IGameEventCallback & gameEvents, const CGHeroInstance *h) const
 {
 	CreatureID crid = creatures[0].second[0];
 	auto *crs = crid.toCreature();
@@ -420,7 +453,7 @@ void CGDwelling::heroAcceptsCreatures( const CGHeroInstance *h) const
 					std::pair<SlotID, SlotID> toMerge;
 					if (h->mergeableStacks(toMerge))
 					{
-						cb->moveStack(StackLocation(h, toMerge.first), StackLocation(h, toMerge.second), -1); //merge toMerge.first into toMerge.second
+						gameEvents.moveStack(StackLocation(h->id, toMerge.first), StackLocation(h->id, toMerge.second), -1); //merge toMerge.first into toMerge.second
 						assert(!h->hasStackAtSlot(toMerge.first)); //we have now a new free slot
 					}
 				}
@@ -434,7 +467,7 @@ void CGDwelling::heroAcceptsCreatures( const CGHeroInstance *h) const
 				iw.player = h->tempOwner;
 				iw.text.appendLocalString(EMetaText::GENERAL_TXT, 425);//The %s would join your hero, but there aren't enough provisions to support them.
 				iw.text.replaceNamePlural(crid);
-				cb->showInfoDialog(&iw);
+				gameEvents.showInfoDialog(&iw);
 			}
 			else //give creatures
 			{
@@ -451,9 +484,9 @@ void CGDwelling::heroAcceptsCreatures( const CGHeroInstance *h) const
 				iw.text.replaceNumber(count);
 				iw.text.replaceNamePlural(crid);
 
-				cb->showInfoDialog(&iw);
-				cb->sendAndApply(sac);
-				cb->addToSlot(StackLocation(h, slot), crs, count);
+				gameEvents.showInfoDialog(&iw);
+				gameEvents.sendAndApply(sac);
+				gameEvents.addToSlot(StackLocation(h->id, slot), crs, count);
 			}
 		}
 		else //there no creatures
@@ -463,7 +496,7 @@ void CGDwelling::heroAcceptsCreatures( const CGHeroInstance *h) const
 			iw.text.appendLocalString(EMetaText::GENERAL_TXT, 422); //There are no %s here to recruit.
 			iw.text.replaceNamePlural(crid);
 			iw.player = h->tempOwner;
-			cb->sendAndApply(iw);
+			gameEvents.sendAndApply(iw);
 		}
 	}
 	else
@@ -474,36 +507,44 @@ void CGDwelling::heroAcceptsCreatures( const CGHeroInstance *h) const
 			SetAvailableCreatures sac;
 			sac.tid = id;
 			sac.creatures = creatures;
-			sac.creatures[0].first = !h->getArt(ArtifactPosition::MACH1); //ballista
-			sac.creatures[1].first = !h->getArt(ArtifactPosition::MACH3); //first aid tent
-			sac.creatures[2].first = !h->getArt(ArtifactPosition::MACH2); //ammo cart
-			cb->sendAndApply(sac);
+
+			for (auto & entry : sac.creatures)
+			{
+				CreatureID creature = entry.second.at(0);
+				ArtifactID warMachine = creature.toCreature()->warMachine;
+
+				if (h->hasArt(warMachine, true, false))
+					entry.first = 0;
+				else
+					entry.first = 1;
+			}
+			gameEvents.sendAndApply(sac);
 		}
 
 		auto windowMode = (ID == Obj::CREATURE_GENERATOR1 || ID == Obj::REFUGEE_CAMP) ? EOpenWindowMode::RECRUITMENT_FIRST : EOpenWindowMode::RECRUITMENT_ALL;
-		cb->showObjectWindow(this, windowMode, h, true);
+		gameEvents.showObjectWindow(this, windowMode, h, true);
 	}
 }
 
-void CGDwelling::battleFinished(const CGHeroInstance *hero, const BattleResult &result) const
+void CGDwelling::battleFinished(IGameEventCallback & gameEvents, const CGHeroInstance *hero, const BattleResult &result) const
 {
 	if (result.winner == BattleSide::ATTACKER)
 	{
-		onHeroVisit(hero);
+		onHeroVisit(gameEvents, hero);
 	}
 }
 
-void CGDwelling::blockingDialogAnswered(const CGHeroInstance *hero, int32_t answer) const
+void CGDwelling::blockingDialogAnswered(IGameEventCallback & gameEvents, const CGHeroInstance *hero, int32_t answer) const
 {
 	auto relations = cb->getPlayerRelations(getOwner(), hero->getOwner());
 	if(stacksCount() > 0  && relations == PlayerRelations::ENEMIES) //guards present
 	{
 		if(answer)
-			cb->startBattle(hero, this);
+			gameEvents.startBattle(hero, this);
 	}
 	else if(answer)
 	{
-		heroAcceptsCreatures(hero);
+		heroAcceptsCreatures(gameEvents, hero);
 	}
 }
 
@@ -545,6 +586,13 @@ ResourceSet CGDwelling::dailyIncome() const
 	return {};
 }
 
+AnimationPath CGDwelling::getKingdomOverviewImage() const
+{
+	const auto & baseHandler = getObjectHandler();
+	const auto & ourHandler = std::dynamic_pointer_cast<const DwellingInstanceConstructor>(baseHandler);
+	return ourHandler ? ourHandler->getKingdomOverviewImage() : AnimationPath{};
+}
+
 std::vector<CreatureID> CGDwelling::providedCreatures() const
 {
 	if (ID == Obj::WAR_MACHINE_FACTORY || ID == Obj::REFUGEE_CAMP)
