@@ -11,16 +11,21 @@
 #include "StdInc.h"
 #include "CGMarket.h"
 
-#include "../texts/CGeneralTextHandler.h"
-#include "../IGameCallback.h"
-#include "../CCreatureHandler.h"
+#include "CGHeroInstance.h"
 #include "CGTownInstance.h"
+
+//#include "../CCreatureHandler.h"
+#include "../CPlayerState.h"
+//#include "../CSkillHandler.h"
 #include "../IGameSettings.h"
-#include "../CSkillHandler.h"
+#include "../callback/IGameEventCallback.h"
+#include "../callback/IGameInfoCallback.h"
+#include "../callback/IGameRandomizer.h"
 #include "../mapObjectConstructors/AObjectTypeHandler.h"
 #include "../mapObjectConstructors/CObjectClassesHandler.h"
-#include "../mapObjectConstructors/CommonConstructors.h"
+#include "../mapObjectConstructors/MarketInstanceConstructor.h"
 #include "../networkPacks/PacksForClient.h"
+#include "../texts/TextIdentifier.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
@@ -29,14 +34,14 @@ ObjectInstanceID CGMarket::getObjInstanceID() const
 	return id;
 }
 
-void CGMarket::initObj(vstd::RNG & rand)
+void CGMarket::initObj(IGameRandomizer & gameRandomizer)
 {
-	getObjectHandler()->configureObject(this, rand);
+	getObjectHandler()->configureObject(this, gameRandomizer);
 }
 
-void CGMarket::onHeroVisit(const CGHeroInstance * h) const
+void CGMarket::onHeroVisit(IGameEventCallback & gameEvents, const CGHeroInstance * h) const
 {
-	cb->showObjectWindow(this, EOpenWindowMode::MARKET_WINDOW, h, true);
+	gameEvents.showObjectWindow(this, EOpenWindowMode::MARKET_WINDOW, h, true);
 }
 
 std::string CGMarket::getPopupText(PlayerColor player) const
@@ -46,7 +51,7 @@ std::string CGMarket::getPopupText(PlayerColor player) const
 
 	MetaString message = MetaString::createFromRawString("{%s}\r\n\r\n%s");
 	message.replaceName(ID, subID);
-	message.replaceTextID(TextIdentifier(getObjectHandler()->getBaseTextID(), "description").get());
+	message.replaceTextID(getMarketHandler()->getDescriptionTextID());
 	return message.toString();
 }
 
@@ -77,8 +82,9 @@ std::set<EMarketMode> CGMarket::availableModes() const
 	return getMarketHandler()->availableModes();
 }
 
-CGMarket::CGMarket(IGameCallback *cb):
-	CGObjectInstance(cb)
+CGMarket::CGMarket(IGameInfoCallback *cb)
+	: CGObjectInstance(cb)
+	, IMarket(cb)
 {}
 
 std::vector<TradeItemBuy> CGBlackMarket::availableItemsIds(EMarketMode mode) const
@@ -97,7 +103,7 @@ std::vector<TradeItemBuy> CGBlackMarket::availableItemsIds(EMarketMode mode) con
 	}
 }
 
-void CGBlackMarket::newTurn(vstd::RNG & rand) const
+void CGBlackMarket::newTurn(IGameEventCallback & gameEvents, IGameRandomizer & gameRandomizer) const
 {
 	int resetPeriod = cb->getSettings().getInteger(EGameSettings::MARKETS_BLACK_MARKET_RESTOCK_PERIOD);
 
@@ -109,8 +115,8 @@ void CGBlackMarket::newTurn(vstd::RNG & rand) const
 
 	SetAvailableArtifacts saa;
 	saa.id = id;
-	cb->pickAllowedArtsSet(saa.arts, rand);
-	cb->sendAndApply(saa);
+	saa.arts = gameRandomizer.rollMarketArtifactSet();
+	gameEvents.sendAndApply(saa);
 }
 
 std::vector<TradeItemBuy> CGUniversity::availableItemsIds(EMarketMode mode) const
@@ -130,9 +136,33 @@ std::string CGUniversity::getSpeechTranslated() const
 	return getMarketHandler()->getSpeechTranslated();
 }
 
-void CGUniversity::onHeroVisit(const CGHeroInstance * h) const
+void CGUniversity::onHeroVisit(IGameEventCallback & gameEvents, const CGHeroInstance * h) const
 {
-	cb->showObjectWindow(this, EOpenWindowMode::UNIVERSITY_WINDOW, h, true);
+	ChangeObjectVisitors cow;
+	cow.object = id;
+	cow.mode = ChangeObjectVisitors::VISITOR_ADD_PLAYER;
+	cow.hero = h->id;
+	gameEvents.sendAndApply(cow);
+
+	gameEvents.showObjectWindow(this, EOpenWindowMode::UNIVERSITY_WINDOW, h, true);
+}
+
+bool CGUniversity::wasVisited (PlayerColor player) const
+{
+	return cb->getPlayerState(player)->visitedObjects.count(id) != 0;
+}
+
+std::vector<Component> CGUniversity::getPopupComponents(PlayerColor player) const
+{
+	std::vector<Component> result;
+
+	if (!wasVisited(player))
+		return result;
+
+	for (auto const & skill : skills)
+		result.emplace_back(ComponentType::SEC_SKILL, skill.as<SecondarySkill>());
+
+	return result;
 }
 
 VCMI_LIB_NAMESPACE_END
