@@ -10,10 +10,7 @@
 #pragma once
 
 // Helper header that includes all map objects, similar to old CObjectHandler.h
-// Possible TODO - remove this header after CObjectHandler.cpp will be fully split into smaller files
-#include "CObjectHandler.h"
 
-#include "CArmedInstance.h"
 #include "CGDwelling.h"
 #include "CGHeroInstance.h"
 #include "CGMarket.h"
@@ -21,4 +18,3 @@
 #include "CGPandoraBox.h"
 #include "CRewardableObject.h"
 #include "MiscObjects.h"
-#include "CQuest.h"
