@@ -11,13 +11,14 @@
 #include "StdInc.h"
 #include "CMapOperation.h"
 
-#include "../VCMI_Lib.h"
-#include "../CRandomGenerator.h"
+#include "../GameLibrary.h"
 #include "../TerrainHandler.h"
 #include "../mapObjects/CGObjectInstance.h"
 #include "CMap.h"
 #include "MapEditUtils.h"
 
+#include <vstd/RNG.h>
+
 VCMI_LIB_NAMESPACE_BEGIN
 
 CMapOperation::CMapOperation(CMap* map) : map(map)
@@ -87,9 +88,10 @@ void CComposedOperation::addOperation(std::unique_ptr<CMapOperation>&& operation
 	operations.push_back(std::move(operation));
 }
 
-CDrawTerrainOperation::CDrawTerrainOperation(CMap * map, CTerrainSelection terrainSel, TerrainId terType, int decorationsPercentage, CRandomGenerator * gen):
+CDrawTerrainOperation::CDrawTerrainOperation(CMap * map, CTerrainSelection terrainSel, TerrainId terType, int decorationsPercentage, vstd::RNG * gen):
 	CMapOperation(map),
 	terrainSel(std::move(terrainSel)),
+	extendedSel(CTerrainSelection(terrainSel.getMap())),
 	terType(terType),
 	decorationsPercentage(decorationsPercentage),
 	gen(gen)
@@ -101,23 +103,49 @@ void CDrawTerrainOperation::execute()
 {
 	for(const auto & pos : terrainSel.getSelectedItems())
 	{
-		auto & tile = map->getTile(pos);
-		tile.terType = const_cast<TerrainType*>(VLC->terrainTypeHandler->getById(terType));
-		invalidateTerrainViews(pos);
+		saveTileState(pos);
+		expandInvalidatedTileList(pos);
 	}
-
-	updateTerrainTypes();
+	changeTerrainType(terrainSel, terType);
+	expandSelection(terrainSel);
 	updateTerrainViews();
 }
 
+void CDrawTerrainOperation::changeTerrainType(CTerrainSelection selection, TerrainId terrainType)
+{
+	for(const auto & pos : selection.getSelectedItems())
+	{
+		auto & tile = map->getTile(pos);
+		tile.terrainType = terrainType;
+	}
+}
+
 void CDrawTerrainOperation::undo()
 {
-	//TODO
+	for (auto const& typeToSelection : formerState)
+	{
+		changeTerrainType(typeToSelection.second, typeToSelection.first);
+	}
+	updateTerrainViews();
 }
 
 void CDrawTerrainOperation::redo()
 {
-	//TODO
+	changeTerrainType(extendedSel, terType);
+	updateTerrainViews();
+}
+
+void CDrawTerrainOperation::saveTileState(int3 tile)
+{
+	if(extendedSel.getSelectedItems().contains(tile))	//tile is being changed a second time
+		return;
+
+	extendedSel.select(tile);
+
+	auto terrainType = map->getTile(tile).terrainType;
+	if (!formerState.contains(terrainType))
+		formerState.insert({terrainType, CTerrainSelection(terrainSel.getMap())});
+	formerState.at(terrainType).select(tile);
 }
 
 std::string CDrawTerrainOperation::getLabel() const
@@ -125,9 +153,9 @@ std::string CDrawTerrainOperation::getLabel() const
 	return "Draw Terrain";
 }
 
-void CDrawTerrainOperation::updateTerrainTypes()
+void CDrawTerrainOperation::expandSelection(CTerrainSelection selection)
 {
-	auto positions = terrainSel.getSelectedItems();
+	auto positions = selection.getSelectedItems();
 	while(!positions.empty())
 	{
 		const auto & centerPos = *(positions.begin());
@@ -136,9 +164,10 @@ void CDrawTerrainOperation::updateTerrainTypes()
 		auto tiles = getInvalidTiles(centerPos);
 		auto updateTerrainType = [&](const int3& pos)
 		{
-			map->getTile(pos).terType = centerTile.terType;
+			saveTileState(pos);
+			map->getTile(pos).terrainType = centerTile.terrainType;
 			positions.insert(pos);
-			invalidateTerrainViews(pos);
+			expandInvalidatedTileList(pos);
 			//logGlobal->debug("Set additional terrain tile at pos '%s' to type '%s'", pos, centerTile.terType);
 		};
 
@@ -160,10 +189,10 @@ void CDrawTerrainOperation::updateTerrainTypes()
 			rect.forEach([&](const int3& posToTest)
 				{
 					auto & terrainTile = map->getTile(posToTest);
-					if(centerTile.terType->getId() != terrainTile.terType->getId())
+					if(centerTile.getTerrain() != terrainTile.getTerrain())
 					{
-						const auto * formerTerType = terrainTile.terType;
-						terrainTile.terType = centerTile.terType;
+						const auto formerTerType = terrainTile.terrainType;
+						terrainTile.terrainType = centerTile.terrainType;
 						auto testTile = getInvalidTiles(posToTest);
 
 						int nativeTilesCntNorm = testTile.nativeTiles.empty() ? std::numeric_limits<int>::max() : static_cast<int>(testTile.nativeTiles.size());
@@ -220,7 +249,7 @@ void CDrawTerrainOperation::updateTerrainTypes()
 							suitableTiles.insert(posToTest);
 						}
 
-						terrainTile.terType = formerTerType;
+						terrainTile.terrainType = formerTerType;
 					}
 				});
 
@@ -263,7 +292,7 @@ void CDrawTerrainOperation::updateTerrainViews()
 {
 	for(const auto & pos : invalidatedTerViews)
 	{
-		const auto & patterns = VLC->terviewh->getTerrainViewPatterns(map->getTile(pos).terType->getId());
+		const auto & patterns = LIBRARY->terviewh->getTerrainViewPatterns(map->getTile(pos).getTerrainID());
 
 		// Detect a pattern which fits best
 		int bestPattern = -1;
@@ -310,7 +339,7 @@ void CDrawTerrainOperation::updateTerrainViews()
 		if(!pattern.diffImages)
 		{
 			tile.terView = gen->nextInt(mapping.first, mapping.second);
-			tile.extTileFlags = valRslt.flip;
+			tile.extTileFlags = (tile.extTileFlags & 0b11111100) | valRslt.flip;
 		}
 		else
 		{
@@ -339,7 +368,7 @@ CDrawTerrainOperation::ValidationResult CDrawTerrainOperation::validateTerrainVi
 
 CDrawTerrainOperation::ValidationResult CDrawTerrainOperation::validateTerrainViewInner(const int3& pos, const TerrainViewPattern& pattern, int recDepth) const
 {
-	const auto * centerTerType = map->getTile(pos).terType;
+	const auto * centerTerType = map->getTile(pos).getTerrain();
 	int totalPoints = 0;
 	std::string transitionReplacement;
 
@@ -371,24 +400,24 @@ CDrawTerrainOperation::ValidationResult CDrawTerrainOperation::validateTerrainVi
 			}
 			else if(widthTooHigh)
 			{
-				terType = map->getTile(int3(currentPos.x - 1, currentPos.y, currentPos.z)).terType;
+				terType = map->getTile(int3(currentPos.x - 1, currentPos.y, currentPos.z)).getTerrain();
 			}
 			else if(heightTooHigh)
 			{
-				terType = map->getTile(int3(currentPos.x, currentPos.y - 1, currentPos.z)).terType;
+				terType = map->getTile(int3(currentPos.x, currentPos.y - 1, currentPos.z)).getTerrain();
 			}
 			else if(widthTooLess)
 			{
-				terType = map->getTile(int3(currentPos.x + 1, currentPos.y, currentPos.z)).terType;
+				terType = map->getTile(int3(currentPos.x + 1, currentPos.y, currentPos.z)).getTerrain();
 			}
 			else if(heightTooLess)
 			{
-				terType = map->getTile(int3(currentPos.x, currentPos.y + 1, currentPos.z)).terType;
+				terType = map->getTile(int3(currentPos.x, currentPos.y + 1, currentPos.z)).getTerrain();
 			}
 		}
 		else
 		{
-			terType = map->getTile(currentPos).terType;
+			terType = map->getTile(currentPos).getTerrain();
 			if(terType != centerTerType && (terType->isPassable() || centerTerType->isPassable()))
 			{
 				isAlien = true;
@@ -404,9 +433,9 @@ CDrawTerrainOperation::ValidationResult CDrawTerrainOperation::validateTerrainVi
 			{
 				if(recDepth == 0 && map->isInTheMap(currentPos))
 				{
-					if(terType->getId() == centerTerType->getId())
+					if(centerTerType->getId() == terType->getId() || (centerTerType->getId() == ETerrainId::DIRT && !terType->isTransitionRequired()))
 					{
-						const auto patternForRule = VLC->terviewh->getTerrainViewPatternsById(centerTerType->getId(), rule.name);
+						const auto patternForRule = LIBRARY->terviewh->getTerrainViewPatternsById(terType->getId(), rule.name);
 						if(auto p = patternForRule)
 						{
 							auto rslt = validateTerrainView(currentPos, &(p->get()), 1);
@@ -495,7 +524,7 @@ CDrawTerrainOperation::ValidationResult CDrawTerrainOperation::validateTerrainVi
 	}
 }
 
-void CDrawTerrainOperation::invalidateTerrainViews(const int3& centerPos)
+void CDrawTerrainOperation::expandInvalidatedTileList(const int3& centerPos)
 {
 	auto rect = extendTileAroundSafely(centerPos);
 	rect.forEach([&](const int3& pos)
@@ -508,14 +537,14 @@ CDrawTerrainOperation::InvalidTiles CDrawTerrainOperation::getInvalidTiles(const
 {
 	//TODO: this is very expensive function for RMG, needs optimization
 	InvalidTiles tiles;
-	const auto * centerTerType = map->getTile(centerPos).terType;
+	const auto * centerTerType = map->getTile(centerPos).getTerrain();
 	auto rect = extendTileAround(centerPos);
 	rect.forEach([&](const int3& pos)
 		{
 			if(map->isInTheMap(pos))
 			{
-				const auto * terType = map->getTile(pos).terType;
-				auto valid = validateTerrainView(pos, VLC->terviewh->getTerrainTypePatternById("n1")).result;
+				const auto * terType = map->getTile(pos).getTerrain();
+				auto valid = validateTerrainView(pos, LIBRARY->terviewh->getTerrainTypePatternById("n1")).result;
 
 				// Special validity check for rock & water
 				if(valid && (terType->isWater() || !terType->isPassable()))
@@ -523,7 +552,7 @@ CDrawTerrainOperation::InvalidTiles CDrawTerrainOperation::getInvalidTiles(const
 					static const std::string patternIds[] = { "s1", "s2" };
 					for(const auto & patternId : patternIds)
 					{
-						valid = !validateTerrainView(pos, VLC->terviewh->getTerrainTypePatternById(patternId)).result;
+						valid = !validateTerrainView(pos, LIBRARY->terviewh->getTerrainTypePatternById(patternId)).result;
 						if(!valid) break;
 					}
 				}
@@ -533,7 +562,7 @@ CDrawTerrainOperation::InvalidTiles CDrawTerrainOperation::getInvalidTiles(const
 					static const std::string patternIds[] = { "n2", "n3" };
 					for(const auto & patternId : patternIds)
 					{
-						valid = validateTerrainView(pos, VLC->terviewh->getTerrainTypePatternById(patternId)).result;
+						valid = validateTerrainView(pos, LIBRARY->terviewh->getTerrainTypePatternById(patternId)).result;
 						if(valid) break;
 					}
 				}
@@ -560,16 +589,13 @@ CDrawTerrainOperation::ValidationResult::ValidationResult(bool result, std::stri
 
 }
 
-CClearTerrainOperation::CClearTerrainOperation(CMap* map, CRandomGenerator* gen) : CComposedOperation(map)
+CClearTerrainOperation::CClearTerrainOperation(CMap* map, vstd::RNG* gen) : CComposedOperation(map)
 {
-	CTerrainSelection terrainSel(map);
-	terrainSel.selectRange(MapRect(int3(0, 0, 0), map->width, map->height));
-	addOperation(std::make_unique<CDrawTerrainOperation>(map, terrainSel, ETerrainId::WATER, 0, gen));
-	if(map->twoLevel)
+	for (int i = 0; i < map->levels(); i++)
 	{
-		terrainSel.clearSelection();
-		terrainSel.selectRange(MapRect(int3(0, 0, 1), map->width, map->height));
-		addOperation(std::make_unique<CDrawTerrainOperation>(map, terrainSel, ETerrainId::ROCK, 0, gen));
+		CTerrainSelection terrainSel(map);
+		terrainSel.selectRange(MapRect(int3(0, 0, i), map->width, map->height));
+		addOperation(std::make_unique<CDrawTerrainOperation>(map, terrainSel, i == 1 ? ETerrainId::ROCK : ETerrainId::WATER, 0, gen));
 	}
 }
 
@@ -578,7 +604,7 @@ std::string CClearTerrainOperation::getLabel() const
 	return "Clear Terrain";
 }
 
-CInsertObjectOperation::CInsertObjectOperation(CMap* map, CGObjectInstance* obj)
+CInsertObjectOperation::CInsertObjectOperation(CMap* map, std::shared_ptr<CGObjectInstance> obj)
 	: CMapOperation(map), obj(obj)
 {
 
@@ -586,19 +612,13 @@ CInsertObjectOperation::CInsertObjectOperation(CMap* map, CGObjectInstance* obj)
 
 void CInsertObjectOperation::execute()
 {
-	obj->id = ObjectInstanceID(map->objects.size());
-
-	do
-	{
-		map->setUniqueInstanceName(obj);
-	} while(vstd::contains(map->instanceNames, obj->instanceName));
-
+	map->generateUniqueInstanceName(obj.get());
 	map->addNewObject(obj);
 }
 
 void CInsertObjectOperation::undo()
 {
-	map->removeObject(obj);
+	map->removeObject(obj->id);
 }
 
 void CInsertObjectOperation::redo()
@@ -611,22 +631,22 @@ std::string CInsertObjectOperation::getLabel() const
 	return "Insert Object";
 }
 
-CMoveObjectOperation::CMoveObjectOperation(CMap* map, CGObjectInstance* obj, const int3& targetPosition)
+CMoveObjectOperation::CMoveObjectOperation(CMap* map, CGObjectInstance * obj, const int3& targetPosition)
 	: CMapOperation(map),
 	obj(obj),
-	initialPos(obj->pos),
+	initialPos(obj->anchorPos()),
 	targetPos(targetPosition)
 {
 }
 
 void CMoveObjectOperation::execute()
 {
-	map->moveObject(obj, targetPos);
+	map->moveObject(obj->id, targetPos);
 }
 
 void CMoveObjectOperation::undo()
 {
-	map->moveObject(obj, initialPos);
+	map->moveObject(obj->id, initialPos);
 }
 
 void CMoveObjectOperation::redo()
@@ -639,46 +659,21 @@ std::string CMoveObjectOperation::getLabel() const
 	return "Move Object";
 }
 
-CRemoveObjectOperation::CRemoveObjectOperation(CMap* map, CGObjectInstance* obj)
-	: CMapOperation(map), obj(obj)
+CRemoveObjectOperation::CRemoveObjectOperation(CMap* map, CGObjectInstance * obj)
+	: CMapOperation(map), targetedObject(obj)
 {
 
 }
 
-CRemoveObjectOperation::~CRemoveObjectOperation()
-{
-	//when operation is destroyed and wasn't undone, the object is lost forever
-
-	if(!obj)
-	{
-		return;
-	}
-
-	//do not destroy an object that belongs to map
-	if(!vstd::contains(map->instanceNames, obj->instanceName))
-	{
-		delete obj;
-		obj = nullptr;
-	}
-}
-
 void CRemoveObjectOperation::execute()
 {
-	map->removeObject(obj);
+	removedObject = map->removeObject(targetedObject->id);
 }
 
 void CRemoveObjectOperation::undo()
 {
-	try
-	{
-		//set new id, but do not rename object
-		obj->id = ObjectInstanceID(static_cast<si32>(map->objects.size()));
-		map->addNewObject(obj);
-	}
-	catch(const std::exception& e)
-	{
-		logGlobal->error(e.what());
-	}
+	assert(removedObject != nullptr);
+	map->addNewObject(removedObject);
 }
 
 void CRemoveObjectOperation::redo()
