@@ -10,6 +10,8 @@
 #include "StdInc.h"
 #include "CMapService.h"
 
+#include "MapFormatSettings.h"
+
 #include "../json/JsonUtils.h"
 #include "../filesystem/Filesystem.h"
 #include "../filesystem/CBinaryReader.h"
@@ -19,39 +21,52 @@
 #include "../modding/CModHandler.h"
 #include "../modding/ModDescription.h"
 #include "../modding/ModScope.h"
-#include "../VCMI_Lib.h"
+#include "../GameLibrary.h"
 
 #include "CMap.h"
 #include "MapFormat.h"
 
 #include "MapFormatH3M.h"
 #include "MapFormatJson.h"
+#include "../callback/EditorCallback.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
 
-std::unique_ptr<CMap> CMapService::loadMap(const ResourcePath & name, IGameCallback * cb) const
+std::unique_ptr<CMap> CMapService::loadMap(const ResourcePath & name, IGameInfoCallback * cb) const
 {
-	std::string modName = VLC->modh->findResourceOrigin(name);
-	std::string encoding = VLC->modh->findResourceEncoding(name);
+	std::string modName = LIBRARY->modh->findResourceOrigin(name);
+	std::string encoding = LIBRARY->modh->findResourceEncoding(name);
 
 	auto stream = getStreamFromFS(name);
-	return getMapLoader(stream, name.getName(), modName, encoding)->loadMap(cb);
+	auto loader = getMapLoader(stream, name.getName(), modName, encoding);
+	// If JSON loader, let it know whether this is editor callback
+	if(auto jsonLoader = dynamic_cast<CMapLoaderJson*>(loader.get()))
+		jsonLoader->setRunningInMapEditor(dynamic_cast<EditorCallback*>(cb) != nullptr);
+
+	return loader->loadMap(cb);
 }
 
-std::unique_ptr<CMapHeader> CMapService::loadMapHeader(const ResourcePath & name) const
+std::unique_ptr<CMapHeader> CMapService::loadMapHeader(const ResourcePath & name, bool isEditor) const
 {
-	std::string modName = VLC->modh->findResourceOrigin(name);
-	std::string encoding = VLC->modh->findResourceEncoding(name);
+	std::string modName = LIBRARY->modh->findResourceOrigin(name);
+	std::string encoding = LIBRARY->modh->findResourceEncoding(name);
 
 	auto stream = getStreamFromFS(name);
-	return getMapLoader(stream, name.getName(), modName, encoding)->loadMapHeader();
+	auto loader = getMapLoader(stream, name.getName(), modName, encoding);
+	// default: not editor; callers may pass explicit overload if needed
+	if(auto jsonLoader = dynamic_cast<CMapLoaderJson*>(loader.get()))
+		jsonLoader->setRunningInMapEditor(false);
+	return loader->loadMapHeader();
 }
 
-std::unique_ptr<CMap> CMapService::loadMap(const uint8_t * buffer, int size, const std::string & name,  const std::string & modName, const std::string & encoding, IGameCallback * cb) const
+std::unique_ptr<CMap> CMapService::loadMap(const uint8_t * buffer, int size, const std::string & name,  const std::string & modName, const std::string & encoding, IGameInfoCallback * cb) const
 {
 	auto stream = getStreamFromMem(buffer, size);
-	std::unique_ptr<CMap> map(getMapLoader(stream, name, modName, encoding)->loadMap(cb));
+	auto loader = getMapLoader(stream, name, modName, encoding);
+	if(auto jsonLoader = dynamic_cast<CMapLoaderJson*>(loader.get()))
+		jsonLoader->setRunningInMapEditor(dynamic_cast<EditorCallback*>(cb) != nullptr);
+	std::unique_ptr<CMap> map(loader->loadMap(cb));
 	std::unique_ptr<CMapHeader> header(map.get());
 
 	//might be original campaign and require patch
@@ -64,7 +79,10 @@ std::unique_ptr<CMap> CMapService::loadMap(const uint8_t * buffer, int size, con
 std::unique_ptr<CMapHeader> CMapService::loadMapHeader(const uint8_t * buffer, int size, const std::string & name, const std::string & modName, const std::string & encoding) const
 {
 	auto stream = getStreamFromMem(buffer, size);
-	std::unique_ptr<CMapHeader> header = getMapLoader(stream, name, modName, encoding)->loadMapHeader();
+	auto loader = getMapLoader(stream, name, modName, encoding);
+	if(auto jsonLoader = dynamic_cast<CMapLoaderJson*>(loader.get()))
+		jsonLoader->setRunningInMapEditor(false);
+	std::unique_ptr<CMapHeader> header = loader->loadMapHeader();
 
 	//might be original campaign and require patch
 	getMapPatcher(name)->patchMapHeader(header);
@@ -90,7 +108,7 @@ void CMapService::saveMap(const std::unique_ptr<CMap> & map, boost::filesystem::
 
 ModCompatibilityInfo CMapService::verifyMapHeaderMods(const CMapHeader & map)
 {
-	const auto & activeMods = VLC->modh->getActiveMods();
+	const auto & activeMods = LIBRARY->modh->getActiveMods();
 	
 	ModCompatibilityInfo missingMods;
 	ModCompatibilityInfo missingModsFiltered;
@@ -98,7 +116,7 @@ ModCompatibilityInfo CMapService::verifyMapHeaderMods(const CMapHeader & map)
 	{
 		if(vstd::contains(activeMods, mapMod.first))
 		{
-			const auto & modInfo = VLC->modh->getModInfo(mapMod.first);
+			const auto & modInfo = LIBRARY->modh->getModInfo(mapMod.first);
 			if(modInfo.getVersion().compatible(mapMod.second.version))
 				continue;
 		}
@@ -139,7 +157,7 @@ std::unique_ptr<IMapLoader> CMapService::getMapLoader(std::unique_ptr<CInputStre
 	case 0x06054b50:
 	case 0x04034b50:
 	case 0x02014b50:
-		return std::unique_ptr<IMapLoader>(new CMapLoaderJson(stream.get()));
+		return std::unique_ptr<IMapLoader>(new CMapLoaderJson(stream.get(), mapName));
 		break;
 	default:
 		// Check which map format is used
@@ -163,23 +181,11 @@ std::unique_ptr<IMapLoader> CMapService::getMapLoader(std::unique_ptr<CInputStre
 	}
 }
 
-static JsonNode loadPatches(const std::string & path)
-{
-	JsonNode node = JsonUtils::assembleFromFiles(path);
-	for (auto & entry : node.Struct())
-		JsonUtils::validate(entry.second, "vcmi:mapHeader", "patch for " + entry.first);
-
-	node.setModScope(ModScope::scopeMap());
-	return node;
-}
-
 std::unique_ptr<IMapPatcher> CMapService::getMapPatcher(std::string scenarioName)
 {
-	static const JsonNode node = loadPatches("config/mapOverrides.json");
-
 	boost::to_lower(scenarioName);
 	logGlobal->debug("Request to patch map %s", scenarioName);
-	return std::unique_ptr<IMapPatcher>(new CMapPatcher(node[scenarioName]));
+	return std::make_unique<CMapPatcher>(LIBRARY->mapFormat->mapOverrides(scenarioName));
 }
 
 VCMI_LIB_NAMESPACE_END
