@@ -131,14 +131,19 @@ MapFormatFeaturesH3M MapFormatFeaturesH3M::getFeaturesHOTA(uint32_t hotaVersion)
 {
 	// even if changes are minimal, we might not be able to parse map header in map selection screen
 	// throw exception - to be caught by map selection screen & excluded as invalid
-	if(hotaVersion > 3)
+	if(hotaVersion > 9)
 		throw std::runtime_error("Invalid map format!");
 
 	MapFormatFeaturesH3M result = getFeaturesSOD();
 	result.levelHOTA0 = true;
 	result.levelHOTA1 = hotaVersion > 0;
-	//result.levelHOTA2 = hotaVersion > 1; // HOTA2 seems to be identical to HOTA1 so far
+	result.levelHOTA2 = hotaVersion > 1;
 	result.levelHOTA3 = hotaVersion > 2;
+	result.levelHOTA5 = hotaVersion > 4;
+	result.levelHOTA6 = hotaVersion > 5;
+	result.levelHOTA7 = hotaVersion > 6;
+	result.levelHOTA8 = hotaVersion > 7;
+	result.levelHOTA9 = hotaVersion > 8;
 
 	result.artifactsBytes = 21;
 	result.heroesBytes = 23;
@@ -147,18 +152,34 @@ MapFormatFeaturesH3M MapFormatFeaturesH3M::getFeaturesHOTA(uint32_t hotaVersion)
 	result.skillsCount = 29; // + Interference
 	result.factionsCount = 10; // + Cove
 	result.creaturesCount = 171; // + Cove + neutrals
+	result.artifactsCount = 163; // + HotA artifacts
+	result.heroesCount = 178; // + Cove
+	result.heroesPortraitsCount = 186; // + Cove
 
-	if(hotaVersion < 3)
-	{
-		result.artifactsCount = 163; // + HotA artifacts
-		result.heroesCount = 178; // + Cove
-		result.heroesPortraitsCount = 186; // + Cove
-	}
-	if(hotaVersion == 3)
+	if(hotaVersion >= 3)
 	{
 		result.artifactsCount = 165; // + HotA artifacts
-		result.heroesCount = 179; // + Cove + Giselle
-		result.heroesPortraitsCount = 188; // + Cove + Giselle
+		result.heroesCount = 179; // + Giselle
+		result.heroesPortraitsCount = 188; // + campaign portrait + Giselle
+	}
+	if (hotaVersion >= 5)
+	{
+		result.factionsCount = 11; // + Factory
+		result.creaturesCount = 186; // + 16 Factory
+		result.artifactsCount = 166; // +pendant of reflection, +sleepkeeper
+		result.heroesCount = 198; // + 16 Factory, +3 campaign
+		result.heroesPortraitsCount = 228; // + 16 Factory, +A LOT campaign
+
+		result.heroesBytes = 25;
+	}
+	if (hotaVersion >= 7)
+	{
+		result.factionsCount = 12; // + Bulwark
+		result.creaturesCount = 200; // + 14 Bulwark
+		result.heroesCount = 215; // + 17 Bulwark
+		result.heroesPortraitsCount = 245; // + 17 Bulwark
+		result.skillsCount = 30; // + Runes
+		result.heroesBytes = 27;
 	}
 
 	assert((result.heroesCount + 7) / 8 == result.heroesBytes);
