@@ -16,8 +16,7 @@ enum class EMapFormat : uint8_t;
 
 struct MapFormatFeaturesH3M
 {
-public:
-	static MapFormatFeaturesH3M find(EMapFormat format, uint32_t hotaVersion);
+private:
 	static MapFormatFeaturesH3M getFeaturesROE();
 	static MapFormatFeaturesH3M getFeaturesAB();
 	static MapFormatFeaturesH3M getFeaturesSOD();
@@ -25,8 +24,11 @@ public:
 	static MapFormatFeaturesH3M getFeaturesWOG();
 	static MapFormatFeaturesH3M getFeaturesHOTA(uint32_t hotaVersion);
 
+public:
 	MapFormatFeaturesH3M() = default;
 
+	static MapFormatFeaturesH3M find(EMapFormat format, uint32_t hotaVersion);
+
 	// number of bytes in bitmask of appropriate type
 
 	int factionsBytes;
@@ -67,9 +69,20 @@ public:
 	bool levelSOD = false;
 	bool levelCHR = false;
 	bool levelWOG = false;
-	bool levelHOTA0 = false;
-	bool levelHOTA1 = false;
-	bool levelHOTA3 = false;
+	// older, unsupported hota maps:
+	// 1.0 -> uses SoD format
+	// 1.1 -> uses SoD or WoG format
+	// 1.2 -> uses custom 0x1e / 30 format
+	bool levelHOTA0 = false; // 1.3.0
+	bool levelHOTA1 = false; // 1.5.0
+	bool levelHOTA2 = false; // 1.6.0
+	bool levelHOTA3 = false; // 1.6.0
+	// level 4 - not released publicly?
+	bool levelHOTA5 = false; // 1.7.0
+	bool levelHOTA6 = false; // 1.7.1
+	bool levelHOTA7 = false; // 1.7.2
+	bool levelHOTA8 = false; // 1.7.3
+	bool levelHOTA9 = false; // 1.8.0
 };
 
 VCMI_LIB_NAMESPACE_END
