@@ -13,16 +13,20 @@
 
 VCMI_LIB_NAMESPACE_BEGIN
 
-class NetworkHandler : public INetworkHandler
+class NetworkHandler final : public INetworkHandler
 {
-	std::shared_ptr<NetworkContext> io;
+	std::unique_ptr<NetworkContext> context;
 
 public:
 	NetworkHandler();
 
 	std::unique_ptr<INetworkServer> createServerTCP(INetworkServerListener & listener) override;
 	void connectToRemote(INetworkClientListener & listener, const std::string & host, uint16_t port) override;
+	void createInternalConnection(INetworkClientListener & listener, INetworkServer & server) override;
+	std::shared_ptr<INetworkConnection> createAsyncConnection(INetworkConnectionListener & listener) override;
 	void createTimer(INetworkTimerListener & listener, std::chrono::milliseconds duration) override;
+	std::shared_ptr<IServerDiscovery> createServerDiscovery(IServerDiscoveryObserver & listener) override;
+	std::shared_ptr<IServerDiscoveryListener> createServerDiscoveryListener(IServerDiscoveryAnnouncer & announcer, uint16_t port = 3030) override;
 
 	void run() override;
 	void stop() override;
