@@ -17,16 +17,16 @@
 #include "NetPacksBase.h"
 #include "ObjProperty.h"
 
-#include "../CCreatureSet.h"
 #include "../ResourceSet.h"
 #include "../TurnTimerInfo.h"
+#include "../bonuses/Bonus.h"
 #include "../gameState/EVictoryLossCheckResult.h"
 #include "../gameState/RumorState.h"
 #include "../gameState/QuestInfo.h"
 #include "../gameState/TavernSlot.h"
 #include "../gameState/GameStatistics.h"
 #include "../int3.h"
-#include "../mapping/CMapDefines.h"
+#include "../mapObjects/army/CSimpleArmy.h"
 #include "../spells/ViewSpellInt.h"
 
 class CClient;
@@ -48,28 +48,64 @@ class BattleInfo;
 // For now it's will be there till teleports code refactored and moved into own file
 using TTeleportExitsList = std::vector<std::pair<ObjectInstanceID, int3>>;
 
+using FowTilesType = std::set<int3>;
+
 /***********************************************************************************************************/
 struct DLL_LINKAGE PackageApplied : public CPackForClient
 {
 	PackageApplied() = default;
-	explicit PackageApplied(ui8 Result)
-		: result(Result)
+	PackageApplied(PlayerColor player, uint32_t requestID, uint16_t packType, bool result)
+		: player(player)
+		, requestID(requestID)
+		, packType(packType)
+		, result(result)
 	{
 	}
-	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState *gs) override {}
 
-	ui8 result = 0; //0 - something went wrong, request hasn't been realized; 1 - OK
-	ui32 packType = 0; //type id of applied package
-	ui32 requestID = 0; //an ID given by client to the request that was applied
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	/// ID of player that sent this package
 	PlayerColor player;
+	/// request ID, as provided by player
+	uint32_t requestID = 0;
+	/// type id of applied package
+	uint16_t packType = 0;
+	/// If false, then pack failed to apply, for example - illegal request
+	bool result = false;
 
 	template <typename Handler> void serialize(Handler & h)
 	{
-		h & result;
-		h & packType;
-		h & requestID;
 		h & player;
+		h & requestID;
+		h & packType;
+		h & result;
+	}
+};
+
+struct DLL_LINKAGE PackageReceived : public CPackForClient
+{
+	PackageReceived() = default;
+	PackageReceived(PlayerColor player, uint32_t requestID, uint16_t packType)
+		: player(player)
+		, requestID(requestID)
+		, packType(packType)
+	{
+	}
+
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	/// ID of player that sent this package
+	PlayerColor player;
+	/// request ID, as provided by player
+	uint32_t requestID;
+	/// type id of applied package
+	uint16_t packType;
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & player;
+		h & requestID;
+		h & packType;
 	}
 };
 
@@ -82,7 +118,6 @@ struct DLL_LINKAGE SystemMessage : public CPackForClient
 	SystemMessage() = default;
 
 	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState *gs) override {}
 
 	MetaString text;
 
@@ -102,7 +137,6 @@ struct DLL_LINKAGE PlayerBlocked : public CPackForClient
 	PlayerColor player;
 
 	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState *gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -114,28 +148,31 @@ struct DLL_LINKAGE PlayerBlocked : public CPackForClient
 
 struct DLL_LINKAGE PlayerCheated : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	PlayerColor player;
+	bool localOnlyCheat = false;
+
 	bool losingCheatCode = false;
 	bool winningCheatCode = false;
+	ColorScheme colorScheme = ColorScheme::KEEP;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
 	{
 		h & player;
+		h & localOnlyCheat;
 		h & losingCheatCode;
 		h & winningCheatCode;
+		h & colorScheme;
 	}
 };
 
 struct DLL_LINKAGE TurnTimeUpdate : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-	
 	PlayerColor player;
 	TurnTimerInfo turnTimer;
+
+	void visitTyped(ICPackVisitor & visitor) override;
 		
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -146,8 +183,6 @@ struct DLL_LINKAGE TurnTimeUpdate : public CPackForClient
 
 struct DLL_LINKAGE PlayerStartsTurn : public Query
 {
-	void applyGs(CGameState * gs) override;
-
 	PlayerColor player;
 
 	void visitTyped(ICPackVisitor & visitor) override;
@@ -161,8 +196,6 @@ struct DLL_LINKAGE PlayerStartsTurn : public Query
 
 struct DLL_LINKAGE DaysWithoutTown : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	PlayerColor player;
 	std::optional<int32_t> daysWithoutCastle;
 
@@ -179,8 +212,6 @@ struct DLL_LINKAGE EntitiesChanged : public CPackForClient
 {
 	std::vector<EntityChanges> changes;
 
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -191,56 +222,80 @@ struct DLL_LINKAGE EntitiesChanged : public CPackForClient
 
 struct DLL_LINKAGE SetResources : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	bool abs = true; //false - changes by value; 1 - sets to value
+	ChangeValueMode mode = ChangeValueMode::ABSOLUTE;
 	PlayerColor player;
 	ResourceSet res; //res[resid] => res amount
 
 	template <typename Handler> void serialize(Handler & h)
 	{
-		h & abs;
+		h & mode;
 		h & player;
 		h & res;
 	}
 };
 
-struct DLL_LINKAGE SetPrimSkill : public CPackForClient
+struct DLL_LINKAGE SetPrimarySkill : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	ui8 abs = 0; //0 - changes by value; 1 - sets to value
+	ChangeValueMode mode = ChangeValueMode::RELATIVE;
 	ObjectInstanceID id;
 	PrimarySkill which = PrimarySkill::ATTACK;
 	si64 val = 0;
 
 	template <typename Handler> void serialize(Handler & h)
 	{
-		h & abs;
+		h & mode;
 		h & id;
 		h & which;
 		h & val;
 	}
 };
 
-struct DLL_LINKAGE SetSecSkill : public CPackForClient
+struct DLL_LINKAGE SetHeroExperience : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	ui8 abs = 0; //0 - changes by value; 1 - sets to value
+	ChangeValueMode mode = ChangeValueMode::RELATIVE;
+	ObjectInstanceID id;
+	si64 val = 0;
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & mode;
+		h & id;
+		h & val;
+	}
+};
+
+struct DLL_LINKAGE GiveStackExperience : public CPackForClient
+{
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	ObjectInstanceID id;
+	std::map<SlotID, si64> val;
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & id;
+		h & val;
+	}
+};
+
+struct DLL_LINKAGE SetSecSkill : public CPackForClient
+{
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	ChangeValueMode mode = ChangeValueMode::RELATIVE;
 	ObjectInstanceID id;
 	SecondarySkill which;
 	ui16 val = 0;
 
 	template <typename Handler> void serialize(Handler & h)
 	{
-		h & abs;
+		h & mode;
 		h & id;
 		h & which;
 		h & val;
@@ -249,22 +304,25 @@ struct DLL_LINKAGE SetSecSkill : public CPackForClient
 
 struct DLL_LINKAGE HeroVisitCastle : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
-	ui8 flags = 0; //1 - start
+	bool startVisit = false;
 	ObjectInstanceID tid;
 	ObjectInstanceID hid;
 
 	bool start() const //if hero is entering castle (if false - leaving)
 	{
-		return flags & 1;
+		return startVisit;
+	}
+
+	bool leave() const //if hero is entering castle (if false - leaving)
+	{
+		return !startVisit;
 	}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
-		h & flags;
+		h & startVisit;
 		h & tid;
 		h & hid;
 	}
@@ -272,8 +330,6 @@ struct DLL_LINKAGE HeroVisitCastle : public CPackForClient
 
 struct DLL_LINKAGE ChangeSpells : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	ui8 learn = 1; //1 - gives spell, 0 - takes
@@ -290,8 +346,6 @@ struct DLL_LINKAGE ChangeSpells : public CPackForClient
 
 struct DLL_LINKAGE SetResearchedSpells : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	ui8 level = 0;
@@ -310,43 +364,37 @@ struct DLL_LINKAGE SetResearchedSpells : public CPackForClient
 
 struct DLL_LINKAGE SetMana : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	SetMana() = default;
-	SetMana(ObjectInstanceID hid, si32 val, bool absolute)
+	SetMana(ObjectInstanceID hid, si32 val, ChangeValueMode mode)
 		: hid(hid)
 		, val(val)
-		, absolute(absolute)
+		, mode(mode)
 	{}
 
 	ObjectInstanceID hid;
 	si32 val = 0;
-	bool absolute = true;
+	ChangeValueMode mode = ChangeValueMode::RELATIVE;
 
 	template <typename Handler> void serialize(Handler & h)
 	{
 		h & val;
 		h & hid;
-		h & absolute;
+		h & mode;
 	}
 };
 
 struct DLL_LINKAGE SetMovePoints : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	SetMovePoints() = default;
-	SetMovePoints(ObjectInstanceID hid, si32 val, bool absolute)
+	SetMovePoints(ObjectInstanceID hid, si32 val)
 		: hid(hid)
 		, val(val)
-		, absolute(absolute)
 	{}
 
 	ObjectInstanceID hid;
 	si32 val = 0;
-	bool absolute = true;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -354,15 +402,12 @@ struct DLL_LINKAGE SetMovePoints : public CPackForClient
 	{
 		h & val;
 		h & hid;
-		h & absolute;
 	}
 };
 
 struct DLL_LINKAGE FoWChange : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
-	std::unordered_set<int3> tiles;
+	FowTilesType tiles;
 	PlayerColor player;
 	ETileVisibility mode;
 	bool waitForDialogs = false;
@@ -384,7 +429,6 @@ struct DLL_LINKAGE SetAvailableHero : public CPackForClient
 	{
 		army.clearSlots();
 	}
-	void applyGs(CGameState * gs) override;
 
 	TavernHeroSlot slotID;
 	TavernSlotRole roleID;
@@ -408,17 +452,29 @@ struct DLL_LINKAGE SetAvailableHero : public CPackForClient
 
 struct DLL_LINKAGE GiveBonus : public CPackForClient
 {
-	enum class ETarget : int8_t { OBJECT, PLAYER, BATTLE };
-	
+	using VariantType = VariantIdentifier<ObjectInstanceID, PlayerColor, BattleID>;
+	enum class ETarget : int8_t
+	{
+		OBJECT,
+		PLAYER,
+		BATTLE,
+		HERO_COMMANDER
+	};
+
 	explicit GiveBonus(ETarget Who = ETarget::OBJECT)
 		:who(Who)
 	{
 	}
 
-	void applyGs(CGameState * gs) override;
+	GiveBonus(ETarget who, const VariantType & id, const Bonus & bonus)
+		: who(who)
+		, id(id)
+		, bonus(bonus)
+	{
+	}
 
 	ETarget who = ETarget::OBJECT;
-	VariantIdentifier<ObjectInstanceID, PlayerColor, BattleID> id;
+	VariantType id;
 	Bonus bonus;
 
 	void visitTyped(ICPackVisitor & visitor) override;
@@ -434,8 +490,6 @@ struct DLL_LINKAGE GiveBonus : public CPackForClient
 
 struct DLL_LINKAGE ChangeObjPos : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	/// Object to move
 	ObjectInstanceID objid;
 	/// New position of visitable tile of an object
@@ -455,8 +509,6 @@ struct DLL_LINKAGE ChangeObjPos : public CPackForClient
 
 struct DLL_LINKAGE PlayerEndsTurn : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	PlayerColor player;
 
 	void visitTyped(ICPackVisitor & visitor) override;
@@ -469,11 +521,10 @@ struct DLL_LINKAGE PlayerEndsTurn : public CPackForClient
 
 struct DLL_LINKAGE PlayerEndsGame : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	PlayerColor player;
 	EVictoryLossCheckResult victoryLossCheckResult;
 	StatisticDataSet statistic;
+	bool silentEnd = false;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -481,24 +532,8 @@ struct DLL_LINKAGE PlayerEndsGame : public CPackForClient
 	{
 		h & player;
 		h & victoryLossCheckResult;
-		if (h.version >= Handler::Version::STATISTICS_SCREEN)
-			h & statistic;
-	}
-};
-
-struct DLL_LINKAGE PlayerReinitInterface : public CPackForClient
-{
-	void applyGs(CGameState * gs) override;
-
-	std::vector<PlayerColor> players;
-	ui8 playerConnectionId; //PLAYER_AI for AI player
-
-	void visitTyped(ICPackVisitor & visitor) override;
-
-	template <typename Handler> void serialize(Handler & h)
-	{
-		h & players;
-		h & playerConnectionId;
+		h & statistic;
+		h & silentEnd;
 	}
 };
 
@@ -509,8 +544,6 @@ struct DLL_LINKAGE RemoveBonus : public CPackForClient
 	{
 	}
 
-	void applyGs(CGameState * gs) override;
-
 	GiveBonus::ETarget who; //who receives bonus
 	VariantIdentifier<HeroTypeID, PlayerColor, BattleID, ObjectInstanceID> whoID;
 
@@ -536,8 +569,6 @@ struct DLL_LINKAGE SetCommanderProperty : public CPackForClient
 {
 	enum ECommanderProperty { ALIVE, BONUS, SECONDARY_SKILL, EXPERIENCE, SPECIAL_SKILL };
 
-	void applyGs(CGameState * gs) override;
-
 	ObjectInstanceID heroid;
 
 	ECommanderProperty which = ALIVE;
@@ -559,8 +590,6 @@ struct DLL_LINKAGE SetCommanderProperty : public CPackForClient
 
 struct DLL_LINKAGE AddQuest : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	PlayerColor player;
 	QuestInfo quest;
 
@@ -573,25 +602,11 @@ struct DLL_LINKAGE AddQuest : public CPackForClient
 	}
 };
 
-struct DLL_LINKAGE UpdateArtHandlerLists : public CPackForClient
-{
-	std::map<ArtifactID, int> allocatedArtifacts;
-
-	void applyGs(CGameState * gs) override;
-	void visitTyped(ICPackVisitor & visitor) override;
-
-	template <typename Handler> void serialize(Handler & h)
-	{
-		h & allocatedArtifacts;
-	}
-};
-
 struct DLL_LINKAGE ChangeFormation : public CPackForClient
 {
 	ObjectInstanceID hid;
 	EArmyFormation formation{};
 
-	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -601,6 +616,34 @@ struct DLL_LINKAGE ChangeFormation : public CPackForClient
 	}
 };
 
+struct DLL_LINKAGE ChangeTactics : public CPackForClient
+{
+	ObjectInstanceID hid;
+	bool enabled = false;
+
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & hid;
+		h & enabled;
+	}
+};
+
+struct DLL_LINKAGE ChangeTownName : public CPackForClient
+{
+	ObjectInstanceID tid;
+	std::string name;
+
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & tid;
+		h & name;
+	}
+};
+
 struct DLL_LINKAGE RemoveObject : public CPackForClient
 {
 	RemoveObject() = default;
@@ -610,7 +653,6 @@ struct DLL_LINKAGE RemoveObject : public CPackForClient
 	{
 	}
 
-	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	/// ID of removed object
@@ -628,8 +670,6 @@ struct DLL_LINKAGE RemoveObject : public CPackForClient
 
 struct DLL_LINKAGE TryMoveHero : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	enum EResult
 	{
 		FAILED,
@@ -640,13 +680,20 @@ struct DLL_LINKAGE TryMoveHero : public CPackForClient
 		DISEMBARK
 	};
 
+	/// ID of moved hero
 	ObjectInstanceID id;
+	/// Movement points that hero will have after movement
 	ui32 movePoints = 0;
-	EResult result = FAILED; //uses EResult
-	int3 start; //h3m format
+	/// Result of movement attempt. FAILED should generally never happen unless client requested invalid operation
+	EResult result = FAILED;
+	/// Hero anchor position from which hero moves
+	int3 start;
+	/// Hero anchor position to which hero moves
 	int3 end;
-	std::unordered_set<int3> fowRevealed; //revealed tiles
-	std::optional<int3> attackedFrom; // Set when stepping into endangered tile.
+	/// Tiles that were revealed by this move
+	FowTilesType fowRevealed;
+	/// If hero moves on guarded tile, this field will be set to visitable pos of attacked wandering monster
+	int3 attackedFrom;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -664,13 +711,17 @@ struct DLL_LINKAGE TryMoveHero : public CPackForClient
 		h & movePoints;
 		h & fowRevealed;
 		h & attackedFrom;
+
+		std::string fow;
+		for (const auto & tile : fowRevealed)
+			fow += tile.toString() + ", ";
+
+		logGlobal->trace("OI %d, mp %d, res %d, start %s, end %s, attack %s, fow %s", id.getNum(), movePoints, static_cast<int>(result), start.toString(), end.toString(), attackedFrom.toString(), fow);
 	}
 };
 
 struct DLL_LINKAGE NewStructures : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	ObjectInstanceID tid;
 	std::set<BuildingID> bid;
 	si16 built = 0;
@@ -687,8 +738,6 @@ struct DLL_LINKAGE NewStructures : public CPackForClient
 
 struct DLL_LINKAGE RazeStructures : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	ObjectInstanceID tid;
 	std::set<BuildingID> bid;
 	si16 destroyed = 0;
@@ -705,8 +754,6 @@ struct DLL_LINKAGE RazeStructures : public CPackForClient
 
 struct DLL_LINKAGE SetAvailableCreatures : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	ObjectInstanceID tid;
 	std::vector<std::pair<ui32, std::vector<CreatureID> > > creatures;
 
@@ -721,8 +768,6 @@ struct DLL_LINKAGE SetAvailableCreatures : public CPackForClient
 
 struct DLL_LINKAGE SetHeroesInTown : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	ObjectInstanceID tid; //id of town
 	ObjectInstanceID visiting; //id of visiting hero
 	ObjectInstanceID garrison; //id of hero in garrison
@@ -739,8 +784,6 @@ struct DLL_LINKAGE SetHeroesInTown : public CPackForClient
 
 struct DLL_LINKAGE HeroRecruited : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	HeroTypeID hid; //subID of hero
 	ObjectInstanceID tid;
 	ObjectInstanceID boatId;
@@ -761,8 +804,6 @@ struct DLL_LINKAGE HeroRecruited : public CPackForClient
 
 struct DLL_LINKAGE GiveHero : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	ObjectInstanceID id; //object id
 	ObjectInstanceID boatId;
 	PlayerColor player;
@@ -784,7 +825,6 @@ struct DLL_LINKAGE OpenWindow : public Query
 	ObjectInstanceID visitor;
 
 	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState *gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -797,10 +837,8 @@ struct DLL_LINKAGE OpenWindow : public Query
 
 struct DLL_LINKAGE NewObject : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	/// Object ID to create
-	CGObjectInstance * newObject;
+	std::shared_ptr<CGObjectInstance> newObject;
 	/// Which player initiated creation of this object
 	PlayerColor initiator;
 
@@ -815,8 +853,6 @@ struct DLL_LINKAGE NewObject : public CPackForClient
 
 struct DLL_LINKAGE SetAvailableArtifacts : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	//two variants: id < 0: set artifact pool for Artifact Merchants in towns; id >= 0: set pool for adv. map Black Market (id is the id of Black Market instance then)
 	ObjectInstanceID id;
 	std::vector<ArtifactID> arts;
@@ -839,9 +875,7 @@ struct DLL_LINKAGE ChangeStackCount : CGarrisonOperationPack
 	ObjectInstanceID army;
 	SlotID slot;
 	TQuantity count;
-	bool absoluteValue; //if not -> count will be added (or subtracted if negative)
-
-	void applyGs(CGameState * gs) override;
+	ChangeValueMode mode = ChangeValueMode::RELATIVE;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -850,7 +884,7 @@ struct DLL_LINKAGE ChangeStackCount : CGarrisonOperationPack
 		h & army;
 		h & slot;
 		h & count;
-		h & absoluteValue;
+		h & mode;
 	}
 };
 
@@ -860,8 +894,6 @@ struct DLL_LINKAGE SetStackType : CGarrisonOperationPack
 	SlotID slot;
 	CreatureID type;
 
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -877,7 +909,6 @@ struct DLL_LINKAGE EraseStack : CGarrisonOperationPack
 	ObjectInstanceID army;
 	SlotID slot;
 
-	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -894,7 +925,6 @@ struct DLL_LINKAGE SwapStacks : CGarrisonOperationPack
 	SlotID srcSlot;
 	SlotID dstSlot;
 
-	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -913,7 +943,6 @@ struct DLL_LINKAGE InsertNewStack : CGarrisonOperationPack
 	CreatureID type;
 	TQuantity count = 0;
 
-	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -935,7 +964,6 @@ struct DLL_LINKAGE RebalanceStacks : CGarrisonOperationPack
 
 	TQuantity count;
 
-	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -952,7 +980,6 @@ struct DLL_LINKAGE BulkRebalanceStacks : CGarrisonOperationPack
 {
 	std::vector<RebalanceStacks> moves;
 
-	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler>
@@ -962,26 +989,28 @@ struct DLL_LINKAGE BulkRebalanceStacks : CGarrisonOperationPack
 	}
 };
 
-struct DLL_LINKAGE BulkSmartRebalanceStacks : CGarrisonOperationPack
-{
-	std::vector<RebalanceStacks> moves;
-	std::vector<ChangeStackCount> changes;
-
-	void applyGs(CGameState * gs) override;
-	void visitTyped(ICPackVisitor & visitor) override;
-
-	template <typename Handler>
-	void serialize(Handler & h)
-	{
-		h & moves;
-		h & changes;
-	}
-};
-
 struct DLL_LINKAGE CArtifactOperationPack : CPackForClient
 {
 };
 
+struct DLL_LINKAGE GrowUpArtifact : CArtifactOperationPack
+{
+	ArtifactInstanceID id;
+
+	GrowUpArtifact() = default;
+	GrowUpArtifact(const ArtifactInstanceID & id)
+		: id(id)
+	{
+	}
+
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & id;
+	}
+};
+
 struct DLL_LINKAGE PutArtifact : CArtifactOperationPack
 {
 	PutArtifact() = default;
@@ -994,7 +1023,6 @@ struct DLL_LINKAGE PutArtifact : CArtifactOperationPack
 	bool askAssemble;
 	ArtifactInstanceID id;
 
-	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -1012,7 +1040,6 @@ struct DLL_LINKAGE NewArtifact : public CArtifactOperationPack
 	SpellID spellId;
 	ArtifactPosition pos;
 
-	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -1030,7 +1057,6 @@ struct DLL_LINKAGE BulkEraseArtifacts : CArtifactOperationPack
 	std::vector<ArtifactPosition> posPack;
 	std::optional<SlotID> creature;
 
-	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -1043,27 +1069,6 @@ struct DLL_LINKAGE BulkEraseArtifacts : CArtifactOperationPack
 
 struct DLL_LINKAGE BulkMoveArtifacts : CArtifactOperationPack
 {
-	struct LinkedSlots
-	{
-		ArtifactPosition srcPos;
-		ArtifactPosition dstPos;
-		bool askAssemble;
-
-		LinkedSlots() = default;
-		LinkedSlots(const ArtifactPosition & srcPos, const ArtifactPosition & dstPos, bool askAssemble = false)
-			: srcPos(srcPos)
-			, dstPos(dstPos)
-			, askAssemble(askAssemble)
-		{
-		}
-		template <typename Handler> void serialize(Handler & h)
-		{
-			h & srcPos;
-			h & dstPos;
-			h & askAssemble;
-		}
-	};
-
 	PlayerColor interfaceOwner;
 	ObjectInstanceID srcArtHolder;
 	ObjectInstanceID dstArtHolder;
@@ -1087,10 +1092,8 @@ struct DLL_LINKAGE BulkMoveArtifacts : CArtifactOperationPack
 	{
 	}
 
-	void applyGs(CGameState * gs) override;
-
-	std::vector<LinkedSlots> artsPack0;
-	std::vector<LinkedSlots> artsPack1;
+	std::vector<MoveArtifactInfo> artsPack0;
+	std::vector<MoveArtifactInfo> artsPack1;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
@@ -1106,13 +1109,34 @@ struct DLL_LINKAGE BulkMoveArtifacts : CArtifactOperationPack
 	}
 };
 
+struct DLL_LINKAGE DischargeArtifact : CArtifactOperationPack
+{
+	ArtifactInstanceID id;
+	uint16_t charges;
+	std::optional<ArtifactLocation> artLoc;
+
+	DischargeArtifact() = default;
+	DischargeArtifact(const ArtifactInstanceID & id, const uint16_t charges)
+		: id(id)
+		, charges(charges)
+	{
+	}
+
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & id;
+		h & charges;
+		h & artLoc;
+	}
+};
+
 struct DLL_LINKAGE AssembledArtifact : CArtifactOperationPack
 {
 	ArtifactLocation al;
 	ArtifactID artId;
 
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -1126,8 +1150,6 @@ struct DLL_LINKAGE DisassembledArtifact : CArtifactOperationPack
 {
 	ArtifactLocation al;
 
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -1144,8 +1166,6 @@ struct DLL_LINKAGE HeroVisit : public CPackForClient
 
 	bool starting; //false -> ending
 
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -1166,7 +1186,6 @@ struct DLL_LINKAGE InfoWindow : public CPackForClient //103  - displays simple i
 	ui16 soundID = 0;
 
 	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1181,8 +1200,6 @@ struct DLL_LINKAGE InfoWindow : public CPackForClient //103  - displays simple i
 
 struct DLL_LINKAGE NewTurn : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	ui32 day = 0;
@@ -1214,7 +1231,6 @@ struct DLL_LINKAGE NewTurn : public CPackForClient
 
 struct DLL_LINKAGE SetObjectProperty : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
 	ObjectInstanceID id;
 	ObjProperty what{};
 
@@ -1238,7 +1254,6 @@ struct DLL_LINKAGE ChangeObjectVisitors : public CPackForClient
 	{
 		VISITOR_ADD_HERO,   // mark hero as one that have visited this object
 		VISITOR_ADD_PLAYER, // mark player as one that have visited this object instance
-		VISITOR_GLOBAL,     // mark player as one that have visited object of this type
 		VISITOR_SCOUTED,    // marks targeted team as having scouted this object
 		VISITOR_CLEAR,      // clear all visitors from this object (object reset)
 	};
@@ -1246,8 +1261,6 @@ struct DLL_LINKAGE ChangeObjectVisitors : public CPackForClient
 	ObjectInstanceID object;
 	ObjectInstanceID hero; // note: hero owner will be also marked as "visited" this object
 
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	ChangeObjectVisitors() = default;
@@ -1273,7 +1286,6 @@ struct DLL_LINKAGE ChangeArtifactsCostume : public CPackForClient
 	uint32_t costumeIdx = 0;
 	const PlayerColor player = PlayerColor::NEUTRAL;
 
-	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	ChangeArtifactsCostume() = default;
@@ -1299,8 +1311,6 @@ struct DLL_LINKAGE HeroLevelUp : public Query
 	PrimarySkill primskill = PrimarySkill::ATTACK;
 	std::vector<SecondarySkill> skills;
 
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -1320,8 +1330,6 @@ struct DLL_LINKAGE CommanderLevelUp : public Query
 
 	std::vector<ui32> skills; //0-5 - secondary skills, val-100 - special skill
 
-	void applyGs(CGameState * gs) override;
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
@@ -1367,7 +1375,6 @@ struct DLL_LINKAGE BlockingDialog : public Query
 	BlockingDialog() = default;
 
 	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1387,7 +1394,6 @@ struct DLL_LINKAGE GarrisonDialog : public Query
 	bool removableUnits = false;
 
 	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1406,7 +1412,6 @@ struct DLL_LINKAGE ExchangeDialog : public Query
 	ObjectInstanceID hero2;
 
 	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1432,7 +1437,6 @@ struct DLL_LINKAGE TeleportDialog : public Query
 	bool impassable = false;
 
 	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1453,7 +1457,6 @@ struct DLL_LINKAGE MapObjectSelectDialog : public Query
 	std::vector<ObjectInstanceID> objects;
 
 	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1478,7 +1481,6 @@ struct DLL_LINKAGE AdvmapSpellCast : public CPackForClient
 
 protected:
 	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState * gs) override {}
 };
 
 struct DLL_LINKAGE ShowWorldViewEx : public CPackForClient
@@ -1488,8 +1490,6 @@ struct DLL_LINKAGE ShowWorldViewEx : public CPackForClient
 
 	std::vector<ObjectPosInfo> objectPositions;
 
-	void applyGs(CGameState * gs) override {}
-
 	template <typename Handler> void serialize(Handler & h)
 	{
 		h & player;
@@ -1510,7 +1510,6 @@ struct DLL_LINKAGE PlayerMessageClient : public CPackForClient
 	{
 	}
 	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState * gs) override {}
 
 	PlayerColor player;
 	std::string text;
@@ -1529,7 +1528,6 @@ struct DLL_LINKAGE CenterView : public CPackForClient
 	ui32 focusTime = 0; //ms
 
 	void visitTyped(ICPackVisitor & visitor) override;
-	void applyGs(CGameState * gs) override {}
 
 	template <typename Handler> void serialize(Handler & h)
 	{
@@ -1539,4 +1537,18 @@ struct DLL_LINKAGE CenterView : public CPackForClient
 	}
 };
 
+struct DLL_LINKAGE ResponseStatistic : public CPackForClient
+{
+	PlayerColor player;
+	StatisticDataSet statistic;
+
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & player;
+		h & statistic;
+	}
+};
+
 VCMI_LIB_NAMESPACE_END
