@@ -21,21 +21,11 @@ VCMI_LIB_NAMESPACE_BEGIN
 struct DLL_LINKAGE GamePause : public CPackForServer
 {
 	void visitTyped(ICPackVisitor & visitor) override;
-
-	template <typename Handler> void serialize(Handler & h)
-	{
-		h & static_cast<CPackForServer &>(*this);
-	}
 };
 
 struct DLL_LINKAGE EndTurn : public CPackForServer
 {
 	void visitTyped(ICPackVisitor & visitor) override;
-
-	template <typename Handler> void serialize(Handler & h)
-	{
-		h & static_cast<CPackForServer &>(*this);
-	}
 };
 
 struct DLL_LINKAGE DismissHero : public CPackForServer
@@ -214,14 +204,14 @@ struct DLL_LINKAGE BulkMergeStacks : public CPackForServer
 	}
 };
 
-struct DLL_LINKAGE BulkSmartSplitStack : public CPackForServer
+struct DLL_LINKAGE BulkSplitAndRebalanceStack : public CPackForServer
 {
 	SlotID src;
 	ObjectInstanceID srcOwner;
 
-	BulkSmartSplitStack() = default;
+	BulkSplitAndRebalanceStack() = default;
 
-	BulkSmartSplitStack(const ObjectInstanceID & srcOwner, const SlotID & src)
+	BulkSplitAndRebalanceStack(const ObjectInstanceID & srcOwner, const SlotID & src)
 		: src(src)
 		, srcOwner(srcOwner)
 	{
@@ -609,6 +599,50 @@ struct DLL_LINKAGE SetFormation : public CPackForServer
 	}
 };
 
+struct DLL_LINKAGE SetTactics : public CPackForServer
+{
+	SetTactics() = default;
+	;
+	SetTactics(const ObjectInstanceID & HID, bool Enabled)
+		: hid(HID)
+		, enabled(Enabled)
+	{
+	}
+	ObjectInstanceID hid;
+	bool enabled = false;
+
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & static_cast<CPackForServer &>(*this);
+		h & hid;
+		h & enabled;
+	}
+};
+
+struct DLL_LINKAGE SetTownName : public CPackForServer
+{
+	SetTownName() = default;
+	;
+	SetTownName(const ObjectInstanceID & TID, std::string Name)
+		: tid(TID)
+		, name(Name)
+	{
+	}
+	ObjectInstanceID tid;
+	std::string name;
+
+	void visitTyped(ICPackVisitor & visitor) override;
+
+	template <typename Handler> void serialize(Handler & h)
+	{
+		h & static_cast<CPackForServer &>(*this);
+		h & tid;
+		h & name;
+	}
+};
+
 struct DLL_LINKAGE HireHero : public CPackForServer
 {
 	HireHero() = default;
@@ -657,7 +691,6 @@ struct DLL_LINKAGE QueryReply : public CPackForServer
 	{
 	}
 	QueryID qid;
-	PlayerColor player;
 	std::optional<int32_t> reply;
 
 	void visitTyped(ICPackVisitor & visitor) override;
@@ -666,7 +699,6 @@ struct DLL_LINKAGE QueryReply : public CPackForServer
 	{
 		h & static_cast<CPackForServer &>(*this);
 		h & qid;
-		h & player;
 		h & reply;
 	}
 };
@@ -721,24 +753,30 @@ struct DLL_LINKAGE CastAdvSpell : public CPackForServer
 	}
 };
 
+struct DLL_LINKAGE RequestStatistic : public CPackForServer
+{
+	void visitTyped(ICPackVisitor & visitor) override;
+};
+
 /***********************************************************************************************************/
 
 struct DLL_LINKAGE SaveGame : public CPackForServer
 {
 	SaveGame() = default;
-	SaveGame(std::string Fname)
+	SaveGame(std::string Fname, bool NotifySuccess)
 		: fname(std::move(Fname))
+		, notifySuccess(NotifySuccess)
 	{
 	}
 	std::string fname;
-
-	void applyGs(CGameState * gs) {};
+	bool notifySuccess = false;
 
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	template <typename Handler> void serialize(Handler & h)
 	{
 		h & static_cast<CPackForServer &>(*this);
+		h & notifySuccess;
 		h & fname;
 	}
 };
@@ -752,8 +790,6 @@ struct DLL_LINKAGE PlayerMessage : public CPackForServer
 	{
 	}
 
-	void applyGs(CGameState * gs) {};
-
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	std::string text;
@@ -767,4 +803,11 @@ struct DLL_LINKAGE PlayerMessage : public CPackForServer
 	}
 };
 
+struct DLL_LINKAGE AdvInterfaceReady : public CPackForServer
+{
+	AdvInterfaceReady() = default;
+
+	void visitTyped(ICPackVisitor & cpackVisitor) override;
+};
+
 VCMI_LIB_NAMESPACE_END
