@@ -12,13 +12,11 @@
 #include "NetPacksBase.h"
 
 #include "../rewardable/Configuration.h"
-#include "../mapObjectConstructors/CBankInstanceConstructor.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
 struct DLL_LINKAGE SetRewardableConfiguration : public CPackForClient
 {
-	void applyGs(CGameState * gs) override;
 	void visitTyped(ICPackVisitor & visitor) override;
 
 	ObjectInstanceID objectID;
@@ -33,19 +31,4 @@ struct DLL_LINKAGE SetRewardableConfiguration : public CPackForClient
 	}
 };
 
-struct DLL_LINKAGE SetBankConfiguration : public CPackForClient
-{
-	void applyGs(CGameState * gs) override;
-	void visitTyped(ICPackVisitor & visitor) override;
-
-	ObjectInstanceID objectID;
-	BankConfig configuration;
-
-	template <typename Handler> void serialize(Handler & h)
-	{
-		h & objectID;
-		h & configuration;
-	}
-};
-
 VCMI_LIB_NAMESPACE_END
