@@ -12,9 +12,9 @@
 
 #include "CPathfinder.h"
 
-#include "../gameState/CGameState.h"
+#include "../callback/IGameInfoCallback.h"
 #include "../mapObjects/CGHeroInstance.h"
-#include "../mapping/CMapDefines.h"
+#include "../mapping/TerrainTile.h"
 
 VCMI_LIB_NAMESPACE_BEGIN
 
@@ -100,50 +100,53 @@ PathNodeInfo::PathNodeInfo()
 {
 }
 
-void PathNodeInfo::setNode(CGameState * gs, CGPathNode * n)
+void PathNodeInfo::setNode(const IGameInfoCallback & gameInfo, CGPathNode * n)
 {
 	node = n;
+	guarded = false;
 
 	if(coord != node->coord)
 	{
-		assert(node->coord.valid());
+		assert(node->coord.isValid());
 
 		coord = node->coord;
-		tile = gs->getTile(coord);
-		nodeObject = tile->topVisitableObj();
+		tile = gameInfo.getTile(coord);
+		nodeObject = nullptr;
+		nodeHero = nullptr;
 
-		if(nodeObject && nodeObject->ID == Obj::HERO)
+		ObjectInstanceID topObjectID = tile->topVisitableObj();
+		if (topObjectID.hasValue())
 		{
-			nodeHero = dynamic_cast<const CGHeroInstance *>(nodeObject);
-			nodeObject = tile->topVisitableObj(true);
+			nodeObject = gameInfo.getObjInstance(topObjectID);
 
-			if(!nodeObject)
-				nodeObject = nodeHero;
-		}
-		else
-		{
-			nodeHero = nullptr;
+			if (nodeObject->ID == Obj::HERO)
+			{
+				nodeHero = dynamic_cast<const CGHeroInstance *>(nodeObject);
+				ObjectInstanceID bottomObjectID = tile->topVisitableObj(true);
+
+				if (bottomObjectID.hasValue())
+					nodeObject = gameInfo.getObjInstance(bottomObjectID);
+			}
 		}
 	}
 
-	guarded = false;
 }
 
-void PathNodeInfo::updateInfo(CPathfinderHelper * hlp, CGameState * gs)
+void PathNodeInfo::updateInfo(CPathfinderHelper * hlp, const IGameInfoCallback & gameInfo)
 {
-	if(gs->guardingCreaturePosition(node->coord).valid() && !isInitialPosition)
+	if(gameInfo.guardingCreaturePosition(node->coord).isValid() && !isInitialPosition)
 	{
 		guarded = true;
 	}
 
 	if(nodeObject)
 	{
-		objectRelations = gs->getPlayerRelations(hlp->owner, nodeObject->tempOwner);
+		objectRelations = gameInfo.getPlayerRelations(hlp->owner, nodeObject->tempOwner);
 	}
 
 	if(nodeHero)
 	{
-		heroRelations = gs->getPlayerRelations(hlp->owner, nodeHero->tempOwner);
+		heroRelations = gameInfo.getPlayerRelations(hlp->owner, nodeHero->tempOwner);
 	}
 }
 
@@ -161,9 +164,9 @@ CDestinationNodeInfo::CDestinationNodeInfo():
 {
 }
 
-void CDestinationNodeInfo::setNode(CGameState * gs, CGPathNode * n)
+void CDestinationNodeInfo::setNode(const IGameInfoCallback & gameInfo, CGPathNode * n)
 {
-	PathNodeInfo::setNode(gs, n);
+	PathNodeInfo::setNode(gameInfo, n);
 
 	blocked = false;
 	action = EPathNodeAction::UNKNOWN;
