@@ -14,16 +14,17 @@
 #include "../TerrainHandler.h"
 #include "../CPlayerState.h"
 #include "../CSoundBase.h"
+#include "../callback/IGameInfoCallback.h"
+#include "../callback/IGameEventCallback.h"
 #include "../entities/hero/CHeroHandler.h"
 #include "../gameState/CGameState.h"
 #include "../spells/CSpellHandler.h"
 #include "../spells/ISpellMechanics.h"
 #include "../mapObjects/CGHeroInstance.h"
 #include "../mapObjects/MiscObjects.h"
-#include "../mapping/CMapDefines.h"
+#include "../mapping/TerrainTile.h"
 #include "../networkPacks/StackLocation.h"
 #include "../networkPacks/PacksForClient.h"
-#include "../IGameCallback.h"
 
 #include <vstd/RNG.h>
 
@@ -43,7 +44,7 @@ std::vector<ui32> Rewardable::Interface::getAvailableRewards(const CGHeroInstanc
 	return ret;
 }
 
-void Rewardable::Interface::grantRewardBeforeLevelup(const Rewardable::VisitInfo & info, const CGHeroInstance * hero) const
+void Rewardable::Interface::grantRewardBeforeLevelup(IGameEventCallback & gameEvents, const Rewardable::VisitInfo & info, const CGHeroInstance * hero) const
 {
 	auto cb = getObject()->cb;
 
@@ -51,7 +52,7 @@ void Rewardable::Interface::grantRewardBeforeLevelup(const Rewardable::VisitInfo
 	assert(hero->tempOwner.isValidPlayer());
 	assert(info.reward.creatures.size() <= GameConstants::ARMY_SIZE);
 
-	cb->giveResources(hero->tempOwner, info.reward.resources);
+	gameEvents.giveResources(hero->tempOwner, info.reward.resources);
 
 	if (info.reward.revealTiles)
 	{
@@ -75,7 +76,7 @@ void Rewardable::Interface::grantRewardBeforeLevelup(const Rewardable::VisitInfo
 			return score > 0;
 		};
 
-		std::unordered_set<int3> tiles;
+		FowTilesType tiles;
 		if (props.radius > 0)
 		{
 			cb->getTilesInRange(tiles, hero->getSightCenter(), props.radius, ETileVisibility::HIDDEN, hero->getOwner());
@@ -93,100 +94,145 @@ void Rewardable::Interface::grantRewardBeforeLevelup(const Rewardable::VisitInfo
 
 		if (props.hide)
 		{
-			for (auto & player : cb->gameState()->players)
+			for (auto & player : cb->gameState().players)
 			{
 				if (cb->getPlayerStatus(player.first) == EPlayerStatus::INGAME && cb->getPlayerRelations(player.first, hero->getOwner()) == PlayerRelations::ENEMIES)
-					cb->changeFogOfWar(tiles, player.first, ETileVisibility::HIDDEN);
+					gameEvents.changeFogOfWar(tiles, player.first, ETileVisibility::HIDDEN);
 			}
 		}
 		else
 		{
-			cb->changeFogOfWar(tiles, hero->getOwner(), ETileVisibility::REVEALED);
+			gameEvents.changeFogOfWar(tiles, hero->getOwner(), ETileVisibility::REVEALED);
 		}
 	}
 
 	for(const auto & entry : info.reward.secondary)
 	{
-		auto currentLevel = static_cast<MasteryLevel::Type>(hero->getSecSkillLevel(entry.first));
-		if(currentLevel == MasteryLevel::EXPERT)
-			continue;
+		int currentLevel = hero->getSecSkillLevel(entry.first);
+		int newLevel = currentLevel + entry.second;
+		int newLevelClamped = std::clamp<int>(newLevel, MasteryLevel::NONE, MasteryLevel::EXPERT);
+		bool canLearn = hero->getSecSkillLevel(entry.first) != 0 || hero->canLearnSkill();
 
-		if(currentLevel != MasteryLevel::NONE || hero->canLearnSkill())
-			cb->changeSecSkill(hero, entry.first, entry.second, false);
+		if(currentLevel != newLevelClamped && canLearn)
+			gameEvents.changeSecSkill(hero, entry.first, newLevelClamped, ChangeValueMode::ABSOLUTE);
 	}
 
 	for(int i=0; i< info.reward.primary.size(); i++)
 		if (info.reward.primary[i] != 0)
-			cb->changePrimSkill(hero, static_cast<PrimarySkill>(i), info.reward.primary[i], false);
+			gameEvents.changePrimSkill(hero, static_cast<PrimarySkill>(i), info.reward.primary[i], ChangeValueMode::RELATIVE);
 
 	TExpType expToGive = 0;
 
 	if (info.reward.heroLevel > 0)
-		expToGive += VLC->heroh->reqExp(hero->level+info.reward.heroLevel) - VLC->heroh->reqExp(hero->level);
+		expToGive += LIBRARY->heroh->reqExp(hero->level+info.reward.heroLevel) - LIBRARY->heroh->reqExp(hero->level);
 
 	if (info.reward.heroExperience > 0)
 		expToGive += hero->calculateXp(info.reward.heroExperience);
 
 	if(expToGive)
-		cb->giveExperience(hero, expToGive);
+		gameEvents.giveExperience(hero, expToGive);
 }
 
-void Rewardable::Interface::grantRewardAfterLevelup(const Rewardable::VisitInfo & info, const CArmedInstance * army, const CGHeroInstance * hero) const
+void Rewardable::Interface::grantRewardAfterLevelup(IGameEventCallback & gameEvents, const Rewardable::VisitInfo & info, const CArmedInstance * army, const CGHeroInstance * hero) const
 {
 	auto cb = getObject()->cb;
 
 	if(info.reward.manaDiff || info.reward.manaPercentage >= 0)
-		cb->setManaPoints(hero->id, info.reward.calculateManaPoints(hero));
+		gameEvents.setManaPoints(hero->id, info.reward.calculateManaPoints(hero));
 
-	if(info.reward.movePoints || info.reward.movePercentage >= 0)
+	if(info.reward.movePoints != 0 || info.reward.movePercentage >= 0)
+		gameEvents.setMovePoints(hero->id, info.reward.calculateMovePoints(hero));
+
+	for(const auto & bonus : info.reward.heroBonuses)
 	{
-		SetMovePoints smp;
-		smp.hid = hero->id;
-		smp.val = hero->movementPointsRemaining();
-
-		if (info.reward.movePercentage >= 0) // percent from max
-			smp.val = hero->movementPointsLimit(hero->boat && hero->boat->layer == EPathfindingLayer::SAIL) * info.reward.movePercentage / 100;
-		smp.val = std::max<si32>(0, smp.val + info.reward.movePoints);
-
-		cb->setMovePoints(&smp);
+		GiveBonus gb(GiveBonus::ETarget::OBJECT, hero->id, *bonus);
+		gameEvents.giveHeroBonus(&gb);
 	}
 
-	for(const Bonus & bonus : info.reward.bonuses)
+	if (hero->getCommander())
 	{
-		GiveBonus gb;
-		gb.who = GiveBonus::ETarget::OBJECT;
-		gb.bonus = bonus;
-		gb.id = hero->id;
-		cb->giveHeroBonus(&gb);
+		for(const auto & bonus : info.reward.commanderBonuses)
+		{
+			GiveBonus gb(GiveBonus::ETarget::HERO_COMMANDER, hero->id, *bonus);
+			gameEvents.giveHeroBonus(&gb);
+		}
 	}
 
-	for(const ArtifactID & art : info.reward.artifacts)
-		cb->giveHeroNewArtifact(hero, art, ArtifactPosition::FIRST_AVAILABLE);
+	for(const auto & bonus : info.reward.playerBonuses)
+	{
+		GiveBonus gb(GiveBonus::ETarget::PLAYER, hero->getOwner(), *bonus);
+		gameEvents.giveHeroBonus(&gb);
+	}
+
+	for(const ArtifactID & art : info.reward.takenArtifacts)
+	{
+		// hero does not have such artifact alone, but he might have it as part of assembled artifact
+		if(!hero->hasArt(art))
+		{
+			const auto * assembly = hero->getCombinedArtWithPart(art);
+			if (assembly)
+			{
+				DisassembledArtifact da;
+				da.al = ArtifactLocation(hero->id, hero->getArtPos(assembly));
+				gameEvents.sendAndApply(da);
+			}
+		}
+
+		if(hero->hasArt(art))
+			gameEvents.removeArtifact(ArtifactLocation(hero->id, hero->getArtPos(art, false)));
+	}
+
+	for(const ArtifactPosition & slot : info.reward.takenArtifactSlots)
+	{
+		const auto & slotContent = hero->getSlot(slot);
+
+		if (!slotContent->locked && slotContent->artifactID.hasValue())
+			gameEvents.removeArtifact(ArtifactLocation(hero->id, slot));
+
+		// TODO: handle locked slots?
+	}
+
+	for(const SpellID & spell : info.reward.takenScrolls)
+	{
+		if(hero->hasScroll(spell, false))
+			gameEvents.removeArtifact(ArtifactLocation(hero->id, hero->getScrollPos(spell, false)));
+	}
+
+	for(const ArtifactID & art : info.reward.grantedArtifacts)
+		gameEvents.giveHeroNewArtifact(hero, art, ArtifactPosition::FIRST_AVAILABLE);
+
+	for(const SpellID & spell : info.reward.grantedScrolls)
+		gameEvents.giveHeroNewScroll(hero, spell, ArtifactPosition::FIRST_AVAILABLE);
 
 	if(!info.reward.spells.empty())
 	{
 		std::set<SpellID> spellsToGive;
 
 		for (auto const & spell : info.reward.spells)
-			if (hero->canLearnSpell(spell.toEntity(VLC), true))
+			if (hero->canLearnSpell(spell.toEntity(LIBRARY), true))
 				spellsToGive.insert(spell);
 
 		if (!spellsToGive.empty())
-			cb->changeSpells(hero, true, spellsToGive);
+			gameEvents.changeSpells(hero, true, spellsToGive);
+	}
+
+	if (!info.reward.takenCreatures.empty())
+	{
+		gameEvents.takeCreatures(hero->id, info.reward.takenCreatures, !info.reward.creatures.empty());
 	}
 
 	if(!info.reward.creaturesChange.empty())
 	{
 		for(const auto & slot : hero->Slots())
 		{
-			const CStackInstance * heroStack = slot.second;
+			const auto & heroStack = slot.second;
 
 			for(const auto & change : info.reward.creaturesChange)
 			{
 				if (heroStack->getId() == change.first)
 				{
-					StackLocation location(hero, slot.first);
-					cb->changeStackType(location, change.second.toCreature());
+					StackLocation location(hero->id, slot.first);
+					gameEvents.changeStackType(location, change.second.toCreature());
 					break;
 				}
 			}
@@ -197,22 +243,25 @@ void Rewardable::Interface::grantRewardAfterLevelup(const Rewardable::VisitInfo
 	{
 		CCreatureSet creatures;
 		for(const auto & crea : info.reward.creatures)
-			creatures.addToSlot(creatures.getFreeSlot(), new CStackInstance(crea.getCreature(), crea.count));
+			creatures.addToSlot(creatures.getFreeSlot(), std::make_unique<CStackInstance>(cb, crea.getId(), crea.getCount()));
 
-		if(auto * army = dynamic_cast<const CArmedInstance*>(this)) //TODO: to fix that, CArmedInstance must be split on map instance part and interface part
-			cb->giveCreatures(army, hero, creatures, false);
+		auto * army = dynamic_cast<const CArmedInstance*>(this);
+		if (army)
+			gameEvents.giveCreatures(army, hero, creatures, false);
+		else
+			gameEvents.giveCreatures(hero, creatures);
 	}
 	
 	if(info.reward.spellCast.first != SpellID::NONE)
 	{
 		caster.setActualCaster(hero);
 		caster.setSpellSchoolLevel(info.reward.spellCast.second);
-		cb->castSpell(&caster, info.reward.spellCast.first, int3{-1, -1, -1});
+		gameEvents.castSpell(&caster, info.reward.spellCast.first, int3{-1, -1, -1});
 	}
 
 	if(info.reward.removeObject)
 		if(auto * instance = dynamic_cast<const CGObjectInstance*>(this))
-			cb->removeAfterVisit(instance);
+			gameEvents.removeAfterVisit(instance->id);
 }
 
 void Rewardable::Interface::serializeJson(JsonSerializeFormat & handler)
@@ -220,7 +269,7 @@ void Rewardable::Interface::serializeJson(JsonSerializeFormat & handler)
 	configuration.serializeJson(handler);
 }
 
-void Rewardable::Interface::grantRewardWithMessage(const CGHeroInstance * contextHero, int index, bool markAsVisit) const
+void Rewardable::Interface::grantRewardWithMessage(IGameEventCallback & gameEvents, const CGHeroInstance * contextHero, int index, bool markAsVisit) const
 {
 	auto vi = configuration.info.at(index);
 	logGlobal->debug("Granting reward %d. Message says: %s", index, vi.message.toString());
@@ -233,21 +282,21 @@ void Rewardable::Interface::grantRewardWithMessage(const CGHeroInstance * contex
 		vi.reward.loadComponents(iw.components, contextHero);
 		iw.type = configuration.infoWindowType;
 		if(!iw.components.empty() || !iw.text.toString().empty())
-			getObject()->cb->showInfoDialog(&iw);
+			gameEvents.showInfoDialog(&iw);
 	}
 	// grant reward afterwards. Note that it may remove object
 	if(markAsVisit)
-		markAsVisited(contextHero);
-	grantReward(index, contextHero);
+		markAsVisited(gameEvents, contextHero);
+	grantReward(gameEvents, index, contextHero);
 }
 
-void Rewardable::Interface::selectRewardWithMessage(const CGHeroInstance * contextHero, const std::vector<ui32> & rewardIndices, const MetaString & dialog) const
+void Rewardable::Interface::selectRewardWithMessage(IGameEventCallback & gameEvents, const CGHeroInstance * contextHero, const std::vector<ui32> & rewardIndices, const MetaString & dialog) const
 {
 	BlockingDialog sd(configuration.canRefuse, rewardIndices.size() > 1);
 	sd.player = contextHero->tempOwner;
 	sd.text = dialog;
 	sd.components = loadComponents(contextHero, rewardIndices);
-	getObject()->cb->showBlockingDialog(getObject(), &sd);
+	gameEvents.showBlockingDialog(getObject(), &sd);
 }
 
 std::vector<Component> Rewardable::Interface::loadComponents(const CGHeroInstance * contextHero, const std::vector<ui32> & rewardIndices) const
@@ -270,7 +319,7 @@ std::vector<Component> Rewardable::Interface::loadComponents(const CGHeroInstanc
 	return result;
 }
 
-void Rewardable::Interface::grantAllRewardsWithMessage(const CGHeroInstance * contextHero, const std::vector<ui32> & rewardIndices, bool markAsVisit) const
+void Rewardable::Interface::grantAllRewardsWithMessage(IGameEventCallback & gameEvents, const CGHeroInstance * contextHero, const std::vector<ui32> & rewardIndices, bool markAsVisit) const
 {
 	if (rewardIndices.empty())
 		return;
@@ -278,14 +327,14 @@ void Rewardable::Interface::grantAllRewardsWithMessage(const CGHeroInstance * co
 	for (auto index : rewardIndices)
 	{
 		// TODO: Merge all rewards of same type, with single message?
-		grantRewardWithMessage(contextHero, index, false);
+		grantRewardWithMessage(gameEvents, contextHero, index, false);
 	}
 	// Mark visited only after all rewards were processed
 	if(markAsVisit)
-		markAsVisited(contextHero);
+		markAsVisited(gameEvents, contextHero);
 }
 
-void Rewardable::Interface::doHeroVisit(const CGHeroInstance *h) const
+void Rewardable::Interface::doHeroVisit(IGameEventCallback & gameEvents, const CGHeroInstance *h) const
 {
 	if(!wasVisitedBefore(h))
 	{
@@ -304,7 +353,7 @@ void Rewardable::Interface::doHeroVisit(const CGHeroInstance *h) const
 			{
 				auto emptyRewards = getAvailableRewards(h, Rewardable::EEventType::EVENT_NOT_AVAILABLE);
 				if (!emptyRewards.empty())
-					grantRewardWithMessage(h, emptyRewards[0], false);
+					grantRewardWithMessage(gameEvents, h, emptyRewards[0], false);
 				else
 					logMod->warn("No applicable message for visiting empty object!");
 				break;
@@ -312,34 +361,34 @@ void Rewardable::Interface::doHeroVisit(const CGHeroInstance *h) const
 			case 1: // one reward. Just give it with message
 			{
 				if (configuration.canRefuse)
-					selectRewardWithMessage(h, rewards, configuration.info.at(rewards.front()).message);
+					selectRewardWithMessage(gameEvents, h, rewards, configuration.info.at(rewards.front()).message);
 				else
-					grantRewardWithMessage(h, rewards.front(), true);
+					grantRewardWithMessage(gameEvents, h, rewards.front(), true);
 				break;
 			}
 			default: // multiple rewards. Act according to select mode
 			{
 				switch (configuration.selectMode) {
 					case Rewardable::SELECT_PLAYER: // player must select
-						selectRewardWithMessage(h, rewards, configuration.onSelect);
+						selectRewardWithMessage(gameEvents, h, rewards, configuration.onSelect);
 						break;
 					case Rewardable::SELECT_FIRST: // give first available
 						if (configuration.canRefuse)
-							selectRewardWithMessage(h, { rewards.front() }, configuration.info.at(rewards.front()).message);
+							selectRewardWithMessage(gameEvents, h, { rewards.front() }, configuration.info.at(rewards.front()).message);
 						else
-							grantRewardWithMessage(h, rewards.front(), true);
+							grantRewardWithMessage(gameEvents, h, rewards.front(), true);
 						break;
 					case Rewardable::SELECT_RANDOM: // give random
 					{
-						ui32 rewardIndex = *RandomGeneratorUtil::nextItem(rewards, getObject()->cb->getRandomGenerator());
+						ui32 rewardIndex = *RandomGeneratorUtil::nextItem(rewards, gameEvents.getRandomGenerator());
 						if (configuration.canRefuse)
-							selectRewardWithMessage(h, { rewardIndex }, configuration.info.at(rewardIndex).message);
+							selectRewardWithMessage(gameEvents, h, { rewardIndex }, configuration.info.at(rewardIndex).message);
 						else
-							grantRewardWithMessage(h, rewardIndex, true);
+							grantRewardWithMessage(gameEvents, h, rewardIndex, true);
 						break;
 					}
 					case Rewardable::SELECT_ALL: // grant all possible
-						grantAllRewardsWithMessage(h, rewards, true);
+						grantAllRewardsWithMessage(gameEvents, h, rewards, true);
 						break;
 				}
 				break;
@@ -347,24 +396,24 @@ void Rewardable::Interface::doHeroVisit(const CGHeroInstance *h) const
 		}
 
 		if(!objectRemovalPossible && getAvailableRewards(h, Rewardable::EEventType::EVENT_FIRST_VISIT).empty())
-			markAsScouted(h);
+			markAsScouted(gameEvents, h);
 	}
 	else
 	{
 		logGlobal->debug("Revisiting already visited object");
 
 		if (!wasVisited(h->getOwner()))
-			markAsScouted(h);
+			markAsScouted(gameEvents, h);
 
 		auto visitedRewards = getAvailableRewards(h, Rewardable::EEventType::EVENT_ALREADY_VISITED);
 		if (!visitedRewards.empty())
-			grantRewardWithMessage(h, visitedRewards[0], false);
+			grantRewardWithMessage(gameEvents, h, visitedRewards[0], false);
 		else
 			logMod->warn("No applicable message for visiting already visited object!");
 	}
 }
 
-void Rewardable::Interface::onBlockingDialogAnswered(const CGHeroInstance * hero, int32_t answer) const
+void Rewardable::Interface::onBlockingDialogAnswered(IGameEventCallback & gameEvents, const CGHeroInstance * hero, int32_t answer) const
 {
 	if (answer == 0)
 		return; //Player refused
@@ -372,8 +421,8 @@ void Rewardable::Interface::onBlockingDialogAnswered(const CGHeroInstance * hero
 	if(answer > 0 && answer - 1 < configuration.info.size())
 	{
 		auto list = getAvailableRewards(hero, Rewardable::EEventType::EVENT_FIRST_VISIT);
-		markAsVisited(hero);
-		grantReward(list[answer - 1], hero);
+		markAsVisited(gameEvents, hero);
+		grantReward(gameEvents, list[answer - 1], hero);
 	}
 	else
 	{
