@@ -36,6 +36,7 @@ Rewardable::Reward::Reward()
 	, manaOverflowFactor(0)
 	, movePoints(0)
 	, movePercentage(-1)
+	, moveOverflowFactor(100)
 	, primary(4, 0)
 	, removeObject(false)
 	, spellCast(SpellID::NONE, MasteryLevel::NONE)
@@ -59,6 +60,23 @@ si32 Rewardable::Reward::calculateManaPoints(const CGHeroInstance * hero) const
 	return manaOutput;
 }
 
+si32 Rewardable::Reward::calculateMovePoints(const CGHeroInstance * hero) const
+{
+	si32 moveScaled = hero->movementPointsRemaining();
+	si32 moveLimit = hero->movementPointsLimit();
+
+	if (movePercentage >= 0)
+		moveScaled = moveLimit * movePercentage / 100;
+
+	si32 moveMissing   = std::max(0, moveLimit - moveScaled);
+	si32 moveGranted   = std::min(moveMissing, movePoints);
+	si32 moveOverflow  = movePoints - moveGranted;
+	si32 moveOverLimit = moveOverflow * moveOverflowFactor / 100;
+	si32 moveOutput    = moveScaled + moveGranted + moveOverLimit;
+
+	return std::max(0, moveOutput);
+}
+
 Component Rewardable::Reward::getDisplayedComponent(const CGHeroInstance * h) const
 {
 	std::vector<Component> comps;
@@ -79,12 +97,12 @@ void Rewardable::Reward::loadComponents(std::vector<Component> & comps, const CG
 	for (auto comp : extraComponents)
 		comps.push_back(comp);
 	
-	for (auto & bonus : bonuses)
+	for (auto & bonus : heroBonuses)
 	{
-		if (bonus.type == BonusType::MORALE)
-			comps.emplace_back(ComponentType::MORALE, bonus.val);
-		if (bonus.type == BonusType::LUCK)
-			comps.emplace_back(ComponentType::LUCK, bonus.val);
+		if (bonus->type == BonusType::MORALE)
+			comps.emplace_back(ComponentType::MORALE, bonus->val);
+		if (bonus->type == BonusType::LUCK)
+			comps.emplace_back(ComponentType::LUCK, bonus->val);
 	}
 	
 	if (heroExperience)
@@ -107,21 +125,43 @@ void Rewardable::Reward::loadComponents(std::vector<Component> & comps, const CG
 		auto skillID = entry.first;
 		int levelsGained = entry.second;
 		int currentLevel = h ? h->getSecSkillLevel(skillID) : 0;
-		int finalLevel = std::min(static_cast<int>(MasteryLevel::EXPERT), currentLevel + levelsGained);
-		comps.emplace_back(ComponentType::SEC_SKILL, entry.first, finalLevel);
+		int finalLevel = std::clamp<int>(currentLevel + levelsGained, MasteryLevel::NONE, MasteryLevel::EXPERT);
+		if (finalLevel == MasteryLevel::NONE)
+			comps.emplace_back(ComponentType::SEC_SKILL, entry.first);
+		else
+			comps.emplace_back(ComponentType::SEC_SKILL, entry.first, finalLevel);
 	}
 
-	for(const auto & entry : artifacts)
+	for(const auto & entry : grantedArtifacts)
 		comps.emplace_back(ComponentType::ARTIFACT, entry);
 
+	for(const auto & entry : takenArtifacts)
+		comps.emplace_back(ComponentType::ARTIFACT, entry);
+
+	for(const auto & entry : takenArtifactSlots)
+	{
+		if (h)
+		{
+			const auto & slotContent = h->getSlot(entry);
+			if (slotContent->artifactID.hasValue())
+				comps.emplace_back(ComponentType::ARTIFACT, slotContent->getArt()->getTypeId());
+		}
+	}
+
+	for(const SpellID & spell : grantedScrolls)
+		comps.emplace_back(ComponentType::SPELL, spell);
+
+	for(const SpellID & spell : takenScrolls)
+		comps.emplace_back(ComponentType::SPELL, spell);
+
 	for(const auto & entry : spells)
 	{
-		bool learnable = !h || h->canLearnSpell(entry.toEntity(VLC), true);
+		bool learnable = !h || h->canLearnSpell(entry.toEntity(LIBRARY), true);
 		comps.emplace_back(ComponentType::SPELL, entry, learnable ?	0 : -1);
 	}
 
 	for(const auto & entry : creatures)
-		comps.emplace_back(ComponentType::CREATURE, entry.getId(), entry.count);
+		comps.emplace_back(ComponentType::CREATURE, entry.getId(), entry.getCount());
 
 	for (size_t i=0; i<resources.size(); i++)
 	{
@@ -141,7 +181,12 @@ void Rewardable::Reward::serializeJson(JsonSerializeFormat & handler)
 	handler.serializeInt("manaDiff", manaDiff);
 	handler.serializeInt("manaOverflowFactor", manaOverflowFactor);
 	handler.serializeInt("movePoints", movePoints);
-	handler.serializeIdArray("artifacts", artifacts);
+	handler.serializeInt("moveOverflowFactor", manaOverflowFactor);
+	handler.serializeIdArray("artifacts", grantedArtifacts);
+	handler.serializeIdArray("takenArtifacts", takenArtifacts);
+	handler.serializeIdArray("takenArtifactSlots", takenArtifactSlots);
+	handler.serializeIdArray("scrolls", grantedScrolls);
+	handler.serializeIdArray("takenScrolls", takenScrolls);
 	handler.serializeIdArray("spells", spells);
 	handler.enterArray("creatures").serializeStruct(creatures);
 	handler.enterArray("primary").serializeArray(primary);
