@@ -16,7 +16,7 @@
 #include "../mapping/CMapHeader.h"
 #include "CRmgTemplateStorage.h"
 #include "CRmgTemplate.h"
-#include "../VCMI_Lib.h"
+#include "../GameLibrary.h"
 #include "serializer/JsonSerializeFormat.h"
 
 #include <vstd/RNG.h>
@@ -24,7 +24,7 @@
 VCMI_LIB_NAMESPACE_BEGIN
 
 CMapGenOptions::CMapGenOptions()
-	: width(CMapHeader::MAP_SIZE_MIDDLE), height(CMapHeader::MAP_SIZE_MIDDLE), hasTwoLevels(true),
+	: width(CMapHeader::MAP_SIZE_MIDDLE), height(CMapHeader::MAP_SIZE_MIDDLE), levels(2),
 	humanOrCpuPlayerCount(RANDOM_SIZE), teamCount(RANDOM_SIZE), compOnlyPlayerCount(RANDOM_SIZE), compOnlyTeamCount(RANDOM_SIZE),
 	waterContent(EWaterContent::RANDOM), monsterStrength(EMonsterStrength::RANDOM), mapTemplate(nullptr),
 	customizedPlayers(false)
@@ -54,14 +54,14 @@ void CMapGenOptions::setHeight(si32 value)
 	height = value;
 }
 
-bool CMapGenOptions::getHasTwoLevels() const
+int CMapGenOptions::getLevels() const
 {
-	return hasTwoLevels;
+	return levels;
 }
 
-void CMapGenOptions::setHasTwoLevels(bool value)
+void CMapGenOptions::setLevels(int value)
 {
-	hasTwoLevels = value;
+	levels = value;
 }
 
 si8 CMapGenOptions::getHumanOrCpuPlayerCount() const
@@ -133,7 +133,7 @@ si8 CMapGenOptions::getMaxPlayersCount(bool withTemplateLimit) const
 	}
 	else
 	{
-		totalPlayers = humans + cpus;
+		totalPlayers = std::max(humans + cpus, 2);
 	}
 
 	if (withTemplateLimit && mapTemplate)
@@ -425,12 +425,12 @@ void CMapGenOptions::setMapTemplate(const CRmgTemplate * value)
 	//validate & adapt options according to template
 	if(mapTemplate)
 	{
-		if(!mapTemplate->matchesSize(int3(getWidth(), getHeight(), 1 + getHasTwoLevels())))
+		if(!mapTemplate->matchesSize(int3(getWidth(), getHeight(), getLevels())))
 		{
 			auto sizes = mapTemplate->getMapSizes();
 			setWidth(sizes.first.x);
 			setHeight(sizes.first.y);
-			setHasTwoLevels(sizes.first.z - 1);
+			setLevels(sizes.first.z);
 		}
 
 		si8 maxPlayerCount = getMaxPlayersCount(false);
@@ -453,7 +453,7 @@ void CMapGenOptions::setMapTemplate(const CRmgTemplate * value)
 void CMapGenOptions::setMapTemplate(const std::string & name)
 {
 	if(!name.empty())
-		setMapTemplate(VLC->tplh->getTemplate(name));
+		setMapTemplate(LIBRARY->tplh->getTemplate(name));
 }
 
 void CMapGenOptions::setRoadEnabled(const RoadId & roadType, bool enable)
@@ -488,7 +488,7 @@ void CMapGenOptions::setPlayerTeam(const PlayerColor & color, const TeamID & tea
 
 void CMapGenOptions::finalize(vstd::RNG & rand)
 {
-	logGlobal->info("RMG map: %dx%d, %s underground", getWidth(), getHeight(), getHasTwoLevels() ? "WITH" : "NO");
+	logGlobal->info("RMG map: %dx%d, %s underground", getWidth(), getHeight(), getLevels() >= 2 ? "WITH" : "NO");
 	logGlobal->info("RMG settings: players %d, teams %d, computer players %d, computer teams %d, water %d, monsters %d",
 		static_cast<int>(getHumanOrCpuPlayerCount()), static_cast<int>(getTeamCount()), static_cast<int>(getCompOnlyPlayerCount()),
 		static_cast<int>(getCompOnlyTeamCount()), static_cast<int>(getWaterContent()), static_cast<int>(getMonsterStrength()));
@@ -700,10 +700,10 @@ bool CMapGenOptions::arePlayersCustomized() const
 
 std::vector<const CRmgTemplate *> CMapGenOptions::getPossibleTemplates() const
 {
-	int3 tplSize(width, height, (hasTwoLevels ? 2 : 1));
+	int3 tplSize(width, height, levels);
 	auto humanPlayers = countHumanPlayers();
 
-	auto templates = VLC->tplh->getTemplates();
+	auto templates = LIBRARY->tplh->getTemplates();
 
 	vstd::erase_if(templates, [this, &tplSize, humanPlayers](const CRmgTemplate * tmpl)
 	{
@@ -719,7 +719,7 @@ std::vector<const CRmgTemplate *> CMapGenOptions::getPossibleTemplates() const
 
 		if(humanOrCpuPlayerCount != CMapGenOptions::RANDOM_SIZE && compOnlyPlayerCount != CMapGenOptions::RANDOM_SIZE)
 		{
-			if (!tmpl->getPlayers().isInRange(humanOrCpuPlayerCount + compOnlyPlayerCount))
+			if (!tmpl->getPlayers().isInRange(std::max(humanOrCpuPlayerCount + compOnlyPlayerCount, 2)))
 				return true;
 
 		}
@@ -784,8 +784,8 @@ void CMapGenOptions::CPlayerSettings::setStartingTown(FactionID value)
 	assert(value >= FactionID::RANDOM);
 	if(value != FactionID::RANDOM)
 	{
-		assert(value < FactionID(VLC->townh->size()));
-		assert((*VLC->townh)[value]->town != nullptr);
+		assert(value < FactionID(LIBRARY->townh->size()));
+		assert((*LIBRARY->townh)[value]->town != nullptr);
 	}
 	startingTown = value;
 }
@@ -797,7 +797,7 @@ HeroTypeID CMapGenOptions::CPlayerSettings::getStartingHero() const
 
 void CMapGenOptions::CPlayerSettings::setStartingHero(HeroTypeID value)
 {
-	assert(value == HeroTypeID::RANDOM || value.toEntity(VLC) != nullptr);
+	assert(value == HeroTypeID::RANDOM || value.toEntity(LIBRARY) != nullptr);
 	startingHero = value;
 }
 
@@ -825,7 +825,15 @@ void CMapGenOptions::serializeJson(JsonSerializeFormat & handler)
 {
 	handler.serializeInt("width", width);
 	handler.serializeInt("height", height);
-	handler.serializeBool("haswoLevels", hasTwoLevels);
+	bool hasTwoLevelsKey = !handler.getCurrent()["haswoLevels"].isNull();
+	bool hasTwoLevels = levels == 2;
+	if(handler.saving || !hasTwoLevelsKey)
+		handler.serializeInt("levels", levels, 1);
+	else
+	{
+		handler.serializeBool("haswoLevels", hasTwoLevels);
+		levels = hasTwoLevels ? 2 : 1;
+	}
 	handler.serializeInt("humanOrCpuPlayerCount", humanOrCpuPlayerCount);
 	handler.serializeInt("teamCount", teamCount);
 	handler.serializeInt("compOnlyPlayerCount", compOnlyPlayerCount);
