@@ -15,12 +15,16 @@
 #include "Functions.h"
 
 #include "../TerrainHandler.h"
-#include "../VCMI_Lib.h"
+#include "../GameLibrary.h"
 #include "../constants/StringConstants.h"
 #include "../entities/faction/CTownHandler.h"
+#include "../entities/ResourceTypeHandler.h"
 #include "../modding/ModScope.h"
 #include "../serializer/JsonSerializeFormat.h"
 
+#include <boost/lexical_cast.hpp>
+
+
 VCMI_LIB_NAMESPACE_BEGIN
 
 namespace
@@ -102,7 +106,38 @@ void ZoneOptions::CTownInfo::serializeJson(JsonSerializeFormat & handler)
 	handler.serializeInt("castles", castleCount, 0);
 	handler.serializeInt("townDensity", townDensity, 0);
 	handler.serializeInt("castleDensity", castleDensity, 0);
-	handler.serializeInt("sourceZone", sourceZone, NO_ZONE);
+	handler.serializeInt("townTypesLikeZone", townTypesLikeZone, NO_ZONE);
+	handler.serializeInt("townTypesNotLikeZone", townTypesNotLikeZone, NO_ZONE);
+	handler.serializeInt("townTypesRelatedToZoneTerrain", townTypesRelatedToZoneTerrain, NO_ZONE);
+}
+
+ZoneOptions::CTownHints::CTownHints()
+	: likeZone(NO_ZONE),
+	relatedToZoneTerrain(NO_ZONE)
+{
+
+}
+
+void ZoneOptions::CTownHints::serializeJson(JsonSerializeFormat & handler)
+{
+	handler.serializeInt("likeZone", likeZone, NO_ZONE);
+	auto node = handler.getCurrent();
+	if (node["notLikeZone"].isVector())
+	{
+		// TODO: Utility to serialize vector of ints?
+		auto notLikeZoneData = handler.enterArray("notLikeZone");
+		notLikeZone.resize(notLikeZoneData.size());
+		for (size_t i = 0; i < notLikeZoneData.size(); ++i)
+		{
+			notLikeZoneData.serializeInt(i, notLikeZone[i]);
+		}
+	}
+	else
+	{
+		notLikeZone.resize(1);
+		handler.serializeInt("notLikeZone", notLikeZone[0], NO_ZONE);
+	}
+	handler.serializeInt("relatedToZoneTerrain", relatedToZoneTerrain, NO_ZONE);
 }
 
 ZoneOptions::ZoneOptions():
@@ -110,13 +145,18 @@ ZoneOptions::ZoneOptions():
 	type(ETemplateZoneType::PLAYER_START),
 	size(1),
 	maxTreasureValue(0),
-	owner(std::nullopt),
+	owner(PlayerColor(0)),
 	matchTerrainToTown(true),
 	townsAreSameType(false),
 	monsterStrength(EMonsterStrength::ZONE_NORMAL),
+	townsLikeZone(NO_ZONE),
 	minesLikeZone(NO_ZONE),
 	terrainTypeLikeZone(NO_ZONE),
-	treasureLikeZone(NO_ZONE)
+	treasureLikeZone(NO_ZONE),
+	customObjectsLikeZone(NO_ZONE),
+	visiblePosition(Point(0, 0)),
+	visibleSize(1.0),
+	forcedLevel(EZoneLevel::AUTOMATIC)
 {
 }
 
@@ -177,7 +217,7 @@ void ZoneOptions::setTerrainTypes(const std::set<TerrainId> & value)
 std::set<TerrainId> ZoneOptions::getDefaultTerrainTypes() const
 {
 	std::set<TerrainId> terrains;
-	for(const auto & terrain : VLC->terrainTypeHandler->objects)
+	for(const auto & terrain : LIBRARY->terrainTypeHandler->objects)
 	{
 		if (terrain->isLand() && terrain->isPassable())
 		{
@@ -189,7 +229,7 @@ std::set<TerrainId> ZoneOptions::getDefaultTerrainTypes() const
 
 std::set<FactionID> ZoneOptions::getDefaultTownTypes() const
 {
-	return VLC->townh->getDefaultAllowed();
+	return LIBRARY->townh->getDefaultAllowed();
 }
 
 std::set<FactionID> ZoneOptions::getTownTypes() const
@@ -220,12 +260,12 @@ std::set<FactionID> ZoneOptions::getMonsterTypes() const
 	return vstd::difference(monsterTypes, bannedMonsters);
 }
 
-void ZoneOptions::setMinesInfo(const std::map<TResource, ui16> & value)
+void ZoneOptions::setMinesInfo(const std::map<GameResID, ui16> & value)
 {
 	mines = value;
 }
 
-std::map<TResource, ui16> ZoneOptions::getMinesInfo() const
+std::map<GameResID, ui16> ZoneOptions::getMinesInfo() const
 {
 	return mines;
 }
@@ -291,6 +331,41 @@ TRmgTemplateZoneId ZoneOptions::getCustomObjectsLikeZone() const
 	return customObjectsLikeZone;
 }
 
+TRmgTemplateZoneId ZoneOptions::getTownsLikeZone() const
+{
+	return townsLikeZone;
+}
+
+Point ZoneOptions::getVisiblePosition() const
+{
+	return visiblePosition;
+}
+
+void ZoneOptions::setVisiblePosition(Point value)
+{
+	visiblePosition = value;
+}
+
+float ZoneOptions::getVisibleSize() const
+{
+	return visibleSize;
+}
+
+void ZoneOptions::setVisibleSize(float value)
+{
+	visibleSize = value;
+}
+
+EZoneLevel ZoneOptions::getForcedLevel() const
+{
+	return forcedLevel;
+}
+
+void ZoneOptions::setForcedLevel(EZoneLevel value)
+{
+	forcedLevel = value;
+}
+
 void ZoneOptions::addConnection(const ZoneConnection & connection)
 {
 	connectedZoneIds.push_back(connection.getOtherZoneId(getId()));
@@ -302,6 +377,28 @@ std::vector<ZoneConnection> ZoneOptions::getConnections() const
 	return connectionDetails;
 }
 
+std::vector<ZoneConnection>& ZoneOptions::getConnectionsRef()
+{
+	return connectionDetails;
+}
+
+void ZoneOptions::setRoadOption(int connectionId, rmg::ERoadOption roadOption)
+{
+	for(auto & connection : connectionDetails)
+	{
+		if(connection.getId() == connectionId)
+		{
+			connection.setRoadOption(roadOption);
+			logGlobal->info("Set road option for connection %d between zones %d and %d to %s", 
+				connectionId, connection.getZoneA(), connection.getZoneB(), 
+				roadOption == rmg::ERoadOption::ROAD_TRUE ? "true" : 
+				roadOption == rmg::ERoadOption::ROAD_FALSE ? "false" : "random");
+			return;
+		}
+	}
+	logGlobal->warn("Failed to find connection with ID %d in zone %d", connectionId, id);
+}
+
 std::vector<TRmgTemplateZoneId> ZoneOptions::getConnectedZoneIds() const
 {
 	return connectedZoneIds;
@@ -317,16 +414,51 @@ bool ZoneOptions::isMatchTerrainToTown() const
 	return matchTerrainToTown;
 }
 
+void ZoneOptions::setMatchTerrainToTown(bool value)
+{
+	matchTerrainToTown = value;
+}
+
 const ZoneOptions::CTownInfo & ZoneOptions::getPlayerTowns() const
 {
 	return playerTowns;
 }
 
+void ZoneOptions::setPlayerTowns(const CTownInfo & value)
+{
+	playerTowns = value;
+}
+
 const ZoneOptions::CTownInfo & ZoneOptions::getNeutralTowns() const
 {
 	return neutralTowns;
 }
 
+void ZoneOptions::setNeutralTowns(const CTownInfo & value)
+{
+	neutralTowns = value;
+}
+
+const std::vector<ZoneOptions::CTownHints> & ZoneOptions::getTownHints() const
+{
+	return townHints;
+}
+
+void ZoneOptions::setTownHints(const std::vector<CTownHints> & value)
+{
+	townHints = value;
+}
+
+std::set<FactionID> ZoneOptions::getBannedTownTypes() const
+{
+	return bannedTownTypes;
+}
+
+void ZoneOptions::setBannedTownTypes(const std::set<FactionID> & value)
+{
+	bannedTownTypes = value;
+}
+
 void ZoneOptions::serializeJson(JsonSerializeFormat & handler)
 {
 	static const std::vector<std::string> zoneTypes =
@@ -348,6 +480,7 @@ void ZoneOptions::serializeJson(JsonSerializeFormat & handler)
 
 	#define SERIALIZE_ZONE_LINK(fieldName) handler.serializeInt(#fieldName, fieldName, NO_ZONE);
 
+	SERIALIZE_ZONE_LINK(townsLikeZone);
 	SERIALIZE_ZONE_LINK(minesLikeZone);
 	SERIALIZE_ZONE_LINK(terrainTypeLikeZone);
 	SERIALIZE_ZONE_LINK(treasureLikeZone);
@@ -367,6 +500,8 @@ void ZoneOptions::serializeJson(JsonSerializeFormat & handler)
 	handler.serializeIdArray("allowedTowns", townTypes);
 	handler.serializeIdArray("bannedTowns", bannedTownTypes);
 
+	handler.enterArray("townHints").serializeStruct(townHints);
+
 	{
 		//TODO: add support for std::map to serializeEnum
 		static const std::vector<std::string> zoneMonsterStrengths =
@@ -409,15 +544,29 @@ void ZoneOptions::serializeJson(JsonSerializeFormat & handler)
 
 	if((minesLikeZone == NO_ZONE) && (!handler.saving || !mines.empty()))
 	{
-		auto minesData = handler.enterStruct("mines");
-
-		for(TResource idx = 0; idx < (GameConstants::RESOURCE_QUANTITY - 1); idx++)
-		{
-			handler.serializeInt(GameConstants::RESOURCE_NAMES[idx], mines[idx], 0);
-		}
+		handler.serializeIdMap<GameResID, ui16>("mines", mines);
 	}
 
 	handler.serializeStruct("customObjects", objectConfig);
+	handler.serializeInt("visiblePositionX", visiblePosition.x);
+	handler.serializeInt("visiblePositionY", visiblePosition.y);
+	handler.serializeFloat("visibleSize", visibleSize);
+
+	if(!handler.saving && visibleSize < 0.01)
+		visibleSize = 1.0;
+
+	{
+		static const std::vector<std::string> zoneLevels =
+		{
+			"automatic",
+			"surface",
+			"underground"
+		};
+
+		auto levelIndex = static_cast<int>(forcedLevel);
+		handler.serializeEnum("forcedLevel", levelIndex, static_cast<int>(EZoneLevel::AUTOMATIC), zoneLevels);
+		forcedLevel = static_cast<EZoneLevel>(levelIndex);
+	}
 }
 
 ZoneConnection::ZoneConnection():
@@ -426,7 +575,7 @@ ZoneConnection::ZoneConnection():
 	zoneB(-1),
 	guardStrength(0),
 	connectionType(rmg::EConnectionType::GUARDED),
-	hasRoad(rmg::ERoadOption::ROAD_TRUE)
+	hasRoad(rmg::ERoadOption::ROAD_RANDOM)
 {
 
 }
@@ -481,12 +630,22 @@ rmg::ERoadOption ZoneConnection::getRoadOption() const
 {
 	return hasRoad;
 }
+
+void ZoneConnection::setRoadOption(rmg::ERoadOption roadOption)
+{
+	hasRoad = roadOption;
+}
 	
 bool operator==(const ZoneConnection & l, const ZoneConnection & r)
 {
 	return l.id == r.id;
 }
 
+bool operator<(const ZoneConnection & l, const ZoneConnection & r)
+{
+	return l.id < r.id;
+}
+
 void ZoneConnection::serializeJson(JsonSerializeFormat & handler)
 {
 	static const std::vector<std::string> connectionTypes =
@@ -500,9 +659,9 @@ void ZoneConnection::serializeJson(JsonSerializeFormat & handler)
 
 	static const std::vector<std::string> roadOptions =
 	{
+		"random",
 		"true",
-		"false",
-		"random"
+		"false"
 	};
 
 	if (handler.saving)
@@ -728,6 +887,16 @@ void CRmgTemplate::serializeJson(JsonSerializeFormat & handler)
 	serializePlayers(handler, players, "players");
 	serializePlayers(handler, humanPlayers, "humans"); // TODO: Rename this parameter
 
+	handler.serializeIdArray("bannedSpells", bannedSpells);
+	handler.serializeIdArray("bannedArtifacts", bannedArtifacts);
+	handler.serializeIdArray("bannedSkills", bannedSkills);
+	handler.serializeIdArray("bannedHeroes", bannedHeroes);
+
+	handler.serializeIdArray("enabledSpells", enabledSpells);
+	handler.serializeIdArray("enabledArtifacts", enabledArtifacts);
+	handler.serializeIdArray("enabledSkills", enabledSkills);
+	handler.serializeIdArray("enabledHeroes", enabledHeroes);
+
 	*mapSettings = handler.getCurrent()["settings"];
 
 	{
@@ -829,18 +998,20 @@ void CRmgTemplate::afterLoad()
 		auto zone = idAndZone.second;
 
 		// Inherit properties recursively
+		inheritTownProperties(zone);
+
 		inheritZoneProperty(zone, 
 							&rmg::ZoneOptions::getTerrainTypes, 
 							&rmg::ZoneOptions::setTerrainTypes, 
 							&rmg::ZoneOptions::getTerrainTypeLikeZone,
 							"terrain types");
-		
+
 		inheritZoneProperty(zone, 
 							&rmg::ZoneOptions::getMinesInfo, 
 							&rmg::ZoneOptions::setMinesInfo, 
 							&rmg::ZoneOptions::getMinesLikeZone,
 							"mine types");
-		
+
 		inheritZoneProperty(zone, 
 							&rmg::ZoneOptions::getTreasureInfo, 
 							&rmg::ZoneOptions::setTreasureInfo, 
@@ -857,7 +1028,7 @@ void CRmgTemplate::afterLoad()
 		auto monsterTypes = zone->getMonsterTypes();
 		if (monsterTypes.empty())
 		{
-			zone->setMonsterTypes(VLC->townh->getAllowedFactions(false));
+			zone->setMonsterTypes(LIBRARY->townh->getAllowedFactions(false));
 		}
 	}
 
@@ -872,7 +1043,7 @@ void CRmgTemplate::afterLoad()
 		zone1->addConnection(connection);
 		zone2->addConnection(connection);
 	}
-	
+
 	if(allowedWaterContent.empty() || allowedWaterContent.count(EWaterContent::RANDOM))
 	{
 		allowedWaterContent.insert(EWaterContent::NONE);
@@ -882,6 +1053,31 @@ void CRmgTemplate::afterLoad()
 	allowedWaterContent.erase(EWaterContent::RANDOM);
 }
 
+void CRmgTemplate::inheritTownProperties(std::shared_ptr<rmg::ZoneOptions> zone, uint32_t iteration)
+{
+	if (iteration >= 50)
+	{
+		logGlobal->error("Infinite recursion for town properties detected in template %s", name);
+		return;
+	}
+
+	if (zone->getTownsLikeZone() != rmg::ZoneOptions::NO_ZONE)
+	{
+		const auto otherZone = zones.at(zone->getTownsLikeZone());
+		
+		// Recursively inherit from the source zone first
+		inheritTownProperties(otherZone, iteration + 1);
+
+		// Now copy all town-related properties from the source zone
+		zone->setPlayerTowns(otherZone->getPlayerTowns());
+		zone->setNeutralTowns(otherZone->getNeutralTowns());
+		zone->setMatchTerrainToTown(otherZone->isMatchTerrainToTown());
+		zone->setTownHints(otherZone->getTownHints());
+		zone->setTownTypes(otherZone->getTownTypes());
+		zone->setBannedTownTypes(otherZone->getBannedTownTypes());
+	}
+}
+
 // TODO: Allow any integer size which does not match enum, as well
 void CRmgTemplate::serializeSize(JsonSerializeFormat & handler, int3 & value, const std::string & fieldName)
 {
