@@ -12,6 +12,7 @@
 
 #include "../int3.h"
 #include "../GameConstants.h"
+#include "../Point.h"
 #include "../ResourceSet.h"
 #include "ObjectInfo.h"
 #include "ObjectConfig.h"
@@ -21,6 +22,7 @@ VCMI_LIB_NAMESPACE_BEGIN
 
 class JsonSerializeFormat;
 struct CompoundMapObjectID;
+class TemplateEditor;
 
 enum class ETemplateZoneType
 {
@@ -58,6 +60,13 @@ namespace EMonsterStrength // used as int in monster generation procedure and in
 	};
 }
 
+enum class EZoneLevel // used to force zone placement on surface or underground
+{
+	AUTOMATIC = 0,
+	SURFACE = 1,
+	UNDERGROUND = 2
+};
+
 class DLL_LINKAGE CTreasureInfo
 {
 public:
@@ -86,13 +95,17 @@ enum class EConnectionType
 
 enum class ERoadOption
 {
+	ROAD_RANDOM = 0,
 	ROAD_TRUE,
-	ROAD_FALSE,
-	ROAD_RANDOM
+	ROAD_FALSE
 };
 
 class DLL_LINKAGE ZoneConnection
 {
+#ifdef ENABLE_TEMPLATE_EDITOR
+	friend class ::TemplateEditor;
+#endif
+
 public:
 
 	ZoneConnection();
@@ -105,10 +118,12 @@ public:
 	int getGuardStrength() const;
 	rmg::EConnectionType getConnectionType() const;
 	rmg::ERoadOption getRoadOption() const;
+	void setRoadOption(rmg::ERoadOption roadOption);
 
 	void serializeJson(JsonSerializeFormat & handler);
 	
 	friend bool operator==(const ZoneConnection &, const ZoneConnection &);
+	friend bool operator<(const ZoneConnection &, const ZoneConnection &);
 private:
 	int id;
 	TRmgTemplateZoneId zoneA;
@@ -120,11 +135,18 @@ private:
 
 class DLL_LINKAGE ZoneOptions
 {
+#ifdef ENABLE_TEMPLATE_EDITOR
+	friend class ::TemplateEditor;
+#endif
+
 public:
 	static const TRmgTemplateZoneId NO_ZONE;
 
 	class DLL_LINKAGE CTownInfo
 	{
+#ifdef ENABLE_TEMPLATE_EDITOR
+		friend class ::TemplateEditor;
+#endif
 	public:
 		CTownInfo();
 
@@ -142,7 +164,22 @@ public:
 		int castleDensity;
 
 		// TODO: Copy from another zone once its randomized
-		TRmgTemplateZoneId sourceZone = NO_ZONE;
+
+		TRmgTemplateZoneId townTypesLikeZone = NO_ZONE;
+		TRmgTemplateZoneId townTypesNotLikeZone = NO_ZONE;
+		TRmgTemplateZoneId townTypesRelatedToZoneTerrain = NO_ZONE;
+	};
+
+	class DLL_LINKAGE CTownHints
+	{
+	public:
+		CTownHints();
+		// TODO: Make private
+		TRmgTemplateZoneId likeZone = NO_ZONE;
+		std::vector<TRmgTemplateZoneId> notLikeZone;
+		TRmgTemplateZoneId relatedToZoneTerrain = NO_ZONE;
+
+		void serializeJson(JsonSerializeFormat & handler);
 	};
 
 	ZoneOptions();
@@ -162,16 +199,25 @@ public:
 	std::set<TerrainId> getDefaultTerrainTypes() const;
 
 	const CTownInfo & getPlayerTowns() const;
+	void setPlayerTowns(const CTownInfo & value);
 	const CTownInfo & getNeutralTowns() const;
-	std::set<FactionID> getDefaultTownTypes() const;
+	void setNeutralTowns(const CTownInfo & value);
+	bool isMatchTerrainToTown() const;
+	void setMatchTerrainToTown(bool value);
+	const std::vector<CTownHints> & getTownHints() const;
+	void setTownHints(const std::vector<CTownHints> & value);
 	std::set<FactionID> getTownTypes() const;
+	void setTownTypes(const std::set<FactionID> & value);
+	std::set<FactionID> getBannedTownTypes() const;
+	void setBannedTownTypes(const std::set<FactionID> & value);
+
+	std::set<FactionID> getDefaultTownTypes() const;
 	std::set<FactionID> getMonsterTypes() const;
 
-	void setTownTypes(const std::set<FactionID> & value);
 	void setMonsterTypes(const std::set<FactionID> & value);
 
-	void setMinesInfo(const std::map<TResource, ui16> & value);
-	std::map<TResource, ui16> getMinesInfo() const;
+	void setMinesInfo(const std::map<GameResID, ui16> & value);
+	std::map<GameResID, ui16> getMinesInfo() const;
 
 	void setTreasureInfo(const std::vector<CTreasureInfo> & value);
 	void addTreasureInfo(const CTreasureInfo & value);
@@ -185,14 +231,17 @@ public:
 
 	void addConnection(const ZoneConnection & connection);
 	std::vector<ZoneConnection> getConnections() const;
+	std::vector<ZoneConnection>& getConnectionsRef();
 	std::vector<TRmgTemplateZoneId> getConnectedZoneIds() const;
 
+	// Set road option for a specific connection by ID
+	void setRoadOption(int connectionId, rmg::ERoadOption roadOption);
+
 	void serializeJson(JsonSerializeFormat & handler);
 	
 	EMonsterStrength::EMonsterStrength monsterStrength;
 	
 	bool areTownsSameType() const;
-	bool isMatchTerrainToTown() const;
 
 	// Get a group of configured objects
 	const std::vector<CompoundMapObjectID> & getBannedObjects() const;
@@ -202,7 +251,17 @@ public:
 	// Copy whole custom object config from another zone
 	ObjectConfig getCustomObjects() const;
 	void setCustomObjects(const ObjectConfig & value);
-	TRmgTemplateZoneId	getCustomObjectsLikeZone() const;
+	TRmgTemplateZoneId getCustomObjectsLikeZone() const;
+	TRmgTemplateZoneId getTownsLikeZone() const;
+
+	Point getVisiblePosition() const;
+	void setVisiblePosition(Point value);
+
+	float getVisibleSize() const;
+	void setVisibleSize(float value);
+
+	EZoneLevel getForcedLevel() const;
+	void setForcedLevel(EZoneLevel value);
 
 protected:
 	TRmgTemplateZoneId id;
@@ -211,6 +270,9 @@ protected:
 	ui32 maxTreasureValue;
 	std::optional<int> owner;
 
+	Point visiblePosition;
+	float visibleSize;
+
 	ObjectConfig objectConfig;
 	CTownInfo playerTowns;
 	CTownInfo neutralTowns;
@@ -218,23 +280,26 @@ protected:
 	std::set<TerrainId> terrainTypes;
 	std::set<TerrainId> bannedTerrains;
 	bool townsAreSameType;
+	std::vector<CTownHints> townHints; // For every town present on map
 
 	std::set<FactionID> townTypes;
 	std::set<FactionID> bannedTownTypes;
 	std::set<FactionID> monsterTypes;
 	std::set<FactionID> bannedMonsters;
 
-	std::map<TResource, ui16> mines; //obligatory mines to spawn in this zone
+	std::map<GameResID, ui16> mines; //obligatory mines to spawn in this zone
 
 	std::vector<CTreasureInfo> treasureInfo;
 
 	std::vector<TRmgTemplateZoneId> connectedZoneIds; //list of adjacent zone ids
 	std::vector<ZoneConnection> connectionDetails; //list of connections linked to that zone
 
+	TRmgTemplateZoneId townsLikeZone;
 	TRmgTemplateZoneId minesLikeZone;
 	TRmgTemplateZoneId terrainTypeLikeZone;
 	TRmgTemplateZoneId treasureLikeZone;
 	TRmgTemplateZoneId customObjectsLikeZone;
+	EZoneLevel forcedLevel;
 };
 
 }
@@ -242,11 +307,18 @@ protected:
 /// The CRmgTemplate describes a random map template.
 class DLL_LINKAGE CRmgTemplate : boost::noncopyable
 {
+#ifdef ENABLE_TEMPLATE_EDITOR
+	friend class ::TemplateEditor;
+#endif
+
 public:
 	using Zones = std::map<TRmgTemplateZoneId, std::shared_ptr<rmg::ZoneOptions>>;
 
 	class DLL_LINKAGE CPlayerCountRange
 	{
+#ifdef ENABLE_TEMPLATE_EDITOR
+		friend class ::TemplateEditor;
+#endif
 	public:
 		void addRange(int lower, int upper);
 		void addNumber(int value);
@@ -283,6 +355,15 @@ public:
 	const JsonNode & getMapSettings() const;
 	const std::vector<rmg::ZoneConnection> & getConnectedZoneIds() const;
 
+	const std::set<SpellID> & getBannedSpells() const { return bannedSpells; }
+	const std::set<ArtifactID> & getBannedArtifacts() const { return bannedArtifacts; }
+	const std::set<SecondarySkill> & getBannedSkills() const { return bannedSkills; }
+	const std::set<HeroTypeID> & getBannedHeroes() const { return bannedHeroes; }
+	const std::set<SpellID> & getEnabledSpells() const { return enabledSpells; }
+	const std::set<ArtifactID> & getEnabledArtifacts() const { return enabledArtifacts; }
+	const std::set<SecondarySkill> & getEnabledSkills() const { return enabledSkills; }
+	const std::set<HeroTypeID> & getEnabledHeroes() const { return enabledHeroes; }
+
 	void validate() const; /// Tests template on validity and throws exception on failure
 
 	void serializeJson(JsonSerializeFormat & handler);
@@ -301,12 +382,21 @@ private:
 	std::set<EWaterContent::EWaterContent> allowedWaterContent;
 	std::unique_ptr<JsonNode> mapSettings;
 
+	std::set<SpellID> bannedSpells;
+	std::set<ArtifactID> bannedArtifacts;
+	std::set<SecondarySkill> bannedSkills;
+	std::set<HeroTypeID> bannedHeroes;
+
+	std::set<SpellID> enabledSpells;
+	std::set<ArtifactID> enabledArtifacts;
+	std::set<SecondarySkill> enabledSkills;
+	std::set<HeroTypeID> enabledHeroes;
+
 	std::set<TerrainId> inheritTerrainType(std::shared_ptr<rmg::ZoneOptions> zone, uint32_t iteration = 0);
-	std::map<TResource, ui16> inheritMineTypes(std::shared_ptr<rmg::ZoneOptions> zone, uint32_t iteration = 0);
+	std::map<GameResID, ui16> inheritMineTypes(std::shared_ptr<rmg::ZoneOptions> zone, uint32_t iteration = 0);
 	std::vector<CTreasureInfo> inheritTreasureInfo(std::shared_ptr<rmg::ZoneOptions> zone, uint32_t iteration = 0);
 
-	// TODO: Copy custom object settings
-	// TODO: Copy town type after source town is actually randomized
+	void inheritTownProperties(std::shared_ptr<rmg::ZoneOptions> zone, uint32_t iteration = 0);
 
 	void serializeSize(JsonSerializeFormat & handler, int3 & value, const std::string & fieldName);
 	void serializePlayers(JsonSerializeFormat & handler, CPlayerCountRange & value, const std::string & fieldName);
@@ -321,4 +411,4 @@ private:
 
 };
 
-VCMI_LIB_NAMESPACE_END
\ No newline at end of file
+VCMI_LIB_NAMESPACE_END
