@@ -20,6 +20,7 @@ class CGObjectInstance;
 
 struct DLL_LINKAGE ObjectInfo
 {
+	ObjectInfo() = default;
 	ObjectInfo(si32 ID, si32 subID);
 	ObjectInfo(CompoundMapObjectID id);
 	ObjectInfo(const ObjectInfo & other);
@@ -32,8 +33,8 @@ struct DLL_LINKAGE ObjectInfo
 	ui16 probability = 0;
 	ui32 maxPerZone = 1;
 	//ui32 maxPerMap; //unused
-	std::function<CGObjectInstance *()> generateObject;
-	std::function<void(CGObjectInstance *)> destroyObject;
+	std::function<std::shared_ptr<CGObjectInstance>()> generateObject;
+	std::function<void(CGObjectInstance &)> destroyObject;
 	
 	void setAllTemplates(MapObjectID type, MapObjectSubID subtype);
 	void setTemplates(MapObjectID type, MapObjectSubID subtype, TerrainId terrain);
@@ -42,4 +43,4 @@ struct DLL_LINKAGE ObjectInfo
 	//bool matchesId(const CompoundMapObjectID & id) const;
 };
 
-VCMI_LIB_NAMESPACE_END
\ No newline at end of file
+VCMI_LIB_NAMESPACE_END
