@@ -35,7 +35,7 @@ const std::string & Modificator::getName() const
 
 bool Modificator::isReady()
 {
-	Lock lock(mx, boost::try_to_lock);
+	Lock lock(mx, std::try_to_lock);
 	if (!lock.owns_lock())
 	{
 		return false;
@@ -63,7 +63,7 @@ bool Modificator::isReady()
 
 bool Modificator::isFinished()
 {
-	Lock lock(mx, boost::try_to_lock);
+	Lock lock(mx, std::try_to_lock);
 	if (!lock.owns_lock())
 	{
 		return false;
@@ -100,6 +100,7 @@ void Modificator::run()
 
 void Modificator::dependency(Modificator * modificator)
 {
+	//TODO: Check for circular dependencies
 	if(modificator && modificator != this)
 	{
 		//TODO: use vstd::contains
@@ -110,6 +111,7 @@ void Modificator::dependency(Modificator * modificator)
 
 void Modificator::postfunction(Modificator * modificator)
 {
+	// TODO:Check for circular dependencies
 	if(modificator && modificator != this)
 	{
 		if(std::find(modificator->preceeders.begin(), modificator->preceeders.end(), this) == modificator->preceeders.end())
