@@ -13,7 +13,7 @@
 #include "../../GameConstants.h"
 #include "../../int3.h"
 #include "../Zone.h"
-#include "../threadpool/MapProxy.h"
+#include "../MapProxy.h"
 
 class RmgMap;
 class CMapGenerator;
@@ -62,9 +62,9 @@ protected:
 
 	bool finished = false;
 	
-	mutable boost::recursive_mutex externalAccessMutex; //Used to communicate between Modificators
-	using RecursiveLock = boost::unique_lock<boost::recursive_mutex>;
-	using Lock = boost::unique_lock<boost::shared_mutex>;
+	mutable std::recursive_mutex externalAccessMutex; //Used to communicate between Modificators
+	using RecursiveLock = std::unique_lock<std::recursive_mutex>;
+	using Lock = std::unique_lock<std::shared_mutex>;
 
 private:
 	virtual void process() = 0;
@@ -73,9 +73,9 @@ private:
 
 	std::list<Modificator*> preceeders; //must be ordered container
 
-	mutable boost::shared_mutex mx; //Used only for task scheduling
+	mutable std::shared_mutex mx; //Used only for task scheduling
 
 	void dump();
 };
 
-VCMI_LIB_NAMESPACE_END
\ No newline at end of file
+VCMI_LIB_NAMESPACE_END
