@@ -21,7 +21,6 @@
 #include "../CZonePlacer.h"
 #include "PrisonHeroPlacer.h"
 #include "QuestArtifactPlacer.h"
-#include "../../ArtifactUtils.h"
 #include "../../mapObjectConstructors/AObjectTypeHandler.h"
 #include "../../mapObjectConstructors/CObjectClassesHandler.h"
 #include "../../mapObjectConstructors/DwellingInstanceConstructor.h"
@@ -49,7 +48,7 @@ void TreasurePlacer::process()
 
 	tierValues = generator.getConfig().pandoraCreatureValues;
 	// Add all native creatures
-	for(auto const & cre : VLC->creh->objects)
+	for(auto const & cre : LIBRARY->creh->objects)
 	{
 		if(!cre->special && cre->getFactionID() == zone.getTownType())
 		{
@@ -109,11 +108,22 @@ void TreasurePlacer::addAllPossibleObjects()
 
 void TreasurePlacer::addCommonObjects()
 {
-	for(auto primaryID : VLC->objtypeh->knownObjects())
+	//objects with these IDs are added elsewhere, see addAllPossibleObjects()
+	std::set<MapObjectID> excludedIDs = {
+		Obj::PRISON,
+		Obj::CREATURE_GENERATOR1,
+		Obj::CREATURE_GENERATOR4,
+		Obj::SPELL_SCROLL,
+		Obj::PANDORAS_BOX,
+		Obj::SEER_HUT
+	};
+	for(auto primaryID : LIBRARY->objtypeh->knownObjects())
 	{
-		for(auto secondaryID : VLC->objtypeh->knownSubObjects(primaryID))
+		if(excludedIDs.find(primaryID) != excludedIDs.end())
+			continue;
+		for(auto secondaryID : LIBRARY->objtypeh->knownSubObjects(primaryID))
 		{
-			auto handler = VLC->objtypeh->getHandlerFor(primaryID, secondaryID);
+			auto handler = LIBRARY->objtypeh->getHandlerFor(primaryID, secondaryID);
 			if(!handler->isStaticObject() && handler->getRMGInfo().value)
 			{
 				auto rmgInfo = handler->getRMGInfo();
@@ -138,9 +148,13 @@ void TreasurePlacer::addCommonObjects()
 
 void TreasurePlacer::setBasicProperties(ObjectInfo & oi, CompoundMapObjectID objid) const
 {
-	oi.generateObject = [this, objid]() -> CGObjectInstance *
+	oi.generateObject = [this, objid]() -> std::shared_ptr<CGObjectInstance>
 	{
-		return VLC->objtypeh->getHandlerFor(objid)->create(map.mapInstance->cb, nullptr);
+		auto obj = LIBRARY->objtypeh->getHandlerFor(objid)->create(map.mapInstance->cb, nullptr);
+		// adjust ownership for ownable objects (such as dwellings)
+		if (obj->asOwnable() && obj->tempOwner == PlayerColor::UNFLAGGABLE)
+			obj->setOwner(PlayerColor::NEUTRAL);		
+		return obj;
 	};
 	oi.setTemplates(objid.primaryID, objid.secondaryID, zone.getTerrainType());
 }
@@ -148,7 +162,7 @@ void TreasurePlacer::setBasicProperties(ObjectInfo & oi, CompoundMapObjectID obj
 void TreasurePlacer::addPrisons()
 {
 	//Generate Prison on water only if it has a template
-	auto prisonTemplates = VLC->objtypeh->getHandlerFor(Obj::PRISON, 0)->getTemplates(zone.getTerrainType());
+	auto prisonTemplates = LIBRARY->objtypeh->getHandlerFor(Obj::PRISON, 0)->getTemplates(zone.getTerrainType());
 	if (!prisonTemplates.empty())
 	{
 		PrisonHeroPlacer * prisonHeroPlacer = nullptr;
@@ -176,11 +190,11 @@ void TreasurePlacer::addPrisons()
 				continue;
 			}
 
-			oi.generateObject = [i, this, prisonHeroPlacer]() -> CGObjectInstance*
+			oi.generateObject = [i, this, prisonHeroPlacer]() -> std::shared_ptr<CGObjectInstance>
 			{
 				HeroTypeID hid = prisonHeroPlacer->drawRandomHero();
-				auto factory = VLC->objtypeh->getHandlerFor(Obj::PRISON, 0);
-				auto* obj = dynamic_cast<CGHeroInstance*>(factory->create(map.mapInstance->cb, nullptr));
+				auto factory = LIBRARY->objtypeh->getHandlerFor(Obj::PRISON, 0);
+				auto obj = std::dynamic_pointer_cast<CGHeroInstance>(factory->create(map.mapInstance->cb, nullptr));
 
 				obj->setHeroType(hid); //will be initialized later
 				obj->exp = generator.getConfig().prisonExperience[i];
@@ -188,11 +202,11 @@ void TreasurePlacer::addPrisons()
 
 				return obj;
 			};
-			oi.destroyObject = [prisonHeroPlacer](CGObjectInstance* obj)
+			oi.destroyObject = [prisonHeroPlacer](CGObjectInstance& obj)
 			{
 				// Hero can be used again
-				auto* hero = dynamic_cast<CGHeroInstance*>(obj);
-				prisonHeroPlacer->restoreDrawnHero(hero->getHeroTypeID());
+				auto & hero = dynamic_cast<CGHeroInstance&>(obj);
+				prisonHeroPlacer->restoreDrawnHero(hero.getHeroTypeID());
 			};
 
 			oi.setTemplates(Obj::PRISON, 0, zone.getTerrainType());
@@ -218,7 +232,7 @@ void TreasurePlacer::addDwellings()
 	
 	for(auto dwellingType : dwellingTypes)
 	{
-		auto subObjects = VLC->objtypeh->knownSubObjects(dwellingType);
+		auto subObjects = LIBRARY->objtypeh->knownSubObjects(dwellingType);
 		
 		if(dwellingType == Obj::CREATURE_GENERATOR1)
 		{
@@ -230,7 +244,7 @@ void TreasurePlacer::addDwellings()
 		
 		for(auto secondaryID : subObjects)
 		{
-			const auto * dwellingHandler = dynamic_cast<const DwellingInstanceConstructor *>(VLC->objtypeh->getHandlerFor(dwellingType, secondaryID).get());
+			const auto * dwellingHandler = dynamic_cast<const DwellingInstanceConstructor *>(LIBRARY->objtypeh->getHandlerFor(dwellingType, secondaryID).get());
 			auto creatures = dwellingHandler->getProducedCreatures();
 			if(creatures.empty())
 				continue;
@@ -238,19 +252,29 @@ void TreasurePlacer::addDwellings()
 			const auto * cre = creatures.front();
 			if(cre->getFactionID() == zone.getTownType())
 			{
-				auto nativeZonesCount = static_cast<float>(map.getZoneCount(cre->getFactionID()));
+				float nativeZonesCount = static_cast<float>(map.getZoneCount(cre->getFactionID()));
+				// value increases, if there are more native zones for the faction
+				float valueModifier = 1 + (nativeZonesCount / map.getTotalZoneCount()) + (nativeZonesCount / 2);
 				ObjectInfo oi(dwellingType, secondaryID);
 				setBasicProperties(oi, CompoundMapObjectID(dwellingType, secondaryID));
 
-				oi.value = static_cast<ui32>(cre->getAIValue() * cre->getGrowth() * (1 + (nativeZonesCount / map.getTotalZoneCount()) + (nativeZonesCount / 2)));
-				oi.probability = 40;
-				
-				oi.generateObject = [this, secondaryID, dwellingType]() -> CGObjectInstance *
+				auto rmgInfo = LIBRARY->objtypeh->getHandlerFor(dwellingType, secondaryID)->getRMGInfo();
+				// rmg info set for dwelling
+				if(rmgInfo.value)
 				{
-					auto * obj = VLC->objtypeh->getHandlerFor(dwellingType, secondaryID)->create(map.mapInstance->cb, nullptr);
-					obj->tempOwner = PlayerColor::NEUTRAL;
-					return obj;
-				};
+					if (rmgInfo.value > zone.getMaxTreasureValue())
+						continue;
+					oi.value = rmgInfo.value * valueModifier;
+					oi.probability = rmgInfo.rarity;
+					if (rmgInfo.zoneLimit != std::numeric_limits<ui32>::max())
+						oi.maxPerZone = rmgInfo.zoneLimit;
+					// FIXME: rmgInfo.mapLimit is not allowed for dwellings
+				}
+				else
+				{
+					oi.value = static_cast<ui32>(cre->getAIValue() * cre->getGrowth() * valueModifier);
+					oi.probability = 40;
+				}
 				if(!oi.templates.empty())
 					addObjectToRandomPool(oi);
 			}
@@ -267,19 +291,19 @@ void TreasurePlacer::addScrolls()
 
 	for(int i = 0; i < generator.getConfig().scrollValues.size(); i++)
 	{
-		oi.generateObject = [i, this]() -> CGObjectInstance *
+		oi.generateObject = [i, this]() -> std::shared_ptr<CGObjectInstance>
 		{
-			auto factory = VLC->objtypeh->getHandlerFor(Obj::SPELL_SCROLL, 0);
-			auto * obj = dynamic_cast<CGArtifact *>(factory->create(map.mapInstance->cb, nullptr));
+			auto factory = LIBRARY->objtypeh->getHandlerFor(Obj::SPELL_SCROLL, 0);
+			auto obj = std::dynamic_pointer_cast<CGArtifact>(factory->create(map.mapInstance->cb, nullptr));
 			std::vector<SpellID> out;
 			
-			for(auto spellID : VLC->spellh->getDefaultAllowed())
+			for(auto spellID : LIBRARY->spellh->getDefaultAllowed())
 			{
 				if(map.isAllowedSpell(spellID) && spellID.toSpell()->getLevel() == i + 1)
 					out.push_back(spellID);
 			}
-			auto * a = ArtifactUtils::createScroll(*RandomGeneratorUtil::nextItem(out, zone.getRand()));
-			obj->storedArtifact = a;
+			auto * a = mapProxy->createScroll(*RandomGeneratorUtil::nextItem(out, zone.getRand()));
+			obj->setArtifactInstance(a);
 			return obj;
 		};
 		oi.setTemplates(Obj::SPELL_SCROLL, 0, zone.getTerrainType());
@@ -307,10 +331,10 @@ void TreasurePlacer::addPandoraBoxesWithGold()
 	ObjectInfo oi(Obj::PANDORAS_BOX, 0);
 	for(int i = 1; i < 5; i++)
 	{
-		oi.generateObject = [this, i]() -> CGObjectInstance *
+		oi.generateObject = [this, i]() -> std::shared_ptr<CGObjectInstance>
 		{
-			auto factory = VLC->objtypeh->getHandlerFor(Obj::PANDORAS_BOX, 0);
-			auto * obj = dynamic_cast<CGPandoraBox *>(factory->create(map.mapInstance->cb, nullptr));
+			auto factory = LIBRARY->objtypeh->getHandlerFor(Obj::PANDORAS_BOX, 0);
+			auto obj = std::dynamic_pointer_cast<CGPandoraBox>(factory->create(map.mapInstance->cb, nullptr));
 			
 			Rewardable::VisitInfo reward;
 			reward.reward.resources[EGameResID::GOLD] = i * 5000;
@@ -332,10 +356,10 @@ void TreasurePlacer::addPandoraBoxesWithExperience()
 	ObjectInfo oi(Obj::PANDORAS_BOX, 0);
 	for(int i = 1; i < 5; i++)
 	{
-		oi.generateObject = [this, i]() -> CGObjectInstance *
+		oi.generateObject = [this, i]() -> std::shared_ptr<CGObjectInstance>
 		{
-			auto factory = VLC->objtypeh->getHandlerFor(Obj::PANDORAS_BOX, 0);
-			auto * obj = dynamic_cast<CGPandoraBox *>(factory->create(map.mapInstance->cb, nullptr));
+			auto factory = LIBRARY->objtypeh->getHandlerFor(Obj::PANDORAS_BOX, 0);
+			auto obj = std::dynamic_pointer_cast<CGPandoraBox>(factory->create(map.mapInstance->cb, nullptr));
 			
 			Rewardable::VisitInfo reward;
 			reward.reward.heroExperience = i * 5000;
@@ -362,10 +386,10 @@ void TreasurePlacer::addPandoraBoxesWithCreatures()
 
 		ObjectInfo oi(Obj::PANDORAS_BOX, 0);
 		
-		oi.generateObject = [this, creature, creaturesAmount]() -> CGObjectInstance *
+		oi.generateObject = [this, creature, creaturesAmount]() -> std::shared_ptr<CGObjectInstance>
 		{
-			auto factory = VLC->objtypeh->getHandlerFor(Obj::PANDORAS_BOX, 0);
-			auto * obj = dynamic_cast<CGPandoraBox *>(factory->create(map.mapInstance->cb, nullptr));
+			auto factory = LIBRARY->objtypeh->getHandlerFor(Obj::PANDORAS_BOX, 0);
+			auto obj = std::dynamic_pointer_cast<CGPandoraBox>(factory->create(map.mapInstance->cb, nullptr));
 			
 			Rewardable::VisitInfo reward;
 			reward.reward.creatures.emplace_back(creature, creaturesAmount);
@@ -388,13 +412,13 @@ void TreasurePlacer::addPandoraBoxesWithSpells()
 	//Pandora with 12 spells of certain level
 	for(int i = 1; i <= GameConstants::SPELL_LEVELS; i++)
 	{
-		oi.generateObject = [i, this]() -> CGObjectInstance *
+		oi.generateObject = [i, this]() -> std::shared_ptr<CGObjectInstance>
 		{
-			auto factory = VLC->objtypeh->getHandlerFor(Obj::PANDORAS_BOX, 0);
-			auto * obj = dynamic_cast<CGPandoraBox *>(factory->create(map.mapInstance->cb, nullptr));
+			auto factory = LIBRARY->objtypeh->getHandlerFor(Obj::PANDORAS_BOX, 0);
+			auto obj = std::dynamic_pointer_cast<CGPandoraBox>(factory->create(map.mapInstance->cb, nullptr));
 
 			std::vector <const CSpell *> spells;
-			for(auto spellID : VLC->spellh->getDefaultAllowed())
+			for(auto spellID : LIBRARY->spellh->getDefaultAllowed())
 			{
 				if(map.isAllowedSpell(spellID) && spellID.toSpell()->getLevel() == i)
 					spells.push_back(spellID.toSpell());
@@ -421,13 +445,13 @@ void TreasurePlacer::addPandoraBoxesWithSpells()
 	//Pandora with 15 spells of certain school
 	for(int i = 0; i < 4; i++)
 	{
-		oi.generateObject = [i, this]() -> CGObjectInstance *
+		oi.generateObject = [i, this]() -> std::shared_ptr<CGObjectInstance>
 		{
-			auto factory = VLC->objtypeh->getHandlerFor(Obj::PANDORAS_BOX, 0);
-			auto * obj = dynamic_cast<CGPandoraBox *>(factory->create(map.mapInstance->cb, nullptr));
+			auto factory = LIBRARY->objtypeh->getHandlerFor(Obj::PANDORAS_BOX, 0);
+			auto obj = std::dynamic_pointer_cast<CGPandoraBox>(factory->create(map.mapInstance->cb, nullptr));
 
 			std::vector <const CSpell *> spells;
-			for(auto spellID : VLC->spellh->getDefaultAllowed())
+			for(auto spellID : LIBRARY->spellh->getDefaultAllowed())
 			{
 				if(map.isAllowedSpell(spellID) && spellID.toSpell()->hasSchool(SpellSchool(i)))
 					spells.push_back(spellID.toSpell());
@@ -453,13 +477,13 @@ void TreasurePlacer::addPandoraBoxesWithSpells()
 	
 	// Pandora box with 60 random spells
 	
-	oi.generateObject = [this]() -> CGObjectInstance *
+	oi.generateObject = [this]() -> std::shared_ptr<CGObjectInstance>
 	{
-		auto factory = VLC->objtypeh->getHandlerFor(Obj::PANDORAS_BOX, 0);
-		auto * obj = dynamic_cast<CGPandoraBox *>(factory->create(map.mapInstance->cb, nullptr));
+		auto factory = LIBRARY->objtypeh->getHandlerFor(Obj::PANDORAS_BOX, 0);
+		auto obj = std::dynamic_pointer_cast<CGPandoraBox>(factory->create(map.mapInstance->cb, nullptr));
 
 		std::vector <const CSpell *> spells;
-		for(auto spellID : VLC->spellh->getDefaultAllowed())
+		for(auto spellID : LIBRARY->spellh->getDefaultAllowed())
 		{
 			if(map.isAllowedSpell(spellID))
 				spells.push_back(spellID.toSpell());
@@ -488,6 +512,7 @@ void TreasurePlacer::addSeerHuts()
 	//Seer huts with creatures or generic rewards
 
 	ObjectInfo oi(Obj::SEER_HUT, 0);
+	const auto seerHutPlacementValue = static_cast<ui32>(generator.getConfig().seerHutValue);
 
 	if(zone.getConnectedZoneIds().size()) //Unlikely, but...
 	{
@@ -505,6 +530,7 @@ void TreasurePlacer::addSeerHuts()
 		
 		//Generate Seer Hut one by one. Duplicated oi possible and should work fine.
 		oi.maxPerZone = 1;
+		oi.value = seerHutPlacementValue;
 
 		std::vector<ObjectInfo> possibleSeerHuts;
 		//14 creatures per town + 4 for each of gold / exp reward
@@ -512,17 +538,17 @@ void TreasurePlacer::addSeerHuts()
 		
 		RandomGeneratorUtil::randomShuffle(creatures, zone.getRand());
 
-		auto setRandomArtifact = [qap](CGSeerHut * obj)
+		auto setRandomArtifact = [qap](CGSeerHut * obj, ui32 rewardValue)
 		{
 			ArtifactID artid = qap->drawRandomArtifact();
-			obj->quest->mission.artifacts.push_back(artid);
-			qap->addQuestArtifact(artid);
+			obj->getQuest().mission.artifacts.push_back(artid);
+			qap->addQuestArtifact(artid, rewardValue);
 		};
-		auto destroyObject = [qap](CGObjectInstance * obj)
+		auto destroyObject = [qap](CGObjectInstance & obj)
 		{
-			auto * seer = dynamic_cast<CGSeerHut *>(obj);
+			auto & seer = dynamic_cast<CGSeerHut &>(obj);
 			// Artifact can be used again
-			ArtifactID artid = seer->quest->mission.artifacts.front();
+			ArtifactID artid = seer.getQuest().mission.artifacts.front();
 			qap->addRandomArtifact(artid);
 			qap->removeQuestArtifact(artid);
 		};
@@ -538,33 +564,31 @@ void TreasurePlacer::addSeerHuts()
 			int randomAppearance = chooseRandomAppearance(zone.getRand(), Obj::SEER_HUT, zone.getTerrainType());
 			
 			// FIXME: Remove duplicated code for gold, exp and creaure reward
-			oi.generateObject = [cb=map.mapInstance->cb, creature, creaturesAmount, randomAppearance, setRandomArtifact]() -> CGObjectInstance *
+			const auto rewardValue = static_cast<ui32>(((2 * (creature->getAIValue()) * creaturesAmount * (1 + static_cast<float>(map.getZoneCount(creature->getFactionID())) / map.getTotalZoneCount())) - 4000) / 3);
+
+			if (rewardValue > zone.getMaxTreasureValue())
 			{
-				auto factory = VLC->objtypeh->getHandlerFor(Obj::SEER_HUT, randomAppearance);
-				auto * obj = dynamic_cast<CGSeerHut *>(factory->create(cb, nullptr));
+				continue;
+			}
+
+			oi.generateObject = [cb=map.mapInstance->cb, creature, creaturesAmount, randomAppearance, setRandomArtifact, rewardValue]() -> std::shared_ptr<CGObjectInstance>
+			{
+				auto factory = LIBRARY->objtypeh->getHandlerFor(Obj::SEER_HUT, randomAppearance);
+				auto obj = std::dynamic_pointer_cast<CGSeerHut>(factory->create(cb, nullptr));
 				
 				Rewardable::VisitInfo reward;
 				reward.reward.creatures.emplace_back(creature->getId(), creaturesAmount);
 				reward.visitType = Rewardable::EEventType::EVENT_FIRST_VISIT;
 				obj->configuration.info.push_back(reward);
-								
-				setRandomArtifact(obj);
+				setRandomArtifact(obj.get(), rewardValue);
 				
 				return obj;
 			};
 			oi.destroyObject = destroyObject;
 			oi.probability = 3;
 			oi.setTemplates(Obj::SEER_HUT, randomAppearance, zone.getTerrainType());
-			oi.value = static_cast<ui32>(((2 * (creature->getAIValue()) * creaturesAmount * (1 + static_cast<float>(map.getZoneCount(creature->getFactionID())) / map.getTotalZoneCount())) - 4000) / 3);
-			if (oi.value > zone.getMaxTreasureValue())
-			{
-				continue;
-			}
-			else
-			{
-				if(!oi.templates.empty())
-					possibleSeerHuts.push_back(oi);
-			}
+			if(!oi.templates.empty())
+				possibleSeerHuts.push_back(oi);
 		}
 		
 		static const int seerLevels = std::min(generator.getConfig().questValues.size(), generator.getConfig().questRewardValues.size());
@@ -573,27 +597,25 @@ void TreasurePlacer::addSeerHuts()
 			int randomAppearance = chooseRandomAppearance(zone.getRand(), Obj::SEER_HUT, zone.getTerrainType());
 			
 			oi.setTemplates(Obj::SEER_HUT, randomAppearance, zone.getTerrainType());
-			oi.value = generator.getConfig().questValues[i];
-			if (oi.value > zone.getMaxTreasureValue())
+			const auto rewardValue = static_cast<ui32>(generator.getConfig().questRewardValues[i]);
+			if (rewardValue > zone.getMaxTreasureValue())
 			{
-				//Both variants have same value
 				continue;
 			}
 
 			oi.probability = 10;
 			oi.maxPerZone = 1;
 			
-			oi.generateObject = [i, randomAppearance, this, setRandomArtifact]() -> CGObjectInstance *
+			oi.generateObject = [i, randomAppearance, this, setRandomArtifact, rewardValue]() -> std::shared_ptr<CGObjectInstance>
 			{
-				auto factory = VLC->objtypeh->getHandlerFor(Obj::SEER_HUT, randomAppearance);
-				auto * obj = dynamic_cast<CGSeerHut *>(factory->create(map.mapInstance->cb, nullptr));
+				auto factory = LIBRARY->objtypeh->getHandlerFor(Obj::SEER_HUT, randomAppearance);
+				auto obj = std::dynamic_pointer_cast<CGSeerHut>(factory->create(map.mapInstance->cb, nullptr));
 				
 				Rewardable::VisitInfo reward;
 				reward.reward.heroExperience = generator.getConfig().questRewardValues[i];
 				reward.visitType = Rewardable::EEventType::EVENT_FIRST_VISIT;
 				obj->configuration.info.push_back(reward);
-
-				setRandomArtifact(obj);
+				setRandomArtifact(obj.get(), rewardValue);
 
 				return obj;
 			};
@@ -602,17 +624,17 @@ void TreasurePlacer::addSeerHuts()
 			if(!oi.templates.empty())
 				possibleSeerHuts.push_back(oi);
 			
-			oi.generateObject = [i, randomAppearance, this, setRandomArtifact]() -> CGObjectInstance *
+			oi.generateObject = [i, randomAppearance, this, setRandomArtifact, rewardValue]() -> std::shared_ptr<CGObjectInstance>
 			{
-				auto factory = VLC->objtypeh->getHandlerFor(Obj::SEER_HUT, randomAppearance);
-				auto * obj = dynamic_cast<CGSeerHut *>(factory->create(map.mapInstance->cb, nullptr));
+				auto factory = LIBRARY->objtypeh->getHandlerFor(Obj::SEER_HUT, randomAppearance);
+				auto obj = std::dynamic_pointer_cast<CGSeerHut>(factory->create(map.mapInstance->cb, nullptr));
 				
 				Rewardable::VisitInfo reward;
 				reward.reward.resources[EGameResID::GOLD] = generator.getConfig().questRewardValues[i];
 				reward.visitType = Rewardable::EEventType::EVENT_FIRST_VISIT;
 				obj->configuration.info.push_back(reward);
 				
-				setRandomArtifact(obj);
+				setRandomArtifact(obj.get(), rewardValue);
 				
 				return obj;
 			};
@@ -748,15 +770,16 @@ rmg::Object TreasurePlacer::constructTreasurePile(const std::vector<ObjectInfo*>
 			accessibleArea.add(int3());
 		}
 		
-		CGObjectInstance * object = nullptr;
+		std::shared_ptr<CGObjectInstance> object = nullptr;
 		if (oi->generateObject)
 		{
 			object = oi->generateObject();
+			object->rmgValue = oi->value;
 			if(oi->templates.empty())
 			{
 				logGlobal->warn("Deleting randomized object with no templates: %s", object->getObjectName());
-				oi->destroyObject(object);
-				delete object;
+				if (oi->destroyObject)
+					oi->destroyObject(*object);
 				continue;
 			}
 		}
@@ -770,7 +793,7 @@ rmg::Object TreasurePlacer::constructTreasurePile(const std::vector<ObjectInfo*>
 
 		if (templates.empty())
 		{
-			throw rmgException(boost::str(boost::format("Did not find template for object (%d,%d) at %s") % object->ID % object->subID % zone.getTerrainType().encode(zone.getTerrainType())));
+			throw rmgException(boost::str(boost::format("Did not find template for object (%d,%d) at %s") % object->ID.getNum() % object->subID.getNum() % zone.getTerrainType().encode(zone.getTerrainType().getNum())));
 		}
 
 		object->appearance = *RandomGeneratorUtil::nextItem(templates, zone.getRand());
@@ -785,7 +808,7 @@ rmg::Object TreasurePlacer::constructTreasurePile(const std::vector<ObjectInfo*>
 			});
 		}
 
-		auto & instance = rmgObject.addInstance(*object);
+		auto & instance = rmgObject.addInstance(object);
 		rmgObject.setValue(rmgObject.getValue() + oi->value);
 		instance.onCleared = oi->destroyObject;
 
@@ -1150,7 +1173,7 @@ void TreasurePlacer::ObjectPool::patchWithZoneConfig(const Zone & zone, Treasure
 			auto category = getObjectCategory(oi.getCompoundID());
 			if (categoriesSet.count(category))
 			{
-				logGlobal->info("Removing object %s from possible objects", oi.templates.front()->stringID);
+				logGlobal->debug("Removing object %s from possible objects", oi.templates.front()->stringID);
 				return true;
 			}
 			return false;
@@ -1163,10 +1186,11 @@ void TreasurePlacer::ObjectPool::patchWithZoneConfig(const Zone & zone, Treasure
 			for (const auto & templ : object.templates)
 			{
 				CompoundMapObjectID key = object.getCompoundID();
-				if (bannedObjectsSet.count(key))
+				CompoundMapObjectID keyGroup( key.primaryID, -1);
+				if (bannedObjectsSet.count(key) || bannedObjectsSet.count(keyGroup))
 				{
 					// FIXME: Stopped working, nothing is banned
-					logGlobal->info("Banning object %s from possible objects", templ->stringID);
+					logGlobal->debug("Banning object %s from possible objects", templ->stringID);
 					return true;
 				}
 			}
@@ -1181,7 +1205,7 @@ void TreasurePlacer::ObjectPool::patchWithZoneConfig(const Zone & zone, Treasure
 	{
 		tp->setBasicProperties(object, object.getCompoundID());
 		addObject(object);
-		logGlobal->info("Added custom object of type %d.%d", object.primaryID, object.secondaryID);
+		logGlobal->debug("Added custom object of type %d.%d", object.primaryID, object.secondaryID);
 	}
 	// TODO: Overwrite or add to possibleObjects
 
@@ -1218,11 +1242,11 @@ void TreasurePlacer::ObjectPool::discardObjectsAboveValue(ui32 value)
 
 ObjectConfig::EObjectCategory TreasurePlacer::ObjectPool::getObjectCategory(CompoundMapObjectID id)
 {
-	auto name = VLC->objtypeh->getObjectHandlerName(id.primaryID);
+	auto name = LIBRARY->objtypeh->getObjectHandlerName(id.primaryID);
 
 	if (name == "configurable")
 	{
-		auto handler = VLC->objtypeh->getHandlerFor(id.primaryID, id.secondaryID);
+		auto handler = LIBRARY->objtypeh->getHandlerFor(id.primaryID, id.secondaryID);
 		if (!handler)
 		{
 			return ObjectConfig::EObjectCategory::NONE;
