@@ -31,15 +31,28 @@ public:
 	CMemorySerializer();
 
 	template <typename T>
-	static std::unique_ptr<T> deepCopy(const T &data)
+	static std::unique_ptr<T> deepCopy(const T &data, IGameInfoCallback * cb = nullptr)
 	{
 		CMemorySerializer mem;
 		mem.oser & &data;
+		mem.iser.cb = cb;
 
 		std::unique_ptr<T> ret;
 		mem.iser & ret;
 		return ret;
 	}
+
+	template <typename T>
+	static std::shared_ptr<T> deepCopyShared(const T &data, IGameInfoCallback * cb = nullptr)
+	{
+		CMemorySerializer mem;
+		mem.oser & &data;
+		mem.iser.cb = cb;
+
+		std::shared_ptr<T> ret;
+		mem.iser & ret;
+		return ret;
+	}
 };
 
 VCMI_LIB_NAMESPACE_END
