@@ -16,6 +16,7 @@
 #include "../../battle/IBattleState.h"
 #include "../../battle/CBattleInfoCallback.h"
 #include "../../battle/Unit.h"
+#include "../../bonuses/BonusParameters.h"
 #include "../../json/JsonBonus.h"
 #include "../../mapObjects/CGHeroInstance.h"
 #include "../../networkPacks/PacksForClientBattle.h"
@@ -144,10 +145,10 @@ void Timed::apply(ServerCallback * server, const Mechanics * m, const EffectTarg
 
 		//Apply hero specials - peculiar enchants
 		const auto tier = std::max(affected->creatureLevel(), 1); //don't divide by 0 for certain creatures (commanders, war machines)
-		if(peculiarBonus)
+		if(peculiarBonus && peculiarBonus->parameters)
 		{
 			si32 power = 0;
-			switch (peculiarBonus->additionalInfo[0])
+			switch (peculiarBonus->parameters->toNumber())
 			{
 			case 0: //normal
 				switch (tier)
@@ -183,18 +184,18 @@ void Timed::apply(ServerCallback * server, const Mechanics * m, const EffectTarg
 
 		}
 
-		if(addedValueBonus)
+		if(addedValueBonus && addedValueBonus->parameters)
 		{
 			for(Bonus & b : buffer)
 			{
-				b.val += addedValueBonus->additionalInfo[0];
+				b.val += addedValueBonus->parameters->toNumber();
 			}
 		}
-		if(fixedValueBonus)
+		if(fixedValueBonus && fixedValueBonus->parameters)
 		{
 			for(Bonus & b : buffer)
 			{
-				b.val = fixedValueBonus->additionalInfo[0];
+				b.val = fixedValueBonus->parameters->toNumber();
 			}
 		}
 
@@ -232,7 +233,7 @@ void Timed::convertBonus(const Mechanics * m, int32_t & duration, std::vector<Bo
 			nb.val = 100 - nb.val;
 		//we need to know who cast Bind
 		else if(nb.sid.as<SpellID>() == SpellID::BIND && nb.type == BonusType::BIND_EFFECT && m->caster->getHeroCaster() == nullptr)
-			nb.additionalInfo = m->caster->getCasterUnitId();
+			nb.parameters = std::make_shared<BonusParameters>(m->caster->getCasterUnitId());
 
 		converted.push_back(nb);
 	}
