@@ -10,7 +10,11 @@
 #pragma once
 
 #include <QDialog>
-#include "../../lib/mapObjects/CGHeroInstance.h"
+
+VCMI_LIB_NAMESPACE_BEGIN
+class ArtifactPosition;
+class CArtifactFittingSet;
+VCMI_LIB_NAMESPACE_END
 
 namespace Ui {
 class ArtifactWidget;
