@@ -10,12 +10,14 @@
 #include "StdInc.h"
 #include "heroskillswidget.h"
 #include "ui_heroskillswidget.h"
-#include "../../lib/constants/StringConstants.h"
-#include "../../lib/entities/hero/CHeroHandler.h"
-#include "../../lib/CSkillHandler.h"
 #include "inspector.h"
 
-static QList<std::pair<QString, QVariant>> LevelIdentifiers
+#include "../../lib/CSkillHandler.h"
+#include "../../lib/GameLibrary.h"
+#include "../../lib/constants/StringConstants.h"
+#include "../../lib/entities/hero/CHeroHandler.h"
+
+static const QList<std::pair<QString, QVariant>> LevelIdentifiers
 {
 	{QObject::tr("Beginner"), QVariant::fromValue(1)},
 	{QObject::tr("Advanced"), QVariant::fromValue(2)},
@@ -35,7 +37,7 @@ HeroSkillsWidget::HeroSkillsWidget(CGHeroInstance & h, QWidget *parent) :
 	ui->labelKnowledge->setText(QString::fromStdString(NPrimarySkill::names[3]));
 	
 	auto * delegate = new InspectorDelegate;
-	for(auto const & s : VLC->skillh->objects)
+	for(auto const & s : LIBRARY->skillh->objects)
 		delegate->options.push_back({QString::fromStdString(s->getNameTranslated()), QVariant::fromValue(s->getId().getNum())});
 	ui->skills->setItemDelegateForColumn(0, delegate);
 	
@@ -83,7 +85,7 @@ void HeroSkillsWidget::obtainData()
 	for(auto & s : hero.secSkills)
 	{
 		auto * itemSkill = new QTableWidgetItem;
-		itemSkill->setText(QString::fromStdString(VLC->skillh->getById(s.first)->getNameTranslated()));
+		itemSkill->setText(QString::fromStdString(LIBRARY->skillh->getById(s.first)->getNameTranslated()));
 		itemSkill->setData(Qt::UserRole, QVariant::fromValue(s.first.getNum()));
 		ui->skills->setItem(i, 0, itemSkill);
 		
@@ -116,7 +118,9 @@ void HeroSkillsWidget::commitChanges()
 	}
 }
 
-HeroSkillsDelegate::HeroSkillsDelegate(CGHeroInstance & h): hero(h), BaseInspectorItemDelegate()
+HeroSkillsDelegate::HeroSkillsDelegate(CGHeroInstance & h)
+	: BaseInspectorItemDelegate()
+	, hero(h)
 {
 }
 
@@ -153,23 +157,31 @@ void HeroSkillsDelegate::setModelData(QWidget *editor, QAbstractItemModel *model
 void HeroSkillsDelegate::updateModelData(QAbstractItemModel * model, const QModelIndex & index) const
 {
 	QStringList textList;
-	textList += QString("%1: %2").arg(QString::fromStdString(VLC->generaltexth->primarySkillNames[PrimarySkill::ATTACK])).arg(hero.getBasePrimarySkillValue(PrimarySkill::ATTACK));
-	textList += QString("%1: %2").arg(QString::fromStdString(VLC->generaltexth->primarySkillNames[PrimarySkill::DEFENSE])).arg(hero.getBasePrimarySkillValue(PrimarySkill::DEFENSE));
-	textList += QString("%1: %2").arg(QString::fromStdString(VLC->generaltexth->primarySkillNames[PrimarySkill::SPELL_POWER])).arg(hero.getBasePrimarySkillValue(PrimarySkill::SPELL_POWER));
-	textList += QString("%1: %2").arg(QString::fromStdString(VLC->generaltexth->primarySkillNames[PrimarySkill::KNOWLEDGE])).arg(hero.getBasePrimarySkillValue(PrimarySkill::KNOWLEDGE));
-;
+	textList += QString("%1: %2").arg(QString::fromStdString(LIBRARY->generaltexth->primarySkillNames[PrimarySkill::ATTACK])).arg(hero.getBasePrimarySkillValue(PrimarySkill::ATTACK));
+	textList += QString("%1: %2").arg(QString::fromStdString(LIBRARY->generaltexth->primarySkillNames[PrimarySkill::DEFENSE])).arg(hero.getBasePrimarySkillValue(PrimarySkill::DEFENSE));
+	textList += QString("%1: %2").arg(QString::fromStdString(LIBRARY->generaltexth->primarySkillNames[PrimarySkill::SPELL_POWER])).arg(hero.getBasePrimarySkillValue(PrimarySkill::SPELL_POWER));
+	textList += QString("%1: %2").arg(QString::fromStdString(LIBRARY->generaltexth->primarySkillNames[PrimarySkill::KNOWLEDGE])).arg(hero.getBasePrimarySkillValue(PrimarySkill::KNOWLEDGE));
+
 	auto heroSecondarySkills = hero.secSkills;
 	if(heroSecondarySkills.size() == 1 && heroSecondarySkills[0].first == SecondarySkill::NONE) 
 	{
-		textList += QObject::tr("Default secondary skills:");
-		heroSecondarySkills = hero.getHeroType()->secSkillsInit;
+		if(hero.getHeroTypeID().isValid())
+		{
+			textList += QObject::tr("Default secondary skills:");
+			heroSecondarySkills = hero.getHeroType()->secSkillsInit;
+		}
+		else
+		{
+			textList += QObject::tr("Random hero secondary skills");
+			heroSecondarySkills.clear();
+		}
 	}
 	else
 	{
 		textList += QObject::tr("Secondary skills:");
 	}
-	for(auto & [skill, skillLevel] : heroSecondarySkills)
-		textList += QString("%1 %2").arg(QString::fromStdString(VLC->skillh->getById(skill)->getNameTranslated())).arg(LevelIdentifiers[skillLevel - 1].first);
+	for(const auto & [skill, skillLevel] : heroSecondarySkills)
+		textList += QString("%1 %2").arg(QString::fromStdString(LIBRARY->skillh->getById(skill)->getNameTranslated())).arg(LevelIdentifiers[skillLevel - 1].first);
 
 	setModelTextData(model, index, textList);
 }
