@@ -11,6 +11,8 @@
 #include "herospellwidget.h"
 #include "ui_herospellwidget.h"
 #include "inspector.h"
+
+#include "../../lib/GameLibrary.h"
 #include "../../lib/constants/StringConstants.h"
 #include "../../lib/spells/CSpellHandler.h"
 
@@ -48,7 +50,7 @@ void HeroSpellWidget::initSpellLists()
 	for (int i = 0; i < GameConstants::SPELL_LEVELS; i++)
 	{
 		std::vector<const CSpell*> spellsByLevel;
-		for (auto const & spellID : VLC->spellh->getDefaultAllowed())
+		for (auto const & spellID : LIBRARY->spellh->getDefaultAllowed())
 		{
 			if (spellID.toSpell()->getLevel() == i + 1)
 				spellsByLevel.push_back(spellID.toSpell());
@@ -94,14 +96,86 @@ void HeroSpellWidget::on_customizeSpells_toggled(bool checked)
 	}
 	else
 	{
-		hero.removeSpellFromSpellbook(SpellID::PRESET);
-		hero.removeSpellbook();
+		hero.removeAllSpells();
 	}
+	ui->filter->clear();
+	ui->filter->setEnabled(checked);
 	ui->tabWidget->setEnabled(checked);
 	initSpellLists();
 }
 
-HeroSpellDelegate::HeroSpellDelegate(CGHeroInstance & h) : hero(h), BaseInspectorItemDelegate()
+void HeroSpellWidget::on_filter_textChanged(const QString & keyword)
+{
+	if (keyword.toStdString().find_first_not_of(' ') == std::string::npos)
+	{
+		const auto exists = QString::fromStdString(".*");
+		showItemIfMatches(exists);
+	}
+	else
+	{
+		const auto doesNotContainKeyword = QString::fromStdString("^((?!") + keyword + QString::fromStdString(").)*$");
+		hideItemIfMatches(doesNotContainKeyword);
+
+		const auto containsKeyword = QString::fromStdString(".*") + keyword + QString::fromStdString(".*");
+		showItemIfMatches(containsKeyword);
+	}
+
+	hideEmptySpellLists();
+}
+
+void HeroSpellWidget::showItemIfMatches(const QString & match)
+{
+	toggleHiddenForItemIfMatches(match, false);
+}
+
+void HeroSpellWidget::hideItemIfMatches(const QString & match)
+{
+	toggleHiddenForItemIfMatches(match, true);
+}
+
+void HeroSpellWidget::toggleHiddenForItemIfMatches(const QString & match, bool hidden)
+{
+	QListWidget * spellLists[] = { ui->spellList1, ui->spellList2, ui->spellList3, ui->spellList4, ui->spellList5 };
+	for (const QListWidget * list : spellLists)
+	{
+		const auto items = list->findItems(match, Qt::MatchRegularExpression);
+		for (QListWidgetItem * item : items)
+		{
+			item->setHidden(hidden);
+		}
+	}
+}
+
+void HeroSpellWidget::hideEmptySpellLists()
+{
+
+	QListWidget * spellLists[] = { ui->spellList1, ui->spellList2, ui->spellList3, ui->spellList4, ui->spellList5 };
+	auto toggleSpellListVisibility = [&](const QListWidget * list, bool visible)
+	{
+		auto * parent = list->parentWidget();
+		int index = ui->tabWidget->indexOf(parent);
+		ui->tabWidget->setTabVisible(index, visible);
+	};
+
+	for (const QListWidget * list : spellLists)
+	{
+		const auto allItems = list->findItems("*", Qt::MatchWildcard);
+		bool isListEmpty = true;
+		for (QListWidgetItem * item : allItems)
+		{
+			if (!item->isHidden())
+			{
+				isListEmpty = false;
+				break;
+			}
+		}
+		toggleSpellListVisibility(list, !isListEmpty);
+	}
+}
+
+HeroSpellDelegate::HeroSpellDelegate(CGHeroInstance & h)
+	: BaseInspectorItemDelegate()
+	, hero(h)
 {
 }
 
