@@ -31,6 +31,7 @@ public:
 
 private slots:
 	void on_customizeSpells_toggled(bool checked);
+	void on_filter_textChanged(const QString & keyword);
 
 private:
 	Ui::HeroSpellWidget * ui;
@@ -38,6 +39,10 @@ private:
 	CGHeroInstance & hero;
 
 	void initSpellLists();
+	void showItemIfMatches(const QString & match);
+	void hideItemIfMatches(const QString & match);
+	void toggleHiddenForItemIfMatches(const QString & match, bool hidden);
+	void hideEmptySpellLists();
 };
 
 class HeroSpellDelegate : public BaseInspectorItemDelegate
