@@ -21,11 +21,32 @@
 #include "../lib/mapObjects/MiscObjects.h"
 #include "../lib/GameConstants.h"
 
+namespace
+{
 const int tileSize = 32;
 
-static bool objectBlitOrderSorter(const ObjectRect & a, const ObjectRect & b)
+bool objectBlitOrderSorter(const ObjectRect & a, const ObjectRect & b)
 {
-	return MapHandler::compareObjectBlitOrder(a.obj, b.obj);
+	return CMap::compareObjectBlitOrder(a.obj, b.obj);
+}
+
+QImage flippedImage(const std::shared_ptr<QImage> & image, ui8 rotationFlags)
+{
+	const ui8 rotation = rotationFlags % 4;
+	const bool hflip = rotation & 0b01;
+	const bool vflip = rotation & 0b10;
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 9, 0)
+	Qt::Orientations orientations;
+	if(hflip)
+		orientations |= Qt::Horizontal;
+	if(vflip)
+		orientations |= Qt::Vertical;
+	return image->flipped(orientations);
+#else
+	return image->mirrored(hflip, vflip);
+#endif
+}
 }
 
 int MapHandler::index(int x, int y, int z) const
@@ -61,98 +82,86 @@ void MapHandler::initTerrainGraphics()
 			animation[type.first]->preload();
 			const size_t views = animation[type.first]->size(0);
 			cache[type.first].resize(views);
-			
+
 			for(int j = 0; j < views; j++)
 				cache[type.first][j] = animation[type.first]->getImage(j);
 		}
 	};
-	
+
 	std::map<std::string, std::string> terrainFiles;
 	std::map<std::string, std::string> roadFiles;
 	std::map<std::string, std::string> riverFiles;
-	for(const auto & terrain : VLC->terrainTypeHandler->objects)
+	for(const auto & terrain : LIBRARY->terrainTypeHandler->objects)
 	{
 		terrainFiles[terrain->getJsonKey()] = terrain->tilesFilename.getName();
 	}
-	for(const auto & river : VLC->riverTypeHandler->objects)
+	for(const auto & river : LIBRARY->riverTypeHandler->objects)
 	{
 		riverFiles[river->getJsonKey()] = river->tilesFilename.getName();
 	}
-	for(const auto & road : VLC->roadTypeHandler->objects)
+	for(const auto & road : LIBRARY->roadTypeHandler->objects)
 	{
 		roadFiles[road->getJsonKey()] = road->tilesFilename.getName();
 	}
-	
+
 	loadFlipped(terrainAnimations, terrainImages, terrainFiles);
 	loadFlipped(riverAnimations, riverImages, riverFiles);
 	loadFlipped(roadAnimations, roadImages, roadFiles);
 }
 
-void MapHandler::drawTerrainTile(QPainter & painter, int x, int y, int z)
+void MapHandler::drawTerrainTile(QPainter & painter, int x, int y, int z, QPointF offset)
 {
-	auto & tinfo = map->getTile(int3(x, y, z));
-	ui8 rotation = tinfo.extTileFlags % 4;
-	
+	const auto & tinfo = map->getTile(int3(x, y, z));
+
 	auto terrainName = tinfo.getTerrain()->getJsonKey();
-	
 	if(terrainImages.at(terrainName).size() <= tinfo.terView)
 		return;
-	
-	bool hflip = (rotation == 1 || rotation == 3);
-	bool vflip = (rotation == 2 || rotation == 3);
-	painter.drawImage(x * tileSize, y * tileSize, terrainImages.at(terrainName)[tinfo.terView]->mirrored(hflip, vflip));
+	painter.drawImage(x * tileSize - offset.x(), y * tileSize - offset.y(), flippedImage(terrainImages.at(terrainName)[tinfo.terView], tinfo.extTileFlags));
 }
 
-void MapHandler::drawRoad(QPainter & painter, int x, int y, int z)
+void MapHandler::drawRoad(QPainter & painter, int x, int y, int z, QPointF offset)
 {
-	auto & tinfo = map->getTile(int3(x, y, z));
+	const auto & tinfo = map->getTile(int3(x, y, z));
 	auto * tinfoUpper = map->isInTheMap(int3(x, y - 1, z)) ? &map->getTile(int3(x, y - 1, z)) : nullptr;
-	
+
 	if(tinfoUpper && tinfoUpper->roadType)
 	{
 		auto roadName = tinfoUpper->getRoad()->getJsonKey();
-		QRect source(0, tileSize / 2, tileSize, tileSize / 2);
-		ui8 rotation = (tinfoUpper->extTileFlags >> 4) % 4;
-		bool hflip = (rotation == 1 || rotation == 3);
-		bool vflip = (rotation == 2 || rotation == 3);
 		if(roadImages.at(roadName).size() > tinfoUpper->roadDir)
 		{
-			painter.drawImage(QPoint(x * tileSize, y * tileSize), roadImages.at(roadName)[tinfoUpper->roadDir]->mirrored(hflip, vflip), source);
+			const QRect source{0, tileSize / 2, tileSize, tileSize / 2};
+			const ui8 rotationFlags = tinfoUpper->extTileFlags >> 4;
+			painter.drawImage(QPoint(x * tileSize - offset.x(), y * tileSize - offset.y()), flippedImage(roadImages.at(roadName)[tinfoUpper->roadDir], rotationFlags), source);
 		}
 	}
-	
+
 	if(tinfo.roadType) //print road from this tile
 	{
 		auto roadName = tinfo.getRoad()->getJsonKey();
-		QRect source(0, 0, tileSize, tileSize / 2);
-		ui8 rotation = (tinfo.extTileFlags >> 4) % 4;
-		bool hflip = (rotation == 1 || rotation == 3);
-		bool vflip = (rotation == 2 || rotation == 3);
 		if(roadImages.at(roadName).size() > tinfo.roadDir)
 		{
-			painter.drawImage(QPoint(x * tileSize, y * tileSize + tileSize / 2), roadImages.at(roadName)[tinfo.roadDir]->mirrored(hflip, vflip), source);
+			const QRect source{0, 0, tileSize, tileSize / 2};
+			const ui8 rotationFlags = tinfo.extTileFlags >> 4;
+			painter.drawImage(QPoint(x * tileSize - offset.x(), y * tileSize + tileSize / 2 - offset.y()), flippedImage(roadImages.at(roadName)[tinfo.roadDir], rotationFlags), source);
 		}
 	}
 }
 
-void MapHandler::drawRiver(QPainter & painter, int x, int y, int z)
+void MapHandler::drawRiver(QPainter & painter, int x, int y, int z, QPointF offset)
 {
-	auto & tinfo = map->getTile(int3(x, y, z));
+	const auto & tinfo = map->getTile(int3(x, y, z));
 
 	if(!tinfo.hasRiver())
 		return;
-	
+
 	//TODO: use ui8 instead of string key
 	auto riverName = tinfo.getRiver()->getJsonKey();
 
 	if(riverImages.at(riverName).size() <= tinfo.riverDir)
 		return;
 
-	ui8 rotation = (tinfo.extTileFlags >> 2) % 4;
-	bool hflip = (rotation == 1 || rotation == 3);
-	bool vflip = (rotation == 2 || rotation == 3);
-
-	painter.drawImage(x * tileSize, y * tileSize, riverImages.at(riverName)[tinfo.riverDir]->mirrored(hflip, vflip));
+	const ui8 rotationFlags = tinfo.extTileFlags >> 2;
+	painter.drawImage(x * tileSize - offset.x(), y * tileSize - offset.y(), flippedImage(riverImages.at(riverName)[tinfo.riverDir], rotationFlags));
 }
 
 void setPlayerColor(QImage * sur, PlayerColor player)
@@ -174,29 +183,29 @@ void setPlayerColor(QImage * sur, PlayerColor player)
 std::shared_ptr<QImage> MapHandler::getObjectImage(const CGObjectInstance * obj)
 {
 	if(	!obj
-	   || (obj->ID==Obj::HERO && static_cast<const CGHeroInstance*>(obj)->inTownGarrison) //garrisoned hero
-	   || (obj->ID==Obj::BOAT && static_cast<const CGBoat*>(obj)->hero)) //boat with hero (hero graphics is used)
+	   || (obj->ID==Obj::HERO && dynamic_cast<const CGHeroInstance*>(obj)->isGarrisoned()) //garrisoned hero
+	   || (obj->ID==Obj::BOAT && dynamic_cast<const CGBoat*>(obj)->getBoardedHero())) //boat with hero (hero graphics is used)
 	{
 		return nullptr;
 	}
-	
+
 	std::shared_ptr<Animation> animation = graphics->getAnimation(obj);
-	
+
 	//no animation at all
 	if(!animation)
 		return nullptr;
-	
+
 	//empty animation
 	if(animation->size(0) == 0)
 		return nullptr;
-	
+
 	auto image = animation->getImage(0, obj->ID == Obj::HERO ? 2 : 0);
 	if(!image)
 	{
 		//workaround for prisons
 		image = animation->getImage(0, 0);
 	}
-	
+
 	return image;
 }
 
@@ -215,7 +224,7 @@ std::set<int3> MapHandler::removeObject(const CGObjectInstance *object)
 			}
 		}
 	}
-	
+
 	tilesCache.erase(object);
 	return result;
 }
@@ -225,7 +234,7 @@ std::set<int3> MapHandler::addObject(const CGObjectInstance * object)
 	auto image = getObjectImage(object);
 	if(!image)
 		return std::set<int3>{};
-	
+
 	for(int fx = 0; fx < object->getWidth(); ++fx)
 	{
 		for(int fy = 0; fy < object->getHeight(); ++fy)
@@ -235,7 +244,7 @@ std::set<int3> MapHandler::addObject(const CGObjectInstance * object)
 					 image->height() - fy * tileSize - tileSize,
 					 tileSize,
 					 tileSize);
-							
+
 			if( map->isInTheMap(currTile) && // within map
 			   cr.x() + cr.width() > 0 &&    // image has data on this tile
 			   cr.y() + cr.height() > 0)
@@ -245,7 +254,7 @@ std::set<int3> MapHandler::addObject(const CGObjectInstance * object)
 			}
 		}
 	}
-	
+
 	return tilesCache[object];
 }
 
@@ -255,48 +264,22 @@ void MapHandler::initObjectRects()
 	tilesCache.clear();
 	if(!map)
 		return;
-	
-	tileObjects.resize(map->width * map->height * (map->twoLevel ? 2 : 1));
-	
+
+	tileObjects.resize(map->width * map->height * map->levels());
+
 	//initializing objects / rects
-	for(const CGObjectInstance * elem : map->objects)
+	for(const auto & elem : map->objects)
 	{
-		addObject(elem);
+		addObject(elem.get());
 	}
-	
+
 	for(auto & tt : tileObjects)
 		stable_sort(tt.begin(), tt.end(), objectBlitOrderSorter);
 }
 
-bool MapHandler::compareObjectBlitOrder(const CGObjectInstance * a, const CGObjectInstance * b)
-{
-	if (!a)
-		return true;
-	if (!b)
-		return false;
-	if (a->appearance->printPriority != b->appearance->printPriority)
-		return a->appearance->printPriority > b->appearance->printPriority;
-	
-	if(a->pos.y != b->pos.y)
-		return a->pos.y < b->pos.y;
-	
-	if(b->ID == Obj::HERO && a->ID != Obj::HERO)
-		return true;
-	if(b->ID != Obj::HERO && a->ID == Obj::HERO)
-		return false;
-	
-	if(!a->isVisitable() && b->isVisitable())
-		return true;
-	if(!b->isVisitable() && a->isVisitable())
-		return false;
-	if(a->pos.x < b->pos.x)
-		return true;
-	return false;
-}
-
 ObjectRect::ObjectRect(const CGObjectInstance * obj_, QRect rect_)
-: obj(obj_),
-rect(rect_)
+	: obj(obj_)
+	, rect(rect_)
 {
 }
 
@@ -306,9 +289,9 @@ ObjectRect::~ObjectRect()
 
 std::shared_ptr<QImage> MapHandler::findFlagBitmap(const CGHeroInstance * hero, int anim, const PlayerColor color, int group) const
 {
-	if(!hero || hero->boat)
+	if(!hero || hero->inBoat())
 		return std::shared_ptr<QImage>();
-	
+
 	return findFlagBitmapInternal(graphics->heroFlagAnimations.at(color.getNum()), anim, group, hero->moveDir, true);
 }
 
@@ -317,7 +300,7 @@ std::shared_ptr<QImage> MapHandler::findFlagBitmapInternal(std::shared_ptr<Anima
 	size_t groupSize = animation->size(group);
 	if(groupSize == 0)
 		return nullptr;
-	
+
 	if(moving)
 		return animation->getImage(anim % groupSize, group);
 	else
@@ -334,15 +317,15 @@ MapHandler::BitmapHolder MapHandler::findObjectBitmap(const CGObjectInstance * o
 	size_t groupSize = animation->size(group);
 	if(groupSize == 0)
 		return MapHandler::BitmapHolder();
-	
+
 	animation->playerColored(obj->tempOwner);
 	auto bitmap = animation->getImage(anim % groupSize, group);
-	
+
 	if(!bitmap)
 		return MapHandler::BitmapHolder();
 
 	setPlayerColor(bitmap.get(), obj->tempOwner);
-	
+
 	return MapHandler::BitmapHolder(bitmap);
 }
 
@@ -356,49 +339,40 @@ std::vector<ObjectRect> & MapHandler::getObjects(int x, int y, int z)
 	return tileObjects[index(x, y, z)];
 }
 
-void MapHandler::drawObjects(QPainter & painter, int x, int y, int z, const std::set<const CGObjectInstance *> & locked)
+
+
+void MapHandler::drawObjects(QPainter & painter, const QRectF & section, int z, std::set<const CGObjectInstance *> & locked)
 {
 	painter.setRenderHint(QPainter::Antialiasing, false);
 	painter.setRenderHint(QPainter::SmoothPixmapTransform, false);
+	auto blitOrder = [](const CGObjectInstance * a, const CGObjectInstance * b) { return CMap::compareObjectBlitOrder(a, b);};
+	std::set<const CGObjectInstance *, decltype(blitOrder)> objects;
 
-	for(auto & object : getObjects(x, y, z))
+
+	int left = static_cast<int>(std::round(section.left()))/tileSize;
+	int right = static_cast<int>(std::round(section.right()))/tileSize;
+	int top = static_cast<int>(std::round(section.top()))/tileSize;
+	int bottom = static_cast<int>(std::round(section.bottom()))/tileSize;
+	for(int x = left; x < right; ++x)
 	{
-		const CGObjectInstance * obj = object.obj;
-		if(!obj)
+		for(int y = top; y < bottom; ++y)
 		{
-			logGlobal->error("Stray map object that isn't fading");
-			return;
-		}
-
-		uint8_t animationFrame = 0;
-
-		auto objData = findObjectBitmap(obj, animationFrame, obj->ID == Obj::HERO ? 2 : 0);
-		if(obj->ID == Obj::HERO && obj->tempOwner.isValidPlayer())
-			objData.flagBitmap = findFlagBitmap(dynamic_cast<const CGHeroInstance*>(obj), 0, obj->tempOwner, 4);
-		
-		if(objData.objBitmap)
-		{
-			auto pos = obj->anchorPos();
-
-			painter.drawImage(QPoint(x * tileSize, y * tileSize), *objData.objBitmap, object.rect, Qt::AutoColor | Qt::NoOpaqueDetection);
-
-			if(locked.count(obj))
+			for(auto & object : getObjects(x, y, z))
 			{
-				painter.setCompositionMode(QPainter::CompositionMode_DestinationIn);
-				painter.fillRect(x * tileSize, y * tileSize, object.rect.width(), object.rect.height(), Qt::Dense4Pattern);
-				painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
-			}
-
-			if(objData.flagBitmap)
-			{
-				if(x == pos.x && y == pos.y)
-					painter.drawImage(QPoint((x - 2) * tileSize, (y - 1) * tileSize), *objData.flagBitmap);
+				if (!objects.contains(object.obj))
+					objects.insert(object.obj);
 			}
 		}
 	}
+
+	for (auto const& object : objects)
+	{
+		int3 pos = object->pos;
+		drawObjectAt(painter, object, pos.x, pos.y, section.topLeft(), locked.count(object));
+	}
 }
 
-void MapHandler::drawObjectAt(QPainter & painter, const CGObjectInstance * obj, int x, int y)
+void MapHandler::drawObjectAt(QPainter & painter, const CGObjectInstance * obj, int x, int y, QPointF offset, bool locked)
 {
 	if (!obj)
 	{
@@ -411,13 +385,22 @@ void MapHandler::drawObjectAt(QPainter & painter, const CGObjectInstance * obj,
 	auto objData = findObjectBitmap(obj, animationFrame, obj->ID == Obj::HERO ? 2 : 0);
 	if(obj->ID == Obj::HERO && obj->tempOwner.isValidPlayer())
 		objData.flagBitmap = findFlagBitmap(dynamic_cast<const CGHeroInstance*>(obj), 0, obj->tempOwner, 4);
-	
+
 	if (objData.objBitmap)
 	{
-		painter.drawImage(QPoint((x + 1) * tileSize - objData.objBitmap->width(), (y + 1) * tileSize - objData.objBitmap->height()), *objData.objBitmap);
-		
+		QPoint point((x + 1) * tileSize - (objData.objBitmap->width() + offset.x()), (y + 1) * tileSize - (objData.objBitmap->height() + offset.y()));
+		QRect rect(point, QSize(objData.objBitmap->width(), objData.objBitmap->height()));
+		painter.drawImage(rect, *objData.objBitmap);
+
+		if (locked)
+		{
+			painter.setCompositionMode(QPainter::CompositionMode_DestinationIn);
+			painter.fillRect(rect, Qt::Dense4Pattern);
+			painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
+		}
+
 		if (objData.flagBitmap)
-			painter.drawImage(QPoint((x + 1) * tileSize - objData.objBitmap->width(), (y + 1) * tileSize - objData.objBitmap->height()), *objData.flagBitmap);
+			painter.drawImage(point, *objData.flagBitmap);
 	}
 }
 
@@ -428,7 +411,7 @@ QRgb MapHandler::getTileColor(int x, int y, int z)
 	{
 		if(!object.obj->getBlockedPos().count(int3(x, y, z)))
 			continue;
-		
+
 		PlayerColor player = object.obj->getOwner();
 		if(player == PlayerColor::NEUTRAL)
 			return graphics->neutralColor;
@@ -436,15 +419,15 @@ QRgb MapHandler::getTileColor(int x, int y, int z)
 			if (player.isValidPlayer())
 				return graphics->playerColors[player.getNum()];
 	}
-	
+
 	// else - use terrain color (blocked version or normal)
-	
+
 	auto & tile = map->getTile(int3(x, y, z));
-	
+
 	auto color = tile.getTerrain()->minimapUnblocked;
 	if (tile.blocked() && (!tile.visitable()))
 		color = tile.getTerrain()->minimapBlocked;
-	
+
 	return qRgb(color.r, color.g, color.b);
 }
 
@@ -459,10 +442,10 @@ std::set<int3> MapHandler::invalidate(const CGObjectInstance * obj)
 	auto t1 = removeObject(obj);
 	auto t2 = addObject(obj);
 	t1.insert(t2.begin(), t2.end());
-	
+
 	for(auto & tt : t2)
 		stable_sort(tileObjects[index(tt)].begin(), tileObjects[index(tt)].end(), objectBlitOrderSorter);
-	
+
 	return t1;
 }
 
