@@ -18,6 +18,8 @@ namespace Ui {
 class MapSettings;
 }
 
+class ModSettings;
+
 class MapSettings : public QDialog
 {
 	Q_OBJECT
@@ -26,6 +28,8 @@ public:
 	explicit MapSettings(MapController & controller, QWidget *parent = nullptr);
 	~MapSettings();
 
+	ModSettings * getModSettings();
+
 private slots:
 	void on_pushButton_clicked();
 
