@@ -14,14 +14,18 @@
 #include "../../lib/texts/Languages.h"
 #include "../../lib/texts/CGeneralTextHandler.h"
 #include "../../lib/mapObjects/CGObjectInstance.h"
-#include "../../lib/VCMI_Lib.h"
+#include "../../lib/GameLibrary.h"
+#include "../../lib/mapping/MapFormatJson.h"
 
 void Translations::cleanupRemovedItems(CMap & map)
 {
 	std::set<std::string> existingObjects{"map", "header"};
 	for(auto object : map.objects)
-		existingObjects.insert(object->instanceName);
-	
+	{
+		if(object)
+			existingObjects.insert(object->instanceName);
+	}
+
 	for(auto & translations : map.translations.Struct())
 	{
 		JsonNode updateTranslations;
@@ -64,6 +68,19 @@ Translations::Translations(CMapHeader & mh, QWidget *parent) :
 	
 	setWindowFlags(Qt::Dialog | Qt::WindowTitleHint | Qt::WindowCloseButtonHint);
 	
+	// Remove "map.<mapname>." prefix from all translation keys for editor display
+	// Internal VMAP translations are saved WITHOUT this prefix
+	for(auto & langPair : mapHeader.translations.Struct())
+	{
+		JsonNode cleanedTranslations;
+		for(auto & entry : langPair.second.Struct())
+		{
+			std::string key = CMapFormatJson::removeMapNamePrefix(entry.first);
+			cleanedTranslations.Struct()[key] = entry.second;
+		}
+		langPair.second = cleanedTranslations;
+	}
+	
 	//fill languages list
 	std::set<int> indexFoundLang;
 	int foundLang = -1;
@@ -74,7 +91,7 @@ Translations::Translations(CMapHeader & mh, QWidget *parent) :
 		ui->languageSelect->setItemData(ui->languageSelect->count() - 1, QVariant(QString::fromStdString(language.identifier)));
 		if(mapHeader.translations.Struct().count(language.identifier) && !mapHeader.translations[language.identifier].Struct().empty())
 			indexFoundLang.insert(ui->languageSelect->count() - 1);
-		if(language.identifier == VLC->generaltexth->getPreferredLanguage())
+		if(language.identifier == LIBRARY->generaltexth->getPreferredLanguage())
 			foundLang = ui->languageSelect->count() - 1;
 	}
 	ui->languageSelect->blockSignals(false);
@@ -89,7 +106,7 @@ Translations::Translations(CMapHeader & mh, QWidget *parent) :
 		ui->languageSelect->setCurrentIndex(foundLang);
 	
 	if(mapPreferredLanguage.empty())
-		mapPreferredLanguage = VLC->generaltexth->getPreferredLanguage();
+		mapPreferredLanguage = LIBRARY->generaltexth->getPreferredLanguage();
 }
 
 Translations::~Translations()
@@ -105,6 +122,7 @@ void Translations::fillTranslationsTable(const std::string & language)
 	ui->translationsTable->blockSignals(true);
 	ui->translationsTable->setRowCount(0);
 	ui->translationsTable->setRowCount(translation.Struct().size());
+	
 	int i = 0;
 	for(auto & s : translation.Struct())
 	{
