@@ -20,13 +20,12 @@
 
 #include "api/Registry.h"
 
+#include "../../lib/callback/IGameInfoCallback.h"
 #include "../../lib/json/JsonNode.h"
 #include "../../lib/filesystem/Filesystem.h"
 #include "../../lib/battle/IBattleInfoCallback.h"
-#include "../../lib/CGameInfoCallback.h"
 #include "../../lib/modding/ModScope.h"
 
-
 VCMI_LIB_NAMESPACE_BEGIN
 
 namespace scripting
