@@ -17,23 +17,30 @@
 #include "../TurnTimerHandler.h"
 
 #include "../../lib/CPlayerState.h"
+#include "../../lib/CSkillHandler.h"
 #include "../../lib/StartInfo.h"
+#include "../../lib/IGameSettings.h"
+#include "../../lib/constants/Enumerations.h"
+#include "../../lib/entities/artifact/CArtHandler.h"
 #include "../../lib/entities/building/CBuilding.h"
 #include "../../lib/entities/hero/CHeroHandler.h"
+#include "../../lib/entities/ResourceTypeHandler.h"
 #include "../../lib/gameState/CGameState.h"
 #include "../../lib/mapObjects/CGTownInstance.h"
 #include "../../lib/mapObjects/CGHeroInstance.h"
+#include "../../lib/mapObjects/MiscObjects.h"
 #include "../../lib/modding/IdentifierStorage.h"
 #include "../../lib/modding/ModScope.h"
 #include "../../lib/mapping/CMap.h"
 #include "../../lib/networkPacks/PacksForClient.h"
 #include "../../lib/networkPacks/StackLocation.h"
-#include "../../lib/serializer/Connection.h"
 #include "../../lib/spells/CSpellHandler.h"
+#include "../../lib/json/JsonUtils.h"
 #include "../lib/VCMIDirs.h"
 
 PlayerMessageProcessor::PlayerMessageProcessor(CGameHandler * gameHandler)
-	:gameHandler(gameHandler)
+	: gameHandler(gameHandler)
+	, cheatConfig(JsonUtils::assembleFromFiles("config/cheats.json"))
 {
 }
 
@@ -48,7 +55,7 @@ void PlayerMessageProcessor::playerMessage(PlayerColor player, const std::string
 
 	if(handleCheatCode(message, player, currObj))
 	{
-		if(!gameHandler->getPlayerSettings(player)->isControlledByAI())
+		if(!gameHandler->gameInfo().getPlayerSettings(player)->isControlledByAI())
 		{
 			MetaString txt;
 			txt.appendLocalString(EMetaText::GENERAL_TXT, 260);
@@ -66,17 +73,17 @@ void PlayerMessageProcessor::playerMessage(PlayerColor player, const std::string
 
 void PlayerMessageProcessor::commandExit(PlayerColor player, const std::vector<std::string> & words)
 {
-	bool isHost = gameHandler->gameLobby()->isPlayerHost(player);
+	bool isHost = gameHandler->gameServer().isPlayerHost(player);
 	if(!isHost)
 		return;
 
 	broadcastSystemMessage(MetaString::createFromTextID("vcmi.broadcast.gameTerminated"));
-	gameHandler->gameLobby()->setState(EServerState::SHUTDOWN);
+	gameHandler->gameServer().setState(EServerState::SHUTDOWN);
 }
 
 void PlayerMessageProcessor::commandKick(PlayerColor player, const std::vector<std::string> & words)
 {
-	bool isHost = gameHandler->gameLobby()->isPlayerHost(player);
+	bool isHost = gameHandler->gameServer().isPlayerHost(player);
 	if(!isHost)
 		return;
 
@@ -88,10 +95,10 @@ void PlayerMessageProcessor::commandKick(PlayerColor player, const std::vector<s
 			playerToKick = PlayerColor(std::stoi(playername));
 		else
 		{
-			for(auto & c : gameHandler->connections)
+			for (PlayerColor color : PlayerColor::ALL_PLAYERS())
 			{
-				if(c.first.toString() == playername)
-					playerToKick = c.first;
+				if(color.toString() == playername)
+					playerToKick = color;
 			}
 		}
 
@@ -108,13 +115,13 @@ void PlayerMessageProcessor::commandKick(PlayerColor player, const std::vector<s
 
 void PlayerMessageProcessor::commandSave(PlayerColor player, const std::vector<std::string> & words)
 {
-	bool isHost = gameHandler->gameLobby()->isPlayerHost(player);
+	bool isHost = gameHandler->gameServer().isPlayerHost(player);
 	if(!isHost)
 		return;
 
 	if(words.size() == 2)
 	{
-		gameHandler->save("Saves/" + words[1]);
+		gameHandler->save("Saves/" + words[1], PlayerColor::CANNOT_DETERMINE);
 		MetaString str;
 		str.appendTextID("vcmi.broadcast.gameSavedAs");
 		str.appendRawString(" ");
@@ -126,12 +133,12 @@ void PlayerMessageProcessor::commandSave(PlayerColor player, const std::vector<s
 void PlayerMessageProcessor::commandCheaters(PlayerColor player, const std::vector<std::string> & words)
 {
 	int playersCheated = 0;
-	for(const auto & player : gameHandler->gameState()->players)
+	for(const auto & playerState : gameHandler->gameState().players)
 	{
-		if(player.second.cheated)
+		if(playerState.second.cheated)
 		{
 			auto str = MetaString::createFromTextID("vcmi.broadcast.playerCheater");
-			str.replaceName(player.first);
+			str.replaceName(playerState.first);
 			broadcastSystemMessage(str);
 			playersCheated++;
 		}
@@ -143,11 +150,11 @@ void PlayerMessageProcessor::commandCheaters(PlayerColor player, const std::vect
 
 void PlayerMessageProcessor::commandStatistic(PlayerColor player, const std::vector<std::string> & words)
 {
-	bool isHost = gameHandler->gameLobby()->isPlayerHost(player);
+	bool isHost = gameHandler->gameServer().isPlayerHost(player);
 	if(!isHost)
 		return;
 
-	std::string path = gameHandler->gameState()->statistic.writeCsv();
+	std::string path = gameHandler->statistics->writeCsv();
 
 	auto str = MetaString::createFromTextID("vcmi.broadcast.statisticFile");
 	str.replaceRawString(path);
@@ -328,7 +335,7 @@ void PlayerMessageProcessor::startVoting(PlayerColor initiator, ECurrentChatVote
 
 	for(PlayerColor player(0); player < PlayerColor::PLAYER_LIMIT; ++player)
 	{
-		auto state = gameHandler->getPlayerState(player, false);
+		auto state = gameHandler->gameInfo().getPlayerState(player, false);
 		if(state && state->isHuman() && initiator != player)
 			awaitingPlayers.insert(player);
 	}
@@ -381,11 +388,14 @@ void PlayerMessageProcessor::cheatGiveSpells(PlayerColor player, const CGHeroIns
 		gameHandler->sendAndApply(giveBonus);
 	}
 
+	giveBonus.bonus = Bonus(BonusDuration::PERMANENT, BonusType::HERO_SPELL_CASTS_PER_COMBAT_TURN, BonusSource::OTHER, 99, BonusSourceID());
+	gameHandler->sendAndApply(giveBonus);
+
 	///Give mana
 	SetMana sm;
 	sm.hid = hero->id;
 	sm.val = 999;
-	sm.absolute = true;
+	sm.mode = ChangeValueMode::ABSOLUTE;
 	gameHandler->sendAndApply(sm);
 }
 
@@ -396,12 +406,19 @@ void PlayerMessageProcessor::cheatBuildTown(PlayerColor player, const CGTownInst
 
 	for (auto & build : town->getTown()->buildings)
 	{
-		if (!town->hasBuilt(build.first)
-			&& !build.second->getNameTranslated().empty()
-			&& build.first != BuildingID::SHIP)
-		{
-			gameHandler->buildStructure(town->id, build.first, true);
-		}
+		if (town->hasBuilt(build.first))
+			continue;
+
+		if (build.second->getNameTranslated().empty())
+			continue;
+
+		if (build.first == BuildingID::SHIP)
+			continue;
+
+		if (build.first == BuildingID::GRAIL && LIBRARY->engineSettings()->getBoolean(EGameSettings::CHEATS_BUILD_WITHOUT_GRAIL))
+			continue;
+		
+		gameHandler->buildStructure(town->id, build.first, true);
 	}
 }
 
@@ -421,7 +438,7 @@ void PlayerMessageProcessor::cheatGiveArmy(PlayerColor player, const CGHeroInsta
 	{
 	}
 
-	std::optional<int32_t> creatureId = VLC->identifiers()->getIdentifier(ModScope::scopeGame(), "creature", creatureIdentifier, false);
+	std::optional<int32_t> creatureId = LIBRARY->identifiers()->getIdentifierCaseInsensitive(ModScope::scopeGame(), "creature", creatureIdentifier, false);
 
 	if(creatureId.has_value())
 	{
@@ -432,9 +449,9 @@ void PlayerMessageProcessor::cheatGiveArmy(PlayerColor player, const CGHeroInsta
 			if (!hero->hasStackAtSlot(SlotID(i)))
 			{
 				if (amountPerSlot.has_value())
-					gameHandler->insertNewStack(StackLocation(hero, SlotID(i)), creature, *amountPerSlot);
+					gameHandler->insertNewStack(StackLocation(hero->id, SlotID(i)), creature, *amountPerSlot);
 				else
-					gameHandler->insertNewStack(StackLocation(hero, SlotID(i)), creature, 5 * std::pow(10, i));
+					gameHandler->insertNewStack(StackLocation(hero->id, SlotID(i)), creature, 5 * std::pow(10, i));
 			}
 		}
 	}
@@ -462,16 +479,16 @@ void PlayerMessageProcessor::cheatGiveArtifacts(PlayerColor player, const CGHero
 	{
 		for (auto const & word : words)
 		{
-			auto artID = VLC->identifiers()->getIdentifier(ModScope::scopeGame(), "artifact", word, false);
-			if(artID &&  VLC->arth->objects[*artID])
+			auto artID = LIBRARY->identifiers()->getIdentifierCaseInsensitive(ModScope::scopeGame(), "artifact", word, false);
+			if(artID &&  LIBRARY->arth->objects[*artID])
 				gameHandler->giveHeroNewArtifact(hero, ArtifactID(*artID), ArtifactPosition::FIRST_AVAILABLE);
 		}
 	}
 	else
 	{
-		for(int g = 7; g < VLC->arth->objects.size(); ++g) //including artifacts from mods
+		for(int g = 7; g < LIBRARY->arth->objects.size(); ++g) //including artifacts from mods
 		{
-			if(VLC->arth->objects[g]->canBePutAt(hero))
+			if(LIBRARY->arth->objects[g]->canBePutAt(hero))
 				gameHandler->giveHeroNewArtifact(hero, ArtifactID(g), ArtifactPosition::FIRST_AVAILABLE);
 		}
 	}
@@ -482,13 +499,22 @@ void PlayerMessageProcessor::cheatGiveScrolls(PlayerColor player, const CGHeroIn
 	if(!hero)
 		return;
 
-	for(const auto & spell : VLC->spellh->objects)
-		if(gameHandler->gameState()->isAllowed(spell->getId()) && !spell->isSpecial())
+	for(const auto & spell : LIBRARY->spellh->objects)
+		if(gameHandler->gameState().isAllowed(spell->getId()) && !spell->isSpecial())
 		{
 			gameHandler->giveHeroNewScroll(hero, spell->getId(), ArtifactPosition::FIRST_AVAILABLE);
 		}
 }
 
+void PlayerMessageProcessor::cheatColorSchemeChange(PlayerColor player, ColorScheme scheme)
+{
+	PlayerCheated pc;
+	pc.player = player;
+	pc.colorScheme = scheme;
+	pc.localOnlyCheat = true;
+	gameHandler->sendAndApply(pc);
+}
+
 void PlayerMessageProcessor::cheatLevelup(PlayerColor player, const CGHeroInstance * hero, std::vector<std::string> words)
 {
 	if (!hero)
@@ -504,7 +530,7 @@ void PlayerMessageProcessor::cheatLevelup(PlayerColor player, const CGHeroInstan
 		levelsToGain = 1;
 	}
 
-	gameHandler->giveExperience(hero, VLC->heroh->reqExp(hero->level + levelsToGain) - VLC->heroh->reqExp(hero->level));
+	gameHandler->giveExperience(hero, LIBRARY->heroh->reqExp(hero->level + levelsToGain) - LIBRARY->heroh->reqExp(hero->level));
 }
 
 void PlayerMessageProcessor::cheatExperience(PlayerColor player, const CGHeroInstance * hero, std::vector<std::string> words)
@@ -577,9 +603,9 @@ void PlayerMessageProcessor::cheatResources(PlayerColor player, std::vector<std:
 	}
 
 	TResources resources;
-	resources[EGameResID::GOLD] = baseResourceAmount * 1000;
-	for (GameResID i = EGameResID::WOOD; i < EGameResID::GOLD; ++i)
+	for (auto & i : LIBRARY->resourceTypeHandler->getAllObjects())
 		resources[i] = baseResourceAmount;
+	resources[EGameResID::GOLD] = baseResourceAmount * 1000;
 
 	gameHandler->giveResources(player, resources);
 }
@@ -605,15 +631,15 @@ void PlayerMessageProcessor::cheatMapReveal(PlayerColor player, bool reveal)
 	FoWChange fc;
 	fc.mode = reveal ? ETileVisibility::REVEALED : ETileVisibility::HIDDEN;
 	fc.player = player;
-	const auto & fowMap = gameHandler->gameState()->getPlayerTeam(player)->fogOfWarMap;
-	const auto & mapSize = gameHandler->gameState()->getMapSize();
+	const auto & fowMap = gameHandler->gameState().getPlayerTeam(player)->fogOfWarMap;
+	const auto & mapSize = gameHandler->gameState().getMapSize();
 	auto hlp_tab = new int3[mapSize.x * mapSize.y * mapSize.z];
 	int lastUnc = 0;
 
 	for(int z = 0; z < mapSize.z; z++)
 		for(int x = 0; x < mapSize.x; x++)
 			for(int y = 0; y < mapSize.y; y++)
-				if(!fowMap[z][x][y] || fc.mode == ETileVisibility::HIDDEN)
+				if(!fowMap[int3(x, y, z)] || fc.mode == ETileVisibility::HIDDEN)
 					hlp_tab[lastUnc++] = int3(x, y, z);
 
 	fc.tiles.insert(hlp_tab, hlp_tab + lastUnc);
@@ -625,11 +651,11 @@ void PlayerMessageProcessor::cheatMapReveal(PlayerColor player, bool reveal)
 
 void PlayerMessageProcessor::cheatPuzzleReveal(PlayerColor player)
 {
-	TeamState *t = gameHandler->gameState()->getPlayerTeam(player);
+	const TeamState * t = gameHandler->gameState().getPlayerTeam(player);
 
-	for(auto & obj : gameHandler->gameState()->map->objects)
+	for(auto & obj : gameHandler->gameState().getMap().getObjects<CGObelisk>())
 	{
-		if(obj && obj->ID == Obj::OBELISK && !obj->wasVisited(player))
+		if(!obj->wasVisited(player))
 		{
 			gameHandler->setObjPropertyID(obj->id, ObjProperty::OBELISK_VISITED, t->id);
 			for(const auto & color : t->players)
@@ -680,50 +706,124 @@ void PlayerMessageProcessor::cheatMaxMorale(PlayerColor player, const CGHeroInst
 	gameHandler->giveHeroBonus(&gb);
 }
 
+void PlayerMessageProcessor::cheatSkill(PlayerColor player, const CGHeroInstance * hero, std::vector<std::string> words)
+{
+	if (!hero)
+		return;
+
+	std::string identifier;
+	try
+	{
+		identifier = words.at(0);
+	}
+	catch(std::logic_error&)
+	{
+		return;
+	}
+
+	MasteryLevel::Type mastery;
+	try
+	{
+		mastery = static_cast<MasteryLevel::Type>(std::stoi(words.at(1)));
+	}
+	catch(std::logic_error&)
+	{
+		mastery = MasteryLevel::Type::EXPERT;
+	}
+
+	if(identifier == "every")
+	{
+		for(const auto & skill : LIBRARY->skillh->objects)
+			gameHandler->changeSecSkill(hero, SecondarySkill(skill->getId()), mastery, ChangeValueMode::ABSOLUTE);
+		return;
+	}
+
+	std::optional<int32_t> skillId = LIBRARY->identifiers()->getIdentifierCaseInsensitive(ModScope::scopeGame(), "skill", identifier, false);
+	if(!skillId.has_value())
+		return;
+	
+	auto skill = SecondarySkill(skillId.value());
+	gameHandler->changeSecSkill(hero, skill, mastery, ChangeValueMode::ABSOLUTE);
+}
+
+void PlayerMessageProcessor::cheatTeleport(PlayerColor player, const CGHeroInstance * hero, std::vector<std::string> words)
+{
+	if (!hero)
+		return;
+
+	int3 pos;
+	try
+	{
+		pos.x = std::stoi(words.at(0));
+		pos.y = std::stoi(words.at(1));
+		pos.z = words.size() > 2 ? std::stoi(words.at(2)) : hero->pos.z;
+	}
+	catch(std::logic_error&)
+	{
+		return;
+	}
+
+	if(!gameHandler->gameState().getMap().isInTheMap(pos))
+		return;
+
+	auto destTile = gameHandler->gameState().getMap().getTile(pos);
+	auto heroTile = gameHandler->gameState().getMap().getTile(hero->pos);
+	if(!destTile.isClear(&heroTile))
+		return;
+
+	gameHandler->moveHero(hero->id, pos, EMovementMode::DIMENSION_DOOR);
+}
+
+void PlayerMessageProcessor::cheatGiveGrail(PlayerColor player, const CGHeroInstance * hero)
+{
+	if (!hero)
+		return;
+
+	auto grail = ArtifactID(ArtifactID::GRAIL);
+	if(grail.toArtifact()->canBePutAt(hero))
+		gameHandler->giveHeroNewArtifact(hero, grail, ArtifactPosition::FIRST_AVAILABLE);
+}
+
 bool PlayerMessageProcessor::handleCheatCode(const std::string & cheat, PlayerColor player, ObjectInstanceID currObj)
 {
 	std::vector<std::string> words;
 	boost::split(words, boost::trim_copy(cheat), boost::is_any_of("\t\r\n "));
 
-	if (words.empty() || !gameHandler->getStartInfo()->extraOptionsInfo.cheatsAllowed)
+	if (words.empty() || !gameHandler->gameInfo().getStartInfo()->extraOptionsInfo.cheatsAllowed)
 		return false;
 
 	//Make cheat name case-insensitive, but keep words/parameters (e.g. creature name) as it
 	std::string cheatName = boost::to_lower_copy(words[0]);
 	words.erase(words.begin());
 
-	std::vector<std::string> townTargetedCheats = { "vcmiarmenelos", "vcmibuild", "nwczion" };
-	std::vector<std::string> playerTargetedCheats = {
-		"vcmiformenos",  "vcmiresources", "nwctheconstruct",
-		"vcmimelkor",    "vcmilose",      "nwcbluepill",
-		"vcmisilmaril",  "vcmiwin",       "nwcredpill",
-		"vcmieagles",    "vcmimap",       "nwcwhatisthematrix",
-		"vcmiungoliant", "vcmihidemap",   "nwcignoranceisbliss",
-		"vcmiobelisk",                    "nwcoracle"
-	};
-	std::vector<std::string> heroTargetedCheats = {
-		"vcmiainur",               "vcmiarchangel",   "nwctrinity",
-		"vcmiangband",             "vcmiblackknight", "nwcagents",
-		"vcmiglaurung",            "vcmicrystal",     "vcmiazure",
-		"vcmifaerie",              "vcmiarmy",        "vcminissi",
-		"vcmiistari",              "vcmispells",      "nwcthereisnospoon",
-		"vcminoldor",              "vcmimachines",    "nwclotsofguns",
-		"vcmiglorfindel",          "vcmilevel",       "nwcneo",
-		"vcminahar",               "vcmimove",        "nwcnebuchadnezzar",
-		"vcmiforgeofnoldorking",   "vcmiartifacts",
-		"vcmiolorin",              "vcmiexp",
-		"vcmiluck",                                   "nwcfollowthewhiterabbit", 
-		"vcmimorale",                                 "nwcmorpheus",
-		"vcmigod",                                    "nwctheone",
-		"vcmiscrolls"
+	auto getCheats = [this](const std::string& category)
+	{
+		std::vector<std::string> tmpCheats;
+		for (auto& item : cheatConfig[category].Struct())
+		{
+			const auto& vec = item.second.Vector();
+			std::transform(vec.begin(), vec.end(), std::back_inserter(tmpCheats), [](const auto& v){ return boost::to_lower_copy(v.String()); });
+		}
+		return tmpCheats;
 	};
 
+	auto localCheats = getCheats("localCheats");
+	auto townTargetedCheats = getCheats("townTargetedCheats");
+	auto playerTargetedCheats = getCheats("playerTargetedCheats");
+	auto heroTargetedCheats = getCheats("heroTargetedCheats");
+
+	if(vstd::contains(localCheats, cheatName))
+	{
+		executeCheatCode(cheatName, player, currObj, words);
+		return true;
+	}
+
 	if (!vstd::contains(townTargetedCheats, cheatName) && !vstd::contains(playerTargetedCheats, cheatName) && !vstd::contains(heroTargetedCheats, cheatName))
 		return false;
 
 	bool playerTargetedCheat = false;
 
-	for (const auto & i : gameHandler->gameState()->players)
+	for (const auto & i : gameHandler->gameState().players)
 	{
 		if (words.empty())
 			break;
@@ -770,10 +870,10 @@ bool PlayerMessageProcessor::handleCheatCode(const std::string & cheat, PlayerCo
 
 void PlayerMessageProcessor::executeCheatCode(const std::string & cheatName, PlayerColor player, ObjectInstanceID currObj, const std::vector<std::string> & words)
 {
-	const CGHeroInstance * hero = gameHandler->getHero(currObj);
-	const CGTownInstance * town = gameHandler->getTown(currObj);
+	const CGHeroInstance * hero = gameHandler->gameInfo().getHero(currObj);
+	const CGTownInstance * town = gameHandler->gameInfo().getTown(currObj);
 	if (!town && hero)
-		town = hero->visitedTown;
+		town = hero->getVisitedTown();
 
 	const auto & doCheatGiveSpells = [&]() { cheatGiveSpells(player, hero); };
 	const auto & doCheatBuildTown = [&]() { cheatBuildTown(player, town); };
@@ -802,92 +902,82 @@ void PlayerMessageProcessor::executeCheatCode(const std::string & cheatName, Pla
 		cheatMovement(player, hero, { });
 		cheatFly(player, hero);
 	};
+	const auto & doCheatColorSchemeChange = [&](ColorScheme filter) { cheatColorSchemeChange(player, filter); };
+	const auto & doCheatSkill = [&]() { cheatSkill(player, hero, words); };
+	const auto & doCheatTeleport = [&]() { cheatTeleport(player, hero, words); };
+	const auto & doCheatGiveGrail = [&]() { cheatGiveGrail(player, hero); };
 
-	// Unimplemented H3 cheats:
-	// nwcphisherprice - Changes and brightens the game colors.
+	auto getCheatKey = [this](const std::string& cheat) {
+		for (const auto& [category, structNode] : cheatConfig.Struct())
+		{
+			for (const auto& [key, node] : structNode.Struct())
+			{
+				for (const auto& v : node.Vector())
+				{
+					if (boost::to_lower_copy(v.String()) == cheat)
+						return key;
+				}
+			}
+		}
+		return std::string("");
+	};
+	auto key = getCheatKey(cheatName);
 
 	std::map<std::string, std::function<void()>> callbacks = {
-		{"vcmiainur",              [&] () {doCheatGiveArmyFixed({ "archangel", "5" });} },
-		{"nwctrinity",             [&] () {doCheatGiveArmyFixed({ "archangel", "5" });} },
-		{"vcmiangband",            [&] () {doCheatGiveArmyFixed({ "blackKnight", "10" });} },
-		{"vcmiglaurung",           [&] () {doCheatGiveArmyFixed({ "crystalDragon", "5000" });} },
-		{"vcmiarchangel",          [&] () {doCheatGiveArmyFixed({ "archangel", "5" });} },
-		{"nwcagents",              [&] () {doCheatGiveArmyFixed({ "blackKnight", "10" });} },
-		{"vcmiblackknight",        [&] () {doCheatGiveArmyFixed({ "blackKnight", "10" });} },
-		{"vcmicrystal",            [&] () {doCheatGiveArmyFixed({ "crystalDragon", "5000" });} },
-		{"vcmiazure",              [&] () {doCheatGiveArmyFixed({ "azureDragon", "5000" });} },
-		{"vcmifaerie",             [&] () {doCheatGiveArmyFixed({ "fairieDragon", "5000" });} },
-		{"vcmiarmy",                doCheatGiveArmyCustom },
-		{"vcminissi",               doCheatGiveArmyCustom },
-		{"vcmiistari",              doCheatGiveSpells     },
-		{"vcmispells",              doCheatGiveSpells     },
-		{"nwcthereisnospoon",       doCheatGiveSpells     },
-		{"vcmiarmenelos",           doCheatBuildTown      },
-		{"vcmibuild",               doCheatBuildTown      },
-		{"nwczion",                 doCheatBuildTown      },
-		{"vcminoldor",              doCheatGiveMachines   },
-		{"vcmimachines",            doCheatGiveMachines   },
-		{"nwclotsofguns",           doCheatGiveMachines   },
-		{"vcmiforgeofnoldorking",   doCheatGiveArtifacts  },
-		{"vcmiartifacts",           doCheatGiveArtifacts  },
-		{"vcmiglorfindel",          doCheatLevelup        },
-		{"vcmilevel",               doCheatLevelup        },
-		{"nwcneo",                  doCheatLevelup        },
-		{"vcmiolorin",              doCheatExperience     },
-		{"vcmiexp",                 doCheatExperience     },
-		{"vcminahar",               doCheatMovement       },
-		{"vcmimove",                doCheatMovement       },
-		{"nwcnebuchadnezzar",       doCheatMovement       },
-		{"vcmiformenos",            doCheatResources      },
-		{"vcmiresources",           doCheatResources      },
-		{"nwctheconstruct",         doCheatResources      },
-		{"nwcbluepill",             doCheatDefeat         },
-		{"vcmimelkor",              doCheatDefeat         },
-		{"vcmilose",                doCheatDefeat         },
-		{"nwcredpill",              doCheatVictory        },
-		{"vcmisilmaril",            doCheatVictory        },
-		{"vcmiwin",                 doCheatVictory        },
-		{"nwcwhatisthematrix",      doCheatMapReveal      },
-		{"vcmieagles",              doCheatMapReveal      },
-		{"vcmimap",                 doCheatMapReveal      },
-		{"vcmiungoliant",           doCheatMapHide        },
-		{"vcmihidemap",             doCheatMapHide        },
-		{"nwcignoranceisbliss",     doCheatMapHide        },
-		{"vcmiobelisk",             doCheatRevealPuzzle   },
-		{"nwcoracle",               doCheatRevealPuzzle   },
-		{"vcmiluck",                doCheatMaxLuck        },
-		{"nwcfollowthewhiterabbit", doCheatMaxLuck        },
-		{"vcmimorale",              doCheatMaxMorale      },
-		{"nwcmorpheus",             doCheatMaxMorale      },
-		{"vcmigod",                 doCheatTheOne         },
-		{"nwctheone",               doCheatTheOne         },
-		{"vcmiscrolls",             doCheatGiveScrolls    },
+		{"giveArchangel",      [&] () {doCheatGiveArmyFixed({ "archangel",        "5" });} },
+		{"giveBlackKnight",    [&] () {doCheatGiveArmyFixed({ "blackKnight",     "10" });} },
+		{"giveCrystalDragon",  [&] () {doCheatGiveArmyFixed({ "crystalDragon", "5000" });} },
+		{"giveAzureDragon",    [&] () {doCheatGiveArmyFixed({ "azureDragon",   "5000" });} },
+		{"giveFairieDragon",   [&] () {doCheatGiveArmyFixed({ "fairieDragon",  "5000" });} },
+		{"giveArmy",           doCheatGiveArmyCustom                                       },
+		{"giveSpells",         doCheatGiveSpells                                           },
+		{"buildTown",          doCheatBuildTown                                            },
+		{"giveMachines",       doCheatGiveMachines                                         },
+		{"giveArtifacts",      doCheatGiveArtifacts                                        },
+		{"levelup",            doCheatLevelup                                              },
+		{"experience",         doCheatExperience                                           },
+		{"movement",           doCheatMovement                                             },
+		{"resources",          doCheatResources                                            },
+		{"defeat",             doCheatDefeat                                               },
+		{"victory",            doCheatVictory                                              },
+		{"mapReveal",          doCheatMapReveal                                            },
+		{"mapHide",            doCheatMapHide                                              },
+		{"revealPuzzle",       doCheatRevealPuzzle                                         },
+		{"maxLuck",            doCheatMaxLuck                                              },
+		{"maxMorale",          doCheatMaxMorale                                            },
+		{"god",                doCheatTheOne                                               },
+		{"giveScrolls",        doCheatGiveScrolls                                          },
+		{"color",              [&] () {doCheatColorSchemeChange(ColorScheme::H2_SCHEME);}  },
+		{"gray",               [&] () {doCheatColorSchemeChange(ColorScheme::GRAYSCALE);}  },
+		{"skill",              doCheatSkill                                                },
+		{"teleport",           doCheatTeleport                                             },
+		{"grail",              doCheatGiveGrail                                            },
 	};
 
-	assert(callbacks.count(cheatName));
-	if (callbacks.count(cheatName))
-		callbacks.at(cheatName)();
+	assert(callbacks.count(key));
+	if (callbacks.count(key))
+		callbacks.at(key)();
 }
 
-void PlayerMessageProcessor::sendSystemMessage(std::shared_ptr<CConnection> connection, const MetaString & message)
+void PlayerMessageProcessor::sendSystemMessage(GameConnectionID connectionID, const MetaString & message)
 {
 	SystemMessage sm;
 	sm.text = message;
-	connection->sendPack(sm);
+	gameHandler->gameServer().sendPack(sm, connectionID);
 }
 
-void PlayerMessageProcessor::sendSystemMessage(std::shared_ptr<CConnection> connection, const std::string & message)
+void PlayerMessageProcessor::sendSystemMessage(GameConnectionID connectionID, const std::string & message)
 {
 	MetaString str;
 	str.appendRawString(message);
-	sendSystemMessage(connection, str);
+	sendSystemMessage(connectionID, str);
 }
 
 void PlayerMessageProcessor::broadcastSystemMessage(MetaString message)
 {
 	SystemMessage sm;
 	sm.text = message;
-	gameHandler->sendToAllClients(sm);
+	gameHandler->gameServer().applyPack(sm);
 }
 
 void PlayerMessageProcessor::broadcastSystemMessage(const std::string & message)
