@@ -9,7 +9,7 @@
  */
 #include "StdInc.h"
 
-#include "../../lib/CArtHandler.h"
+#include "../../lib/entities/artifact/CArtifact.h"
 
 namespace test
 {
@@ -40,7 +40,6 @@ TEST_F(CArtifactTest, RegistersIcons)
 	};
 
 	EXPECT_CALL(*this, registarCb(Eq(4242), Eq(0), "ARTIFACT", "Test1"));
-	EXPECT_CALL(*this, registarCb(Eq(4242), Eq(0), "ARTIFACTLARGE", "Test2"));
 
 	subject->registerIcons(cb);
 }
