@@ -10,6 +10,7 @@
 #include "StdInc.h"
 
 #include "TargetConditionItemFixture.h"
+#include "../../../lib/bonuses/BonusParameters.h"
 
 namespace test
 {
@@ -24,9 +25,10 @@ public:
 
 	void setDefaultExpectations()
 	{
-		EXPECT_CALL(unitMock, getAllBonuses(_, _, _)).Times(AtLeast(1));
+		EXPECT_CALL(unitMock, getAllBonuses(_, _)).Times(AtLeast(1));
 		EXPECT_CALL(unitMock, getTreeVersion()).Times(AtLeast(0));
 		EXPECT_CALL(mechanicsMock, getSpellIndex()).WillRepeatedly(Return(castSpell));
+		EXPECT_CALL(mechanicsMock, getSpellId()).WillRepeatedly(Return(SpellID(castSpell)));
 	}
 
 	void SetUp() override
@@ -40,11 +42,11 @@ public:
 	}
 };
 
-TEST_P(AbsoluteSpellConditionTest, DISABLED_ChecksAbsoluteCase)
+TEST_P(AbsoluteSpellConditionTest, ChecksAbsoluteCase)
 {
 	setDefaultExpectations();
 	auto bonus = std::make_shared<Bonus>(BonusDuration::ONE_BATTLE, BonusType::SPELL_IMMUNITY, BonusSource::OTHER, 4, BonusSourceID(), BonusSubtypeID(SpellID(immuneSpell)));
-	bonus->additionalInfo = 1;
+	bonus->parameters = std::make_shared<BonusParameters>(1);
 
 	unitBonuses.addNewBonus(bonus);
 
@@ -54,7 +56,7 @@ TEST_P(AbsoluteSpellConditionTest, DISABLED_ChecksAbsoluteCase)
 		EXPECT_TRUE(subject->isReceptive(&mechanicsMock, &unitMock));
 }
 
-TEST_P(AbsoluteSpellConditionTest, DISABLED_IgnoresNormalCase)
+TEST_P(AbsoluteSpellConditionTest, IgnoresNormalCase)
 {
 	setDefaultExpectations();
 	auto bonus = std::make_shared<Bonus>(BonusDuration::ONE_BATTLE, BonusType::SPELL_IMMUNITY, BonusSource::OTHER, 4, BonusSourceID(), BonusSubtypeID(SpellID(immuneSpell)));
