@@ -34,4 +34,9 @@ QString mapsPath()
 {
 	return pathToQString(VCMIDirs::get().userDataPath() / "Maps");
 }
+
+QSettings getSettings(QString appName)
+{
+    return QSettings(pathToQString(VCMIDirs::get().userConfigPath() / (appName.toStdString() + ".ini")), QSettings::IniFormat);
+}
 }
